@@ -32,7 +32,9 @@ from tensorflow_datasets.audio.savee import Savee
 from tensorflow_datasets.audio.speech_commands import SpeechCommands
 from tensorflow_datasets.audio.spoken_digit import SpokenDigit
 from tensorflow_datasets.audio.tedlium import Tedlium
+from tensorflow_datasets.audio.userlibri_audio_data import UserLibriAudio
 from tensorflow_datasets.audio.vctk import Vctk
 from tensorflow_datasets.audio.voxceleb import Voxceleb
 from tensorflow_datasets.audio.voxforge import Voxforge
+from tensorflow_datasets.audio.xtreme_s import XtremeS
 from tensorflow_datasets.audio.yesno import YesNo
