@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,11 +15,14 @@
 
 """DementiaBank dataset."""
 
+from __future__ import annotations
+
 import collections
 import os
 import textwrap
+
 import numpy as np
-import tensorflow.compat.v2 as tf
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 _CITATION = """
@@ -132,7 +135,7 @@ class Dementiabank(tfds.core.GeneratorBasedBuilder):
         features=tfds.features.FeaturesDict({
             'audio': tfds.features.Audio(file_format='mp3', sample_rate=44100),
             'label': tfds.features.ClassLabel(names=['dementia', 'control']),
-            'speaker_id': tf.string,
+            'speaker_id': np.str_,
         }),
         supervised_keys=('audio', 'label'),
         homepage='https://dementia.talkbank.org/',
