@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,9 +15,14 @@
 
 """FUSS dataset."""
 
+from __future__ import annotations
+
 import os
+
 from absl import logging
-import tensorflow.compat.v2 as tf
+from etils import epath
+import numpy as np
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 _CITATION = r"""\
@@ -25,13 +30,14 @@ _CITATION = r"""\
   title = {What's All the {FUSS} About Free Universal Sound Separation Data?},
   author = {Scott Wisdom and Hakan Erdogan and Daniel P. W. Ellis and Romain Serizel and Nicolas Turpault and Eduardo Fonseca and Justin Salamon and Prem Seetharaman and John R. Hershey},
   year = {2020},
+  url = {https://arxiv.org/abs/2011.00803},
 }
 
 @inproceedings{fonseca2020fsd50k,
   author = {Eduardo Fonseca and Xavier Favory and Jordi Pons and Frederic Font Corbera and Xavier Serra},
   title = {{FSD}50k: an open dataset of human-labeled sound events},
   year = {2020},
-}
+  url = {https://arxiv.org/abs/2010.00475},
 }
 """
 
@@ -96,7 +102,7 @@ class Fuss(tfds.core.GeneratorBasedBuilder):
                     file_format="wav",
                     shape=(160000,),
                     sample_rate=16000,
-                    dtype=tf.int16),
+                    dtype=np.int16),
             "sources":
                 tfds.features.Sequence({
                     "audio":
@@ -104,20 +110,20 @@ class Fuss(tfds.core.GeneratorBasedBuilder):
                             file_format="wav",
                             shape=(160000,),
                             sample_rate=16000,
-                            dtype=tf.int16),
+                            dtype=np.int16),
                     "label":
                         tfds.features.ClassLabel(names=source_labels),
                 }),
             "segments":
                 tfds.features.Sequence({
-                    "start_time_seconds": tf.float32,
-                    "end_time_seconds": tf.float32,
-                    "label": tf.string
+                    "start_time_seconds": np.float32,
+                    "end_time_seconds": np.float32,
+                    "label": np.str_
                 }),
             "jams":
-                tf.string,
+                np.str_,
             "id":
-                tf.string,
+                np.str_,
         }),
         supervised_keys=("mixture_audio", "sources"),
         homepage=_URL,
@@ -146,7 +152,7 @@ class Fuss(tfds.core.GeneratorBasedBuilder):
       # Some segments files are missing in the "unprocessed" set.
       logging.info("Missing segments file: %s", path)
       return segments
-    with tf.io.gfile.GFile(path) as f:
+    with epath.Path(path).open() as f:
       for l in f:
         try:
           start, end, label = l.split()
@@ -163,7 +169,7 @@ class Fuss(tfds.core.GeneratorBasedBuilder):
     """Generates examples for the given split."""
     path = os.path.join(base_dir, "%s_example_list.txt" % split)
     split_dir = os.path.join(base_dir, split)
-    with tf.io.gfile.GFile(path) as example_list:
+    with epath.Path(path).open() as example_list:
       for line in example_list:
         paths = line.split()
         key = _basename_without_ext(paths[0])
