@@ -21,11 +21,13 @@ import datetime
 import hashlib
 import json
 import tempfile
-from typing import Any, List, Optional, Type
+from typing import Any, Iterator, List, Optional, Type
 
 from absl import logging
 
+from etils import epath
 from tensorflow_datasets.core import dataset_builder
+from tensorflow_datasets.core import naming
 from tensorflow_datasets.core import registered
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.community import cache
@@ -50,7 +52,7 @@ class DatasetPackage:
     name: Dataset name
     source: Source that contains the source code (e.g. `github://...`)
   """
-  name: utils.DatasetName
+  name: naming.DatasetName
   source: dataset_sources_lib.DatasetSource
   # Ideally, we should also save the version so `tfds.load('ns:ds/1.0.0')`
   # fetch a specific version (e.g. at an older commit).
@@ -59,7 +61,7 @@ class DatasetPackage:
   def from_json(cls, data: utils.Json) -> 'DatasetPackage':
     """Factory which creates the cls from json."""
     return cls(
-        name=utils.DatasetName(namespace_name=data['name']),
+        name=naming.DatasetName(namespace_name=data['name']),
         source=dataset_sources_lib.DatasetSource.from_json(data['source']),
     )
 
@@ -94,7 +96,7 @@ class _InstalledPackage:
     return f'{_IMPORT_MODULE_NAME}.{name.namespace}.{name.name}.{self.hash}.{name.name}'
 
   @property
-  def installation_path(self) -> utils.ReadWritePath:
+  def installation_path(self) -> epath.Path:
     """Local path of the package."""
     name = self.package.name
     sub_dir = f'{_IMPORT_MODULE_NAME}/{name.namespace}/{name.name}/{self.hash}'
@@ -105,9 +107,8 @@ class _InstalledPackage:
     """Factory which creates the cls from json."""
     return cls(
         package=DatasetPackage.from_json(data['package']),
-        # TODO(py3.7): Should use `datetime.fromisoformat`
-        instalation_date=datetime.datetime.strptime(data['instalation_date'],
-                                                    '%Y-%m-%dT%H:%M:%S.%f'),
+        instalation_date=datetime.datetime.fromisoformat(
+            data['instalation_date']),
         hash=data['hash'],
     )
 
@@ -120,7 +121,7 @@ class _InstalledPackage:
     }
 
 
-# TODO(py3.9): Should be `UserDict[utils.DatasetName, _DatasetPackage]`
+# TODO(tfds): py3.9 Should be `UserDict[naming.DatasetName, _DatasetPackage]`
 class _PackageIndex(collections.UserDict):
   """Package index.
 
@@ -137,10 +138,9 @@ class _PackageIndex(collections.UserDict):
   {"name": "tensorflow_graphics:shapenet", "source": "github://..."}
   [...]
   ```
-
   """
 
-  def __init__(self, path: utils.PathLike):
+  def __init__(self, path: epath.PathLike):
     """Contructor.
 
     Args:
@@ -148,8 +148,8 @@ class _PackageIndex(collections.UserDict):
         dataset packages)
     """
     super().__init__()
-    self._remote_path: utils.ReadOnlyPath = utils.as_path(path)
-    self._cached_path: utils.ReadOnlyPath = (
+    self._remote_path: epath.Path = epath.Path(path)
+    self._cached_path: epath.Path = (
         cache.cache_path() / 'community-datasets-list.jsonl')
 
     # Pre-load the index from the cache
@@ -174,7 +174,7 @@ class _PackageIndex(collections.UserDict):
 
     try:
       content = self._remote_path.read_text()
-    except gcs_utils.GCS_UNAVAILABLE_EXCEPTIONS as e:
+    except gcs_utils.gcs_unavailable_exceptions() as e:
       # Do not crash if GCS access not available, but instead silently reuse
       # the cache.
       logging.info('Could not refresh the package index (GCS unavailable): %s',
@@ -206,17 +206,16 @@ class PackageRegister(register_base.BaseRegister):
   # Load a specific dataset
   builder = register.builder('tensorflow_graphics:shapenet')
   ```
-
   """
 
-  def __init__(self, path: utils.PathLike):
+  def __init__(self, path: epath.PathLike):
     """Contructor.
 
     Args:
       path: Path to the register files containing the list of dataset sources,
         forwarded to `_PackageIndex`
     """
-    self._path = utils.as_path(path)
+    self._path = path
 
   @utils.memoized_property
   def _package_index(self) -> _PackageIndex:
@@ -231,9 +230,13 @@ class PackageRegister(register_base.BaseRegister):
       self._package_index.refresh()  # Try updating the index
     return sorted(str(name) for name in self._package_index)  # pylint: disable=not-an-iterable
 
+  def list_dataset_references(self) -> Iterator[naming.DatasetReference]:
+    for name in self.list_builders():
+      yield naming.DatasetReference.from_tfds_name(tfds_name=name)
+
   def builder_cls(
       self,
-      name: utils.DatasetName,
+      name: naming.DatasetName,
   ) -> Type[dataset_builder.DatasetBuilder]:
     """Returns the builder class."""
     # Download the dataset generation code, or reuse the cache
@@ -248,15 +251,102 @@ class PackageRegister(register_base.BaseRegister):
 
   def builder(
       self,
-      name: utils.DatasetName,
+      name: naming.DatasetName,
       **builder_kwargs: Any,
   ) -> dataset_builder.DatasetBuilder:
     """Returns the dataset builder."""
     return self.builder_cls(name)(**builder_kwargs)  # pytype: disable=not-instantiable
 
 
+def list_ds_packages_for_namespace(
+    namespace: str,
+    path: epath.Path,
+) -> List[DatasetPackage]:
+  """Returns the dataset names found in a specific directory.
+
+  Directories that contain code should have the following structure:
+
+  ```
+  <path>/
+      <dataset0>/
+          <dataset0>.py
+      <dataset1>/
+          <dataset1>.py
+      ...
+  ```
+
+  Additional files or folders which are not detected as datasets will be
+  ignored (e.g. `__init__.py`).
+
+  Args:
+    namespace: Namespace of the datasets
+    path: The directory path containing the datasets.
+
+  Returns:
+    ds_packages: The dataset packages found in the directory (sorted for
+      determinism).
+
+  Raises:
+    FileNotFoundError: If the path cannot be reached.
+  """
+  if not path.exists():
+    # Should be fault-tolerant in the future
+    raise FileNotFoundError(f'Could not find datasets at {path}')
+
+  all_packages = []
+  for ds_path in path.iterdir():
+    source = get_dataset_source(ds_path)
+    if source:
+      pkg = DatasetPackage(
+          name=naming.DatasetName(namespace=namespace, name=ds_path.name),
+          source=source,
+      )
+      all_packages.append(pkg)
+
+  return all_packages
+
+
+def get_dataset_source(
+    ds_path: epath.Path,) -> Optional[dataset_sources_lib.DatasetSource]:
+  """Returns a `DatasetSource` instance if the given path corresponds to a dataset.
+
+  To determine whether the given path contains a dataset, a simple heuristic is
+  used that checks whether the path has the following structure:
+
+  ```
+  <ds_name>/
+      <ds_name>.py
+  ```
+
+  If so, all `.py`, `.txt`, `.tsv`, `.json` files will be added to the package.
+
+  Args:
+    ds_path: Path of the dataset module
+
+  Returns:
+    A `DatasetSource` instance if the path matches the expected file structure.
+  """
+  filter_list = {'__init__.py'}
+  suffixes_list = ('.txt', '.tsv', '.py', '.json')
+
+  def is_interesting_file(fname: str) -> bool:
+    return fname.endswith(suffixes_list) and fname not in filter_list
+
+  if not ds_path.is_dir():
+    return None
+  all_filenames = set(f.name for f in ds_path.iterdir())
+  if f'{ds_path.name}.py' not in all_filenames:
+    return None
+
+  return dataset_sources_lib.DatasetSource(
+      root_path=ds_path,
+      filenames=sorted(
+          [fname for fname in all_filenames if is_interesting_file(fname)]),
+  )
+
+
 def _download_or_reuse_cache(
-    name: utils.DatasetName,
+    name: naming.DatasetName,
     package_index: _PackageIndex,
 ) -> _InstalledPackage:
   """Downloads the dataset generation source code.
@@ -306,7 +396,7 @@ def _download_or_reuse_cache(
 
 
 def _get_last_installed_version(
-    name: utils.DatasetName,) -> Optional[_InstalledPackage]:
+    name: naming.DatasetName,) -> Optional[_InstalledPackage]:
   """Checks whether the datasets is installed locally and returns it."""
   root_dir = (
       cache.module_path() / _IMPORT_MODULE_NAME / name.namespace / name.name)
@@ -342,7 +432,7 @@ def _download_and_cache(package: DatasetPackage) -> _InstalledPackage:
   Returns:
     installed_dataset: The installed dataset package.
   """
-  tmp_dir = utils.as_path(tempfile.mkdtemp())
+  tmp_dir = epath.Path(tempfile.mkdtemp())
   try:
     # Download the package in a tmp directory
     dataset_sources_lib.download_from_source(
@@ -378,7 +468,7 @@ def _download_and_cache(package: DatasetPackage) -> _InstalledPackage:
   return installed_package
 
 
-def _compute_dir_hash(path: utils.ReadOnlyPath) -> str:
+def _compute_dir_hash(path: epath.Path) -> str:
   """Computes the checksums of the given directory deterministically."""
   all_files = sorted(path.iterdir())
 
@@ -389,7 +479,3 @@ def _compute_dir_hash(path: utils.ReadOnlyPath) -> str:
   all_checksums = [f.name for f in all_files]
   all_checksums += [checksums.compute_url_info(f).checksum for f in all_files]
   return hashlib.sha256(''.join(all_checksums).encode()).hexdigest()
-
-
-# Register pointing to the GCS community list.
-community_register = PackageRegister(path=gcs_utils.GCS_COMMUNITY_INDEX_PATH)
