@@ -26,9 +26,11 @@ import textwrap
 from typing import Iterator
 from unittest import mock
 
+from etils import epath
 import pytest
 
 from tensorflow_datasets.core import dataset_builder
+from tensorflow_datasets.core import naming
 from tensorflow_datasets.core import registered
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.community import cache
@@ -37,9 +39,9 @@ from tensorflow_datasets.core.community import register_package
 
 
 @contextlib.contextmanager
-def mock_cache_path(new_cache_dir: utils.PathLike) -> Iterator[None]:
+def mock_cache_path(new_cache_dir: epath.PathLike) -> Iterator[None]:
   """Mock which overwrite the cache path."""
-  new_dir = utils.as_path(new_cache_dir)
+  new_dir = epath.Path(new_cache_dir)
 
   # Use `__wrapped__` to access the original function wrapped inside
   # `functools.lru_cache`
@@ -84,6 +86,14 @@ def dummy_register():
       yield register_package.PackageRegister(path=dummy_path)
 
 
+def test_list_dataset_references(dummy_register):  # pylint: disable=redefined-outer-name
+  assert sorted(dummy_register.list_dataset_references()) == [
+      naming.DatasetReference(dataset_name='ds1', namespace='kaggle'),
+      naming.DatasetReference(dataset_name='dummy_dataset', namespace='kaggle'),
+      naming.DatasetReference(dataset_name='dummy_dataset', namespace='mlds'),
+  ]
+
+
 def test_builder_cls(dummy_register):  # pylint: disable=redefined-outer-name
 
   # The dataset will be installed in the cache
@@ -91,11 +101,11 @@ def test_builder_cls(dummy_register):  # pylint: disable=redefined-outer-name
   installed_path /= 'modules/tfds_community/kaggle/dummy_dataset'
   assert not installed_path.exists()
 
-  ds_name = utils.DatasetName('kaggle:dummy_dataset')
+  ds_name = naming.DatasetName('kaggle:dummy_dataset')
   builder_cls = dummy_register.builder_cls(ds_name)
   assert builder_cls.name == 'dummy_dataset'
 
-  clshash = 'e58f413affd65c267bae7acbd27fd5ac673d3e3ae13c316ffc2a461d00c8ab56'
+  clshash = '7e99279671c2fcaad6c99d5fcae88619befeb854196f0d858b24246bf8b06ac9'
   assert installed_path / f'{clshash}/dummy_dataset.py' == builder_cls.code_path
   assert 'kaggle' in builder_cls.code_path.parts
   assert issubclass(builder_cls, dataset_builder.DatasetBuilder)
@@ -110,12 +120,12 @@ def test_builder_cls(dummy_register):  # pylint: disable=redefined-outer-name
       register_package,
       '_download_and_cache',
       side_effect=ValueError('Dataset should have been cached already')):
-    ds_name = utils.DatasetName('kaggle:dummy_dataset')
+    ds_name = naming.DatasetName('kaggle:dummy_dataset')
     builder_cls2 = dummy_register.builder_cls(ds_name)
   assert builder_cls is builder_cls2
 
   # Datasets from different namespace can have the same name
-  ds_name = utils.DatasetName('mlds:dummy_dataset')
+  ds_name = naming.DatasetName('mlds:dummy_dataset')
   builder_cls = dummy_register.builder_cls(ds_name)
   assert 'mlds' in builder_cls.code_path.parts
   assert issubclass(builder_cls, dataset_builder.DatasetBuilder)
@@ -123,7 +133,7 @@ def test_builder_cls(dummy_register):  # pylint: disable=redefined-outer-name
   assert 'http://dummy.org/data.txt' in builder_cls.url_infos
 
   with pytest.raises(registered.DatasetNotFoundError):
-    dummy_register.builder(utils.DatasetName('other:ds0'))
+    dummy_register.builder(naming.DatasetName('other:ds0'))
 
 
 def test_register_path_list_builders(dummy_register):  # pylint: disable=redefined-outer-name
@@ -137,7 +147,7 @@ def test_register_path_list_builders(dummy_register):  # pylint: disable=redefin
 def test_dataset_package():
   """Exports/imports operation should be identity."""
   pkg = register_package.DatasetPackage(
-      name=utils.DatasetName('ns:ds'),
+      name=naming.DatasetName('ns:ds'),
       source=dataset_sources.DatasetSource.from_json(
           'github://<owner>/<name>/tree/<branch>/my_ds/ds.py',),
   )
