@@ -19,8 +19,9 @@ import dataclasses
 import difflib
 import functools
 import os
-from typing import Any, List, Mapping, Type
+from typing import Any, Iterable, List, Mapping, Type
 
+from absl import logging
 from etils import epath
 from tensorflow_datasets.core import dataset_builder
 # Make sure that github paths are registered
@@ -140,6 +141,16 @@ class DatasetRegistry(register_base.BaseRegister):
         builders.extend(register.list_builders())
     return builders
 
+  def list_dataset_references(self) -> Iterable[naming.DatasetReference]:
+    for registers in self.registers_per_namespace.values():
+      for register in registers:
+        try:
+          yield from register.list_dataset_references()
+        except Exception:  # pylint: disable=broad-except
+          logging.exception(
+              'Exception while getting dataset references from register %s',
+              register)
+
   def list_builders_per_namespace(self, namespace: str) -> List[str]:
     """Lists the builders available for a specific namespace."""
     builders = []
