@@ -30,8 +30,9 @@ from typing import Any, ClassVar, Dict, Iterable, List, Optional, Tuple, Type, U
 from absl import logging
 from etils import epath
 import six
-import tensorflow as tf
+from tensorflow_datasets.core import constants
 from tensorflow_datasets.core import dataset_info
+from tensorflow_datasets.core import dataset_metadata
 from tensorflow_datasets.core import decode
 from tensorflow_datasets.core import download
 from tensorflow_datasets.core import file_adapters
@@ -48,8 +49,11 @@ from tensorflow_datasets.core.proto import dataset_info_pb2
 from tensorflow_datasets.core.utils import file_utils
 from tensorflow_datasets.core.utils import gcs_utils
 from tensorflow_datasets.core.utils import read_config as read_config_lib
+from tensorflow_datasets.core.utils import tree_utils
 from tensorflow_datasets.core.utils import type_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import termcolor
+
 if typing.TYPE_CHECKING:
   from apache_beam.runners import runner
 
@@ -101,6 +105,21 @@ class BuilderConfig:
     )
 
 
+def _get_builder_datadir_path(builder_cls: Type[Any]) -> epath.Path:
+  """Returns the path to ConfigBuilder data dir.
+
+  Args:
+    builder_cls: a Builder class.
+
+  Returns:
+    The path to directory where the config data files of `builders_cls` can be
+    read from. e.g. "/usr/lib/[...]/tensorflow_datasets/datasets/foo".
+  """
+  pkg_names = builder_cls.__module__.split(".")
+  # -1 to remove the xxx.py python file.
+  return epath.resource_path(pkg_names[0]).joinpath(*pkg_names[1:-1])
+
+
 class DatasetBuilder(registered.RegisteredDataset):
   """Abstract base class for all datasets.
 
@@ -166,6 +185,35 @@ class DatasetBuilder(registered.RegisteredDataset):
   # displayed in the dataset documentation.
   MANUAL_DOWNLOAD_INSTRUCTIONS = None
 
+  # Optional max number of simultaneous downloads. Setting this value will
+  # override download config settings if necessary.
+  MAX_SIMULTANEOUS_DOWNLOADS: Optional[int] = None
+
+  # If not set, pkg_dir_path is inferred. However, if user of class knows better
+  # then this can be set directly before init, to avoid heuristic inferences.
+  # Example: `imported_builder_cls` function in `registered.py` module sets it.
+  pkg_dir_path: Optional[epath.Path] = None
+
+  @classmethod
+  def _get_pkg_dir_path(cls) -> epath.Path:
+    """Returns class pkg_dir_path, infer it first if not set."""
+    # We use cls.__dict__.get to check whether `pkg_dir_path` attribute has been
+    # set on the class, and not on a parent class. If we were accessing the
+    # attribute directly, a dataset Builder inheriting from another would read
+    # its metadata from its parent directory (which would be wrong).
+    if cls.__dict__.get("pkg_dir_path") is None:
+      cls.pkg_dir_path = _get_builder_datadir_path(cls)
+    return cls.pkg_dir_path
+
+  @classmethod
+  def get_metadata(cls) -> dataset_metadata.DatasetMetadata:
+    """Returns metadata (README, CITATIONS, ...) specified in config files.
+
+    The config files are read from the same package where the DatasetBuilder has
+    been defined, so those metadata might be wrong for legacy builders.
+    """
+    return dataset_metadata.load(cls._get_pkg_dir_path())
+
   @tfds_logging.builder_init()
   def __init__(
       self,
@@ -393,6 +441,35 @@ class DatasetBuilder(registered.RegisteredDataset):
         builder_configs=self.BUILDER_CONFIGS,
         default_config_name=self.DEFAULT_BUILDER_CONFIG_NAME)
 
+  def get_reference(
+      self,
+      namespace: Optional[str] = None,
+  ) -> naming.DatasetReference:
+    """Returns a reference to the dataset produced by this dataset builder.
+
+    Includes the config if specified, the version, and the data_dir that should
+    contain this dataset.
+
+    Arguments:
+      namespace: if this dataset is a community dataset, and therefore has a
+        namespace, then the namespace must be provided such that it can be set
+        in the reference. Note that a dataset builder is not aware that it is
+        part of a namespace.
+
+    Returns:
+      a reference to this instantiated builder.
+    """
+    if self.builder_config:
+      config = self.builder_config.name
+    else:
+      config = None
+    return naming.DatasetReference(
+        dataset_name=self.name,
+        namespace=namespace,
+        config=config,
+        version=self.version,
+        data_dir=epath.Path(self._data_dir_root))
+
   def download_and_prepare(
       self,
       *,
@@ -534,6 +611,18 @@ class DatasetBuilder(registered.RegisteredDataset):
           self.info.write_to_directory(self._data_dir)
       # The generated DatasetInfo contains references to `tmp_data_dir`
       self.info.update_data_dir(self._data_dir)
+
+    # Clean up incomplete files from preempted workers.
+    deleted_incomplete_files = []
+    for f in self.data_path.iterdir():
+      if utils.is_incomplete_file(f):
+        deleted_incomplete_files.append(os.fspath(f))
+        f.unlink()
+    if deleted_incomplete_files:
+      logging.info("Deleted %d incomplete files. A small selection: %s",
+                   len(deleted_incomplete_files),
+                   "\n".join(deleted_incomplete_files[:3]))
+
     self._log_download_done()
 
   @tfds_logging.as_dataset()
@@ -646,7 +735,7 @@ class DatasetBuilder(registered.RegisteredDataset):
         read_config=read_config,
         as_supervised=as_supervised,
     )
-    all_ds = tf.nest.map_structure(build_single_dataset, split)
+    all_ds = tree_utils.map_structure(build_single_dataset, split)
     return all_ds
 
   def _build_single_dataset(
@@ -697,8 +786,8 @@ class DatasetBuilder(registered.RegisteredDataset):
         Returns:
           A tuple with elements structured according to `supervised_keys`
         """
-        return tf.nest.map_structure(lambda key: features[key],
-                                     self.info.supervised_keys)
+        return tree_utils.map_structure(lambda key: features[key],
+                                        self.info.supervised_keys)
 
       ds = ds.map(lookup_nest)
 
@@ -828,6 +917,25 @@ class DatasetBuilder(registered.RegisteredDataset):
         attrs=["bold"],
     )
 
+  def dataset_info_from_configs(self, **kwargs):
+    """Returns the DatasetInfo using given kwargs and config files.
+
+    Sub-class should call this and add information not present in config files
+    using kwargs directly passed to tfds.core.DatasetInfo object.
+
+    If information is present both in passed arguments and config files, config
+    files will prevail.
+
+    Args:
+      **kwargs: kw args to pass to DatasetInfo directly.
+    """
+    metadata = self.get_metadata()
+    if metadata.description:
+      kwargs["description"] = metadata.description
+    if metadata.citation:
+      kwargs["citation"] = metadata.citation
+    return dataset_info.DatasetInfo(builder=self, **kwargs)
+
   @abc.abstractmethod
   @utils.docs.doc_private
   def _info(self) -> dataset_info.DatasetInfo:
@@ -909,6 +1017,18 @@ class DatasetBuilder(registered.RegisteredDataset):
     else:
       register_checksums_path = None
 
+    max_simultaneous_downloads = (
+        download_config.override_max_simultaneous_downloads or
+        self.MAX_SIMULTANEOUS_DOWNLOADS)
+    if (max_simultaneous_downloads and self.MAX_SIMULTANEOUS_DOWNLOADS and
+        self.MAX_SIMULTANEOUS_DOWNLOADS < max_simultaneous_downloads):
+      logging.warning(
+          "The dataset %r sets `MAX_SIMULTANEOUS_DOWNLOADS`=%s. The download "
+          "config overrides it with value `override_max_simultaneous_downloads`"
+          "=%s. Using the higher value might cause `ConnectionRefusedError`",
+          self.name, self.MAX_SIMULTANEOUS_DOWNLOADS,
+          download_config.override_max_simultaneous_downloads)
+
     return download.DownloadManager(
         download_dir=download_dir,
         extract_dir=extract_dir,
@@ -922,6 +1042,7 @@ class DatasetBuilder(registered.RegisteredDataset):
         register_checksums_path=register_checksums_path,
         verify_ssl=download_config.verify_ssl,
         dataset_name=self.name,
+        max_simultaneous_downloads=max_simultaneous_downloads,
     )
 
   @utils.docs.do_not_doc_in_subclasses
@@ -996,7 +1117,6 @@ class FileReaderBuilder(DatasetBuilder):
    * `ReadOnlyBuilder`: Can only read pre-generated datasets. A user can
      generate a dataset with `GeneratorBasedBuilder`, and read them with
      `ReadOnlyBuilder` without requiring the original generation code.
-
   """
 
   @tfds_logging.builder_init()
@@ -1066,7 +1186,6 @@ class GeneratorBasedBuilder(FileReaderBuilder):
 
   It expects subclasses to overwrite `_split_generators` to return a dict of
   splits, generators. See the method docstrings for details.
-
   """
 
   @abc.abstractmethod
@@ -1123,7 +1242,7 @@ class GeneratorBasedBuilder(FileReaderBuilder):
     """Default function to generate examples for each split.
 
     The function should return a collection of `(key, examples)`. Examples
-    will be encoded are written to disk. See `yields` section for details.
+    will be encoded and written to disk. See `yields` section for details.
 
     The function can return/yield:
 
@@ -1217,10 +1336,12 @@ class GeneratorBasedBuilder(FileReaderBuilder):
     split_builder = split_builder_lib.SplitBuilder(
         split_dict=self.info.splits,
         features=self.info.features,
+        dataset_size=self.info.dataset_size,
         max_examples_per_split=download_config.max_examples_per_split,
         beam_options=download_config.beam_options,
         beam_runner=download_config.beam_runner,
         file_format=self.info.file_format,
+        shard_config=download_config.get_shard_config(),
     )
     # Wrap the generation inside a context manager.
     # If `beam` is used during generation (when a pipeline gets created),
