@@ -20,9 +20,11 @@ import os
 import tempfile
 from unittest import mock
 
+from absl import flags
 from absl.testing import parameterized
 
 import dill
+from etils import epath
 import numpy as np
 import tensorflow as tf
 from tensorflow_datasets import testing
@@ -33,10 +35,12 @@ from tensorflow_datasets.core import dataset_utils
 from tensorflow_datasets.core import download
 from tensorflow_datasets.core import features
 from tensorflow_datasets.core import load
+from tensorflow_datasets.core import naming
 from tensorflow_datasets.core import splits as splits_lib
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.utils import file_utils
 from tensorflow_datasets.core.utils import read_config as read_config_lib
+from tensorflow_datasets.testing.dummy_config_based_datasets.dummy_ds_1 import dummy_ds_1_dataset_builder
 
 DummyDatasetSharedGenerator = testing.DummyDatasetSharedGenerator
 
@@ -85,6 +89,10 @@ class DummyDatasetWithConfigs(dataset_builder.GeneratorBasedBuilder):
       yield i, {"x": x}
 
 
+class DummyDatasetWithDefaultConfig(DummyDatasetWithConfigs):
+  DEFAULT_BUILDER_CONFIG_NAME = "plus2"
+
+
 class InvalidSplitDataset(DummyDatasetWithConfigs):
 
   def _split_generators(self, _):
@@ -92,6 +100,39 @@ class InvalidSplitDataset(DummyDatasetWithConfigs):
     return {"all": self._generate_examples(range(5))}
 
 
+class GetBuilderDatadirPathTest(testing.TestCase):
+
+  def test_builder_data_dir_path_is_correct(self):
+    with mock.patch.object(
+        epath, "resource_path", return_value=epath.Path("/base/path")):
+      path = dataset_builder._get_builder_datadir_path(
+          dummy_ds_1_dataset_builder.Builder)
+    self.assertEqual(
+        os.fspath(path),
+        "/base/path/testing/dummy_config_based_datasets/dummy_ds_1")
+
+
+class ConfigBasedBuilderTest(testing.TestCase):
+
+  def test_get_metadata(self):
+    builder_cls = dummy_ds_1_dataset_builder.Builder()
+    metadata = builder_cls.get_metadata()
+    self.assertEqual(metadata.tags, ["content.data-type.image"])
+
+  def test_dummy_ds_1_read_from_config(self):
+    ds_builder = dummy_ds_1_dataset_builder.Builder()
+    info = ds_builder._info()
+    self.assertEqual(info.description,
+                     "Description of `dummy_ds_1` dummy config-based dataset.")
+    self.assertEqual(
+        info.citation, """@Article{google22tfds,
+author = "The TFDS team",
+title = "TFDS: a collection of ready-to-use datasets for use with TensorFlow, Jax, and other Machine Learning frameworks.",
+journal = "ML gazette",
+year = "2022"
+}""")
+
+
 class DatasetBuilderTest(testing.TestCase):
 
   @classmethod
@@ -113,6 +154,34 @@ class DatasetBuilderTest(testing.TestCase):
       self.assertEqual(20, len(data))
       self.assertLess(data[0]["x"], 30)
 
+  # Disable test until dependency on Riegeli is fixed.
+  # @testing.run_in_graph_and_eager_modes()
+  # def test_load_with_specified_format(self):
+  #   with testing.tmp_dir(self.get_temp_dir()) as tmp_dir:
+  #     dataset, ds_info = load.load(
+  #         name="dummy_dataset_with_configs",
+  #         with_info=True,
+  #         data_dir=tmp_dir,
+  #         download=True,
+  #         split=splits_lib.Split.TRAIN,
+  #         download_and_prepare_kwargs={"file_format": "riegeli"})
+  #     self.assertEqual(ds_info.file_format.name, "RIEGELI")
+  #     files = tf.io.gfile.listdir(
+  #         os.path.join(tmp_dir, "dummy_dataset_with_configs",
+  #                      "plus1", "0.0.1"))
+  #     self.assertSetEqual(
+  #         set(files), {
+  #             "dummy_dataset_with_configs-test.riegeli-00000-of-00001",
+  #             "dummy_dataset_with_configs-test.riegeli-00000-of-00001_index.json",
+  #             "dummy_dataset_with_configs-train.riegeli-00000-of-00001",
+  #             "dummy_dataset_with_configs-train.riegeli-00000-of-00001_index.json",
+  #             "features.json",
+  #             "dataset_info.json",
+  #         })
+  #     data = list(dataset_utils.as_numpy(dataset))
+  #     self.assertEqual(20, len(data))
+  #     self.assertLess(data[0]["x"], 30)
+
   @testing.run_in_graph_and_eager_modes()
   def test_determinism(self):
     ds = self.builder.as_dataset(
@@ -151,6 +220,7 @@ class DatasetBuilderTest(testing.TestCase):
       self.assertSetEqual(
           set([
               "dataset_info.json",
+              "features.json",
               "image.image.json",
               "mnist-test.tfrecord-00000-of-00001",
               "mnist-train.tfrecord-00000-of-00001",
@@ -207,7 +277,7 @@ class DatasetBuilderTest(testing.TestCase):
       builder = DummyDatasetWithConfigs(config=plus1_config, data_dir=tmp_dir)
       self.assertIs(plus1_config, builder.builder_config)
       self.assertIs(builder.builder_config,
-                    DummyDatasetWithConfigs.BUILDER_CONFIGS[0])
+                    DummyDatasetWithConfigs.default_builder_config)
 
   @testing.run_in_graph_and_eager_modes()
   def test_with_configs(self):
@@ -245,6 +315,12 @@ class DatasetBuilderTest(testing.TestCase):
         self.assertCountEqual([incr + el for el in range(30)],
                               train_data + test_data)
 
+  def test_default_builder_config(self):
+    self.assertEqual(DummyDatasetWithConfigs.default_builder_config.name,
+                     "plus1")
+    self.assertEqual(DummyDatasetWithDefaultConfig.default_builder_config.name,
+                     "plus2")
+
   def test_read_config(self):
     is_called = []
 
@@ -373,6 +449,17 @@ class DatasetBuilderTest(testing.TestCase):
     self.assertEqual(builder.version, config_version)
     self.assertEqual(builder.release_notes, config_release_notes)
 
+  def test_get_reference(self):
+    tmp_dir = self.get_temp_dir()
+    builder = DummyDatasetWithConfigs(config="plus1", data_dir=tmp_dir)
+    reference = builder.get_reference()
+    expected_reference = naming.DatasetReference(
+        dataset_name="dummy_dataset_with_configs",
+        config="plus1",
+        version="0.0.1",
+        data_dir=epath.Path(tmp_dir))
+    self.assertEqual(reference, expected_reference)
+
 
 class DatasetBuilderMultiDirTest(testing.TestCase):
   """Tests for multi-dir."""
@@ -390,6 +477,7 @@ class DatasetBuilderMultiDirTest(testing.TestCase):
     # Create a new temp dir
     self.other_data_dir = os.path.join(self.get_temp_dir(), "other_dir")
     # Overwrite the default data_dir (as files get created)
+
     self._original_data_dir = constants.DATA_DIR
     constants.DATA_DIR = os.path.join(self.get_temp_dir(), "default_dir")
     self.default_data_dir = constants.DATA_DIR
@@ -521,12 +609,10 @@ class OrderedDatasetBuilderTest(testing.TestCase):
   @classmethod
   def setUpClass(cls):
     super(OrderedDatasetBuilderTest, cls).setUpClass()
-    with mock.patch(
-        "tensorflow_datasets.core.tfrecords_writer._get_number_shards",
-        lambda x, y: 10):
-      cls.builder = DummyOrderedDataset(
-          data_dir=os.path.join(tempfile.gettempdir(), "tfds"))
-      cls.builder.download_and_prepare()
+    cls.builder = DummyOrderedDataset(
+        data_dir=os.path.join(tempfile.gettempdir(), "tfds"))
+    cls.builder.download_and_prepare(
+        download_config=download.DownloadConfig(num_shards=10))
 
   @testing.run_in_graph_and_eager_modes()
   def test_sorted_by_key(self):
