@@ -30,17 +30,20 @@ processed the dataset as well:
  - etc.
 """
 
+from __future__ import annotations
+
 import abc
+import dataclasses
 import json
 import os
 import posixpath
 import tempfile
-from typing import Dict, Optional, Tuple, Union
+from typing import Any, Dict, Optional, Tuple, Union
 
 from absl import logging
+from etils import epath
 import six
-import tensorflow as tf
-
+from tensorflow_datasets.core import constants
 from tensorflow_datasets.core import file_adapters
 from tensorflow_datasets.core import lazy_imports_lib
 from tensorflow_datasets.core import naming
@@ -50,7 +53,7 @@ from tensorflow_datasets.core.features import feature as feature_lib
 from tensorflow_datasets.core.features import top_level_feature
 from tensorflow_datasets.core.proto import dataset_info_pb2
 from tensorflow_datasets.core.utils import gcs_utils
-from tensorflow_datasets.core.utils import type_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 from google.protobuf import json_format
 
@@ -58,10 +61,13 @@ from google.protobuf import json_format
 Nest = Union[Tuple["Nest", ...], Dict[str, "Nest"], str]  # pytype: disable=not-supported-yet
 SupervisedKeysType = Union[Tuple[Nest, Nest], Tuple[Nest, Nest, Nest]]
 
-# Name of the file to output the DatasetInfo protobuf object.
-DATASET_INFO_FILENAME = "dataset_info.json"
-LICENSE_FILENAME = "LICENSE"
-METADATA_FILENAME = "metadata.json"
+
+def dataset_info_path(dataset_info_dir: epath.PathLike) -> epath.Path:
+  return epath.Path(dataset_info_dir) / constants.DATASET_INFO_FILENAME
+
+
+def license_path(dataset_info_dir: epath.PathLike) -> epath.Path:
+  return epath.Path(dataset_info_dir) / constants.LICENSE_FILENAME
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -90,6 +96,52 @@ class Metadata(dict):
     raise NotImplementedError()
 
 
+@dataclasses.dataclass()
+class DatasetIdentity:
+  """Identity of a dataset that completely identifies a dataset."""
+  name: str
+  version: utils.Version
+  data_dir: str
+  module_name: str
+  config_name: Optional[str] = None
+  config_description: Optional[str] = None
+  release_notes: Optional[Dict[str, str]] = None
+
+  @classmethod
+  def from_builder(cls, builder) -> "DatasetIdentity":
+    if builder.builder_config:
+      config_name = builder.builder_config.name
+      config_description = builder.builder_config.description
+    else:
+      config_name = None
+      config_description = None
+    return cls(
+        name=builder.name,
+        version=utils.Version(builder.version),
+        data_dir=builder.data_dir,
+        module_name=str(builder.__module__),
+        config_name=config_name,
+        config_description=config_description,
+        release_notes=builder.release_notes,
+    )
+
+  @classmethod
+  def from_proto(
+      cls,
+      info_proto: dataset_info_pb2.DatasetInfo,
+      data_dir: str,
+  ) -> "DatasetIdentity":
+    return cls(
+        name=info_proto.name,
+        version=utils.Version(info_proto.version),
+        data_dir=data_dir,
+        module_name=info_proto.module_name,
+        config_name=info_proto.config_name,
+        config_description=info_proto.config_description,
+        release_notes={k: v for k, v in info_proto.release_notes.items()},
+    )
+
+
 class DatasetInfo(object):
   """Information about a dataset.
 
@@ -100,13 +152,12 @@ class DatasetInfo(object):
   (before the first `builder.download_and_prepare()` call). For example splits
   names or number of examples might be missing (as they are computed
   at dataset creation time).
-
   """
 
   def __init__(
       self,
       *,
-      builder,
+      builder: Union[DatasetIdentity, Any],
       description: Optional[str] = None,
       features: Optional[feature_lib.FeatureConnector] = None,
       supervised_keys: Optional[SupervisedKeysType] = None,
@@ -117,10 +168,12 @@ class DatasetInfo(object):
       license: Optional[str] = None,  # pylint: disable=redefined-builtin
       redistribution_info: Optional[Dict[str, str]] = None,
       split_dict: Optional[splits_lib.SplitDict] = None):
+    # pyformat: disable
     """Constructs DatasetInfo.
 
     Args:
-      builder: `DatasetBuilder`, dataset builder for this info.
+      builder: `DatasetBuilder` or `DatasetIdentity`. The dataset builder or
+        identity will be used to populate this info.
       description: `str`, description of this dataset.
       features: `tfds.features.FeaturesDict`, Information on the feature dict of
         the `tf.data.Dataset()` object from the `builder.as_dataset()` method.
@@ -158,25 +211,23 @@ class DatasetInfo(object):
         dataset.
       split_dict: information about the splits in this dataset.
     """
-    self._builder = builder
-
-    if builder.builder_config:
-      config_name = builder.builder_config.name
-      config_description = builder.builder_config.description
+    # pyformat: enable
+    self._builder_or_identity = builder
+    if isinstance(builder, DatasetIdentity):
+      self._identity = builder
     else:
-      config_name = None
-      config_description = None
+      self._identity = DatasetIdentity.from_builder(builder)
 
     self._info_proto = dataset_info_pb2.DatasetInfo(
-        name=builder.name,
+        name=self._identity.name,
         description=utils.dedent(description),
-        version=str(builder.version),
-        release_notes=builder.release_notes,
+        version=str(self._identity.version),
+        release_notes=self._identity.release_notes,
         disable_shuffling=disable_shuffling,
-        config_name=config_name,
-        config_description=config_description,
+        config_name=self._identity.config_name,
+        config_description=self._identity.config_description,
         citation=utils.dedent(citation),
-        module_name=str(builder.__module__),
+        module_name=self._identity.module_name,
         redistribution_info=dataset_info_pb2.RedistributionInfo(
             license=utils.dedent(license or redistribution_info.pop("license")),
             **redistribution_info) if redistribution_info else None)
@@ -190,7 +241,7 @@ class DatasetInfo(object):
             "DatasetInfo.features only supports FeaturesDict or Sequence at "
             "the top-level. Got {}".format(features))
     self._features = features
-    self._splits = splits_lib.SplitDict([], dataset_name=self._builder.name)
+    self._splits = splits_lib.SplitDict([])
     if split_dict:
       self.set_splits(split_dict)
     if supervised_keys is not None:
@@ -206,6 +257,11 @@ class DatasetInfo(object):
     # Is this object initialized with both the static and the dynamic data?
     self._fully_initialized = False
 
+  @property
+  def _builder(self) -> Any:
+    logging.warning("DEPRECATED: please do not use _builder as this may change")
+    return self._builder_or_identity
+
   @classmethod
   def from_proto(cls, builder,
                  proto: dataset_info_pb2.DatasetInfo) -> "DatasetInfo":
@@ -219,6 +275,10 @@ class DatasetInfo(object):
     supervised_keys = None
     if proto.HasField("supervised_keys"):
       supervised_keys = _supervised_keys_from_proto(proto.supervised_keys)
+    filename_template = naming.ShardedFileTemplate(
+        dataset_name=builder.name,
+        data_dir=builder.data_dir,
+        filetype_suffix=proto.file_format or "tfrecord")
     return cls(
         builder=builder,
         description=proto.description,
@@ -227,7 +287,9 @@ class DatasetInfo(object):
         disable_shuffling=proto.disable_shuffling,
         citation=proto.citation,
         license=proto.redistribution_info.license,
-        split_dict=splits_lib.SplitDict.from_proto(proto.name, proto.splits),
+        split_dict=splits_lib.SplitDict.from_proto(
+            repeated_split_infos=proto.splits,
+            filename_template=filename_template),
     )
 
   @property
@@ -236,18 +298,22 @@ class DatasetInfo(object):
 
   @property
   def name(self) -> str:
-    return self.as_proto.name
+    return self._identity.name
 
   @property
   def config_name(self) -> str:
-    return self.as_proto.config_name
+    return self._info_proto.config_name
+
+  @property
+  def config_description(self) -> str:
+    return self._identity.config_description
 
   @property
   def full_name(self):
     """Full canonical name: (<dataset_name>/<config_name>/<version>)."""
-    names = [self._builder.name]
-    if self._builder.builder_config:
-      names.append(self._builder.builder_config.name)
+    names = [self.name]
+    if self.config_name:
+      names.append(self.config_name)
     names.append(str(self.version))
     return posixpath.join(*names)
 
@@ -257,11 +323,11 @@ class DatasetInfo(object):
 
   @property
   def version(self):
-    return self._builder.version
+    return self._identity.version
 
   @property
   def release_notes(self) -> Optional[Dict[str, str]]:
-    return self._builder.release_notes
+    return self._identity.release_notes
 
   @property
   def disable_shuffling(self) -> bool:
@@ -279,7 +345,7 @@ class DatasetInfo(object):
 
   @property
   def data_dir(self):
-    return self._builder.data_dir
+    return self._identity.data_dir
 
   @property
   def dataset_size(self) -> utils.Size:
@@ -319,7 +385,7 @@ class DatasetInfo(object):
 
   @property
   def module_name(self) -> str:
-    return self.as_proto.module_name
+    return self._identity.module_name
 
   @property
   def file_format(self) -> Optional[file_adapters.FileFormat]:
@@ -330,6 +396,7 @@ class DatasetInfo(object):
   def set_file_format(
       self,
       file_format: Union[None, str, file_adapters.FileFormat],
+      override: bool = False,
   ) -> None:
     """Internal function to define the file format.
 
@@ -338,23 +405,30 @@ class DatasetInfo(object):
 
     Args:
       file_format: The file format.
+      override: Whether the file format should be overridden if it is already
+        set.
+
+    Raises:
+      ValueError: if the file format was already set and the `override`
+      parameter was False.
+      RuntimeError: if an incorrect combination of options is given, e.g.
+      `override=True` when the DatasetInfo is already fully initialized.
     """
     # If file format isn't present already, fallback to `DEFAULT_FILE_FORMAT`
     file_format = (
         file_format  # Format explicitly given: tfds.builder(..., file_format=x)
         or self.file_format  # Format restored from dataset_info.json
         or file_adapters.DEFAULT_FILE_FORMAT)
-    try:
-      new_file_format = file_adapters.FileFormat(file_format)
-    except ValueError as e:
-      all_values = [f.value for f in file_adapters.FileFormat]
-      utils.reraise(e, suffix=f". Valid file formats: {all_values}")
+    file_format = file_adapters.FileFormat.from_value(file_format)
 
     # If the file format has been set once, file format should be consistent
-    if self.file_format and self.file_format != new_file_format:
+    if (not override and self.file_format and self.file_format != file_format):
       raise ValueError(f"File format is already set to {self.file_format}. "
-                       f"Got {new_file_format}")
-    self.as_proto.file_format = new_file_format.value
+                       f"Got {file_format}")
+    if override and self._fully_initialized:
+      raise RuntimeError("Cannot override the file format "
+                         "when the DatasetInfo is already fully initialized!")
+    self._info_proto.file_format = file_format.value
 
   @property
   def splits(self) -> splits_lib.SplitDict:
@@ -362,49 +436,76 @@ class DatasetInfo(object):
 
   def set_splits(self, split_dict: splits_lib.SplitDict) -> None:
     """Split setter (private method)."""
-    if self._builder.name != split_dict._dataset_name:  # pylint: disable=protected-access
-      raise AssertionError(
-          "SplitDict dataset_name does not seem to match dataset_info. "  # pylint: disable=protected-access
-          f"{self._builder.name} != {split_dict._dataset_name}")
+    for split, split_info in split_dict.items():
+      if isinstance(split_info, splits_lib.MultiSplitInfo):
+        # When splits are from multiple folders, the dataset can be different.
+        continue
+      if (split_info.filename_template and
+          self.name != split_info.filename_template.dataset_name):
+        raise AssertionError(
+            f"SplitDict contains SplitInfo for split {split} whose "
+            "dataset_name does not match to the dataset name in dataset_info. "
+            f"{self.name} != {split_info.filename_template.dataset_name}")
 
     # If the statistics have been pre-loaded, forward the statistics
-    # into the new split_dict
+    # into the new split_dict. Also add the filename template if it's not set.
     new_split_infos = []
+    incomplete_filename_template = naming.ShardedFileTemplate(
+        dataset_name=self.name,
+        data_dir=self.data_dir,
+        filetype_suffix=(self.as_proto.file_format or
+                         file_adapters.DEFAULT_FILE_FORMAT.value))
     for split_info in split_dict.values():
+      if isinstance(split_info, splits_lib.MultiSplitInfo):
+        new_split_infos.append(split_info)
+        continue
       old_split_info = self._splits.get(split_info.name)
       if (not split_info.statistics.ByteSize() and old_split_info and
           old_split_info.statistics.ByteSize() and
           old_split_info.shard_lengths == split_info.shard_lengths):
         split_info = split_info.replace(statistics=old_split_info.statistics)
+      if not split_info.filename_template:
+        filename_template = incomplete_filename_template.replace(
+            split=split_info.name)
+        split_info = split_info.replace(filename_template=filename_template)
       new_split_infos.append(split_info)
 
     # Update the dictionary representation.
-    self._splits = splits_lib.SplitDict(
-        new_split_infos,
-        dataset_name=self._builder.name,
-    )
+    self._splits = splits_lib.SplitDict(new_split_infos)
 
     # Update the proto
+    # Note that the proto should not be saved or used for multi-folder datasets.
     del self.as_proto.splits[:]  # Clear previous
-    for split_info in split_dict.to_proto():
-      self.as_proto.splits.add().CopyFrom(split_info)
+    for split_info in self._splits.values():
+      if isinstance(split_info, splits_lib.MultiSplitInfo):
+        for si in split_info.split_infos:
+          self.as_proto.splits.add().CopyFrom(si.to_proto())
+      else:
+        self.as_proto.splits.add().CopyFrom(split_info.to_proto())
+
+  def update_data_dir(self, data_dir: str) -> None:
+    """Updates the data dir for each split."""
+    new_split_infos = []
+    for split_info in self._splits.values():
+      if isinstance(split_info, splits_lib.MultiSplitInfo):
+        raise RuntimeError(
+            "Updating the data_dir for MultiSplitInfo is not supported!")
+      filename_template = split_info.filename_template.replace(
+          data_dir=data_dir)
+      new_split_info = split_info.replace(filename_template=filename_template)
+      new_split_infos.append(new_split_info)
+    self.set_splits(splits_lib.SplitDict(new_split_infos))
 
   @property
   def initialized(self) -> bool:
     """Whether DatasetInfo has been fully initialized."""
     return self._fully_initialized
 
-  def _dataset_info_path(self, dataset_info_dir):
-    return os.path.join(dataset_info_dir, DATASET_INFO_FILENAME)
-
-  def _license_path(self, dataset_info_dir):
-    return os.path.join(dataset_info_dir, LICENSE_FILENAME)
-
   @property
   def as_json(self) -> str:
     return json_format.MessageToJson(self.as_proto, sort_keys=True)
 
-  def write_to_directory(self, dataset_info_dir) -> None:
+  def write_to_directory(self, dataset_info_dir: epath.PathLike) -> None:
     """Write `DatasetInfo` as JSON to `dataset_info_dir`."""
     # Save the features structure & metadata (vocabulary, labels,...)
     if self.features:
@@ -415,13 +516,12 @@ class DatasetInfo(object):
       self.metadata.save_metadata(dataset_info_dir)
 
     if self.redistribution_info.license:
-      with tf.io.gfile.GFile(self._license_path(dataset_info_dir), "w") as f:
-        f.write(self.redistribution_info.license)
+      license_path(dataset_info_dir).write_text(
+          self.redistribution_info.license)
 
-    with tf.io.gfile.GFile(self._dataset_info_path(dataset_info_dir), "w") as f:
-      f.write(self.as_json)
+    dataset_info_path(dataset_info_dir).write_text(self.as_json)
 
-  def read_from_directory(self, dataset_info_dir: str) -> None:
+  def read_from_directory(self, dataset_info_dir: epath.PathLike) -> None:
     """Update DatasetInfo from the JSON files in `dataset_info_dir`.
 
     This function updates all the dynamically generated fields (num_examples,
@@ -430,39 +530,53 @@ class DatasetInfo(object):
     This will overwrite all previous metadata.
 
     Args:
-      dataset_info_dir: `str` The directory containing the metadata file. This
-        should be the root directory of a specific dataset version.
+      dataset_info_dir: The directory containing the metadata file. This should
+        be the root directory of a specific dataset version.
 
     Raises:
       FileNotFoundError: If the dataset_info.json can't be found.
     """
     logging.info("Load dataset info from %s", dataset_info_dir)
 
-    json_filename = self._dataset_info_path(dataset_info_dir)
-    if not tf.io.gfile.exists(json_filename):
+    # Load the metadata from disk
+    try:
+      parsed_proto = read_from_json(dataset_info_path(dataset_info_dir))
+    except Exception as e:
       raise FileNotFoundError(
           "Tried to load `DatasetInfo` from a directory which does not exist or"
           " does not contain `dataset_info.json`. Please delete the directory "
           f"`{dataset_info_dir}`  if you are trying to re-generate the "
-          "dataset.")
+          "dataset.") from e
 
-    # Load the metadata from disk
-    parsed_proto = read_from_json(json_filename)
+    if str(self.version) != parsed_proto.version:
+      raise AssertionError(
+          "The constructed DatasetInfo instance and the restored proto version "
+          "do not match. Builder version: {}. Proto version: {}".format(
+              self.version, parsed_proto.version))
+
+    self._identity = DatasetIdentity.from_proto(
+        info_proto=parsed_proto, data_dir=dataset_info_dir)
 
     # Update splits
-    split_dict = splits_lib.SplitDict.from_proto(self.name, parsed_proto.splits)
+    filename_template = naming.ShardedFileTemplate(
+        dataset_name=self.name,
+        data_dir=self.data_dir,
+        filetype_suffix=parsed_proto.file_format or "tfrecord")
+    split_dict = splits_lib.SplitDict.from_proto(
+        repeated_split_infos=parsed_proto.splits,
+        filename_template=filename_template)
     self.set_splits(split_dict)
 
     # Restore the feature metadata (vocabulary, labels names,...)
     if self.features:
       self.features.load_metadata(dataset_info_dir)
     # For `ReadOnlyBuilder`, reconstruct the features from the config.
-    elif tf.io.gfile.exists(feature_lib.make_config_path(dataset_info_dir)):
+    elif feature_lib.make_config_path(dataset_info_dir).exists():
       self._features = feature_lib.FeatureConnector.from_config(
           dataset_info_dir)
     # Restore the MetaDataDict from metadata.json if there is any
     if (self.metadata is not None or
-        tf.io.gfile.exists(_metadata_filepath(dataset_info_dir))):
+        _metadata_filepath(dataset_info_dir).exists()):
       # If the dataset was loaded from file, self.metadata will be `None`, so
       # we create a MetadataDict first.
       if self.metadata is None:
@@ -486,12 +600,12 @@ class DatasetInfo(object):
       except ValueError:
         is_defined_in_restored = bool(field_value_restored)
 
-      # If field is defined in code, we ignore the value
+      # If field is defined in code, we ignore the value.
       if is_defined:
         if field_value != field_value_restored:
           logging.info(
-              "Field info.%s from disk and from code do not match. Keeping "
-              "the one from code.", field_name)
+              "Field info.%s from disk and from code do not match. "
+              "Keeping the one from code.", field_name)
         continue
       # If the field is also not defined in JSON file, we do nothing
       if not is_defined_in_restored:
@@ -502,12 +616,6 @@ class DatasetInfo(object):
       else:
         setattr(self._info_proto, field_name, field_value_restored)
 
-    if self._builder._version != self.version:  # pylint: disable=protected-access
-      raise AssertionError(
-          "The constructed DatasetInfo instance and the restored proto version "
-          "do not match. Builder version: {}. Proto version: {}".format(
-              self._builder._version, self.version))  # pylint: disable=protected-access
-
     # Mark as fully initialized.
     self._fully_initialized = True
 
@@ -515,16 +623,16 @@ class DatasetInfo(object):
     """Initialize DatasetInfo from GCS bucket info files."""
     # In order to support Colab, we use the HTTP GCS API to access the metadata
     # files. They are copied locally and then loaded.
-    tmp_dir = tempfile.mkdtemp("tfds")
+    tmp_dir = epath.Path(tempfile.mkdtemp("tfds"))
     data_files = gcs_utils.gcs_dataset_info_files(self.full_name)
     if not data_files:
       return
     logging.info(
         "Load pre-computed DatasetInfo (eg: splits, num examples,...) "
         "from GCS: %s", self.full_name)
-    for fname in data_files:
-      out_fname = os.path.join(tmp_dir, os.path.basename(fname))
-      tf.io.gfile.copy(os.fspath(gcs_utils.gcs_path(fname)), out_fname)
+    for path in data_files:
+      out_fname = tmp_dir / path.name
+      epath.Path(path).copy(out_fname)
     self.read_from_directory(tmp_dir)
 
   def __repr__(self):
@@ -541,6 +649,9 @@ class DatasetInfo(object):
     else:
       config_description = SKIP
 
+    file_format_str = (
+        self.file_format.value
+        if self.file_format else file_adapters.DEFAULT_FILE_FORMAT.value)
     lines = ["tfds.core.DatasetInfo("]
     for key, value in [
         ("name", repr(self.name)),
@@ -549,6 +660,7 @@ class DatasetInfo(object):
         ("config_description", config_description),
         ("homepage", repr(self.homepage)),
         ("data_path", repr(self.data_dir)),
+        ("file_format", file_format_str),
         ("download_size", self.download_size),
         ("dataset_size", self.dataset_size),
         ("features", _indent(repr(self.features))),
@@ -620,8 +732,8 @@ def _nest_from_proto(proto: dataset_info_pb2.SupervisedKeys.Nest) -> Nest:
     return tuple(_nest_from_proto(item) for item in proto.tuple.items)
   elif proto.HasField("dict"):
     return {
-        key: _nest_from_proto(proto.dict.dict[key])
-        for key in sorted(proto.dict.dict.keys())
+        key: _nest_from_proto(value)
+        for key, value in sorted(proto.dict.dict.items())
     }
   elif proto.HasField("feature_key"):
     return proto.feature_key
@@ -682,7 +794,9 @@ def get_dataset_feature_statistics(builder, split):
   # Avoid generating a large number of buckets in rank histogram
   # (default is 1000).
   stats_options = tfdv.StatsOptions(
-      num_top_values=10, num_rank_histogram_buckets=10)
+      num_top_values=10,
+      num_rank_histogram_buckets=10,
+      use_sketch_based_topk_uniques=False)
   if filetype_suffix == "csv":
     statistics = tfdv.generate_statistics_from_csv(
         filepattern, stats_options=stats_options)
@@ -701,16 +815,16 @@ def get_dataset_feature_statistics(builder, split):
   return statistics.datasets[0], schema
 
 
-def read_from_json(path: type_utils.PathLike) -> dataset_info_pb2.DatasetInfo:
+def read_from_json(path: epath.PathLike) -> dataset_info_pb2.DatasetInfo:
   """Read JSON-formatted proto into DatasetInfo proto."""
-  json_str = utils.as_path(path).read_text()
+  json_str = epath.Path(path).read_text()
   # Parse it back into a proto.
   parsed_proto = json_format.Parse(json_str, dataset_info_pb2.DatasetInfo())
   return parsed_proto
 
 
 def read_proto_from_builder_dir(
-    builder_dir: type_utils.PathLike) -> dataset_info_pb2.DatasetInfo:
+    builder_dir: epath.PathLike) -> dataset_info_pb2.DatasetInfo:
   """Reads the dataset info from the given builder dir.
 
   Args:
@@ -720,13 +834,13 @@ def read_proto_from_builder_dir(
     The DatasetInfo proto as read from the builder dir.
 
   Raises:
-    FileNotFoundError: If the builder_dir does not exists.
+    FileNotFoundError: If the builder_dir does not exist.
   """
-  info_path = os.path.join(
-      os.path.expanduser(builder_dir), DATASET_INFO_FILENAME)
-  if not tf.io.gfile.exists(info_path):
+  info_path = epath.Path(
+      os.path.expanduser(builder_dir)) / constants.DATASET_INFO_FILENAME
+  if not info_path.exists():
     raise FileNotFoundError(
-        f"Could not load dataset info: {info_path} does not exists.")
+        f"Could not load dataset info: {info_path} does not exist.")
   return read_from_json(info_path)
 
 
@@ -744,8 +858,8 @@ def pack_as_supervised_ds(
     return ds
 
 
-def _metadata_filepath(data_dir):
-  return os.path.join(data_dir, METADATA_FILENAME)
+def _metadata_filepath(data_dir: epath.PathLike) -> epath.Path:
+  return epath.Path(data_dir) / constants.METADATA_FILENAME
 
 
 class MetadataDict(Metadata, dict):
@@ -756,13 +870,13 @@ class MetadataDict(Metadata, dict):
 
   def save_metadata(self, data_dir):
     """Save the metadata."""
-    with tf.io.gfile.GFile(_metadata_filepath(data_dir), "w") as f:
+    with _metadata_filepath(data_dir).open(mode="w") as f:
       json.dump(self, f)
 
   def load_metadata(self, data_dir):
     """Restore the metadata."""
     self.clear()
-    with tf.io.gfile.GFile(_metadata_filepath(data_dir), "r") as f:
+    with _metadata_filepath(data_dir).open(mode="r") as f:
       self.update(json.load(f))
 
 
@@ -771,10 +885,10 @@ class BeamMetadataDict(MetadataDict):
 
   def __init__(self, *args, **kwargs):
     super(BeamMetadataDict, self).__init__(*args, **kwargs)
-    self._tempdir = tempfile.mkdtemp("tfds_beam_metadata")
+    self._tempdir = epath.Path(tempfile.mkdtemp("tfds_beam_metadata"))
 
-  def _temp_filepath(self, key):
-    return os.path.join(self._tempdir, "%s.json" % key)
+  def _temp_filepath(self, key) -> epath.Path:
+    return self._tempdir / f"{key}.json"
 
   def __setitem__(self, key, item):
     """Creates write sink for beam PValues or sets value of key in `dict`.
@@ -816,7 +930,7 @@ class BeamMetadataDict(MetadataDict):
           | "metadata_%s_tolist" % key >> beam.combiners.ToList()
           | "metadata_%s_tojson" % key >> beam.Map(_to_json)
           | "metadata_%s_write" % key >> beam.io.WriteToText(
-              self._temp_filepath(key),
+              os.fspath(self._temp_filepath(key)),
               num_shards=1,
               shard_name_template="",
           ))
@@ -827,7 +941,7 @@ class BeamMetadataDict(MetadataDict):
     beam = lazy_imports_lib.lazy_imports.apache_beam
     for key, item in self.items():
       if isinstance(item, beam.pvalue.PValue):
-        with tf.io.gfile.GFile(self._temp_filepath(key), "r") as f:
+        with self._temp_filepath(key).open(mode="r") as f:
           self[key] = json.load(f)
-    tf.io.gfile.rmtree(self._tempdir)
+    self._tempdir.rmtree()
     super(BeamMetadataDict, self).save_metadata(data_dir)
