@@ -22,17 +22,19 @@ import os
 import pickle
 from unittest import mock
 
+from absl.testing import parameterized
+from etils import epath
 import promise
 
 import tensorflow as tf
 
 from tensorflow_datasets import testing
-from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.download import checksums as checksums_lib
 from tensorflow_datasets.core.download import download_manager as dm
 from tensorflow_datasets.core.download import downloader
 from tensorflow_datasets.core.download import extractor
 from tensorflow_datasets.core.download import resource as resource_lib
+from tensorflow_datasets.core.utils import tree_utils
 
 ZIP = resource_lib.ExtractMethod.ZIP
 TAR = resource_lib.ExtractMethod.TAR
@@ -44,7 +46,7 @@ def _sha256(str_):
 
 
 def _as_path(nested_paths):
-  return tf.nest.map_structure(utils.as_path, nested_paths)
+  return tree_utils.map_structure(epath.Path, nested_paths)
 
 
 def _info_path(path):
@@ -62,7 +64,7 @@ class PathDict(collections.UserDict):
     return super().__getitem__(os.fspath(key))
 
   def __setitem__(self, key, value):
-    return super().__setitem__(os.fspath(key), utils.as_path(value))
+    return super().__setitem__(os.fspath(key), epath.Path(value))
 
 
 class Artifact(object):
@@ -78,12 +80,12 @@ class Artifact(object):
         filename=name,
     )
     self.file_name = resource_lib.get_dl_fname(url, self.url_info.checksum)
-    self.file_path = utils.as_path(f'/dl_dir/{self.file_name}')
+    self.file_path = epath.Path(f'/dl_dir/{self.file_name}')
     self.url_name = resource_lib.get_dl_fname(url, _sha256(url))
-    self.url_path = utils.as_path(f'/dl_dir/{self.url_name}')
+    self.url_path = epath.Path(f'/dl_dir/{self.url_name}')
 
 
-class DownloadManagerTest(testing.TestCase):
+class DownloadManagerTest(testing.TestCase, parameterized.TestCase):
   """Download manager test.
 
   During tests, the `tf.io.gfile` API is mocked. Instead, files I/O are tracked
@@ -117,7 +119,7 @@ class DownloadManagerTest(testing.TestCase):
       path = os.path.join(tmpdir_path, filename)
       self.fs.add_file(path)
       dl_result = downloader.DownloadResult(
-          path=utils.as_path(path),
+          path=epath.Path(path),
           url_info=self.dl_results[url],
       )
       return promise.Promise.resolve(dl_result)
@@ -158,6 +160,8 @@ class DownloadManagerTest(testing.TestCase):
     self.fs.__enter__()
     self._make_downloader_mock().start()
     self._make_extractor_mock().start()
+    # Mock `_checksum_paths` (do not load the precomputed checksums)
+    mock.patch.object(checksums_lib, '_checksum_paths', lambda: {}).start()
 
     self.addCleanup(mock.patch.stopall)
 
@@ -166,7 +170,8 @@ class DownloadManagerTest(testing.TestCase):
     self.fs.__exit__(None, None, None)
 
   def assertContainFiles(self, filepaths):
-    self.assertCountEqual(_as_path(list(self.fs.files)), _as_path(filepaths))
+    for path in filepaths:
+      assert tf.io.gfile.exists(path)
 
   def _write_info(self, path, info):
     content = json.dumps(info)
@@ -179,6 +184,7 @@ class DownloadManagerTest(testing.TestCase):
                    extract_dir='/extract_dir',
                    manual_dir='/manual_dir',
                    **kwargs):
+    tf.io.gfile.mkdir('/checksums')
     manager = dm.DownloadManager(
         dataset_name='mnist',
         download_dir=dl_dir,
@@ -241,7 +247,7 @@ class DownloadManagerTest(testing.TestCase):
         'download': b.url,
     })
     expected = {
-        'manual': utils.as_path(a_manual_path),
+        'manual': epath.Path(a_manual_path),
         'download': b.file_path,
     }
     self.assertEqual(downloads, expected)
@@ -527,7 +533,7 @@ class DownloadManagerTest(testing.TestCase):
         register_checksums=False,
         force_download=True,
     )
-    with self.assertRaisesRegexp(ValueError, 'contains a different checksum'):
+    with self.assertRaisesRegex(ValueError, 'contains a different checksum'):
       dl_manager.download(a.url)
 
     # If the url is re-downloaded with the same hash, no error is raised
@@ -555,7 +561,7 @@ class DownloadManagerTest(testing.TestCase):
 
     # Checksums are created and url recorded
     self.assertEqual(
-        self.fs.files['/checksums/checksums.tsv'],
+        self.fs.read_file('/checksums/checksums.tsv'),
         f'{a.url}\t{a.url_info.size}\t{a.url_info.checksum}\t{a.url_info.filename}\n',
     )
 
@@ -596,6 +602,14 @@ class DownloadManagerTest(testing.TestCase):
         _info_path(new_a.file_path),
     ])
 
+  @parameterized.parameters([(1,), (10,), (50,)])
+  def test_max_simultaneous_downloads(self, max_simultaneous_downloads):
+    dl_manager = self._get_manager(
+        max_simultaneous_downloads=max_simultaneous_downloads)
+
+    self.assertEqual(max_simultaneous_downloads,
+                     dl_manager._downloader._executor._max_workers)
+
 
 if __name__ == '__main__':
   testing.test_main()
