@@ -26,15 +26,16 @@ import re
 from typing import Any, ContextManager, Iterable, Iterator, Optional, Tuple, Union
 import urllib
 
+from etils import epath
 import promise
 import requests
-
-import tensorflow as tf
 from tensorflow_datasets.core import units
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.download import checksums as checksums_lib
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 _DRIVE_URL = re.compile(r'^https://drive\.google\.com/')
+MAX_RETRIES = 10
 
 # Response interface. Has `.url` and `.headers` attribute
 Response = Union[requests.Response, urllib.response.addinfourl]
@@ -42,7 +43,7 @@ Response = Union[requests.Response, urllib.response.addinfourl]
 
 @dataclasses.dataclass(eq=False, frozen=True)
 class DownloadResult:
-  path: utils.ReadWritePath
+  path: epath.Path
   url_info: checksums_lib.UrlInfo
 
 
@@ -111,16 +112,23 @@ class _Downloader(object):
 
   Do not instantiate this class directly. Instead, call `get_downloader()`.
   """
+  _DEFAULT_MAX_SIMULTANEOUS_DOWNLOADS = 50
 
-  def __init__(self, max_simultaneous_downloads: int = 50, checksumer=None):
+  def __init__(self,
+               max_simultaneous_downloads: Optional[
+                   int] = _DEFAULT_MAX_SIMULTANEOUS_DOWNLOADS,
+               checksumer=None):
     """Init _Downloader instance.
 
     Args:
-      max_simultaneous_downloads: `int`, max number of simultaneous downloads.
+      max_simultaneous_downloads: `int`, optional max number of simultaneous
+        downloads. If None then it defaults to
+        `self._DEFAULT_MAX_SIMULTANEOUS_DOWNLOADS`.
       checksumer: `hashlib.HASH`. Defaults to `hashlib.sha256`.
     """
     self._executor = concurrent.futures.ThreadPoolExecutor(
-        max_workers=max_simultaneous_downloads)
+        max_workers=max_simultaneous_downloads or
+        self._DEFAULT_MAX_SIMULTANEOUS_DOWNLOADS)
     self._checksumer_cls = checksumer or hashlib.sha256
     self._pbar_url = None
     self._pbar_dl_size = None
@@ -181,7 +189,7 @@ class _Downloader(object):
     self._pbar_dl_size.update_total(url_info.size)
     self._pbar_dl_size.update(url_info.size)
     self._pbar_url.update(1)
-    return DownloadResult(path=utils.as_path(out_path), url_info=url_info)
+    return DownloadResult(path=epath.Path(out_path), url_info=url_info)
 
   def _sync_download(self,
                      url: str,
@@ -237,7 +245,7 @@ class _Downloader(object):
             size_mb %= unit_mb
     self._pbar_url.update(1)
     return DownloadResult(
-        path=utils.as_path(path),
+        path=epath.Path(path),
         url_info=checksums_lib.UrlInfo(
             checksum=checksum.hexdigest(),
             size=utils.Size(size),
@@ -272,8 +280,17 @@ def _open_with_requests(
 ) -> Iterator[Tuple[Response, Iterable[bytes]]]:
   """Open url with request."""
   with requests.Session() as session:
+    retries = requests.packages.urllib3.util.retry.Retry(
+        total=MAX_RETRIES,
+        backoff_factor=0.2,
+        status_forcelist=[500, 502, 503, 504],
+        raise_on_redirect=True,
+        raise_on_status=True)
+    session.mount('http://', requests.adapters.HTTPAdapter(max_retries=retries))
+    session.mount('https://',
+                  requests.adapters.HTTPAdapter(max_retries=retries))
     if _DRIVE_URL.match(url):
-      url = _get_drive_url(url, session)
+      url = _normalize_drive_url(url)
     with session.get(url, stream=True, **kwargs) as response:
       _assert_status(response)
       yield (response, response.iter_content(chunk_size=io.DEFAULT_BUFFER_SIZE))
@@ -292,15 +309,11 @@ def _open_with_urllib(
     )
 
 
-def _get_drive_url(url: str, session: requests.Session) -> str:
-  """Returns url, possibly with confirmation token."""
-  with session.get(url, stream=True) as response:
-    _assert_status(response)
-    for k, v in response.cookies.items():
-      if k.startswith('download_warning'):
-        return url + '&confirm=' + v  # v is the confirm token
-  # No token found, let's try with original URL:
-  return url
+def _normalize_drive_url(url: str) -> str:
+  """Returns Google Drive url with confirmation token."""
+  # This bypasses the "Google Drive can't scan this file for viruses" warning
+  # when dowloading large files.
+  return url + '&confirm=t'
 
 
 def _assert_status(response: requests.Response) -> None:
