@@ -15,22 +15,46 @@
 
 """To serialize Dict or sequence to Example."""
 
+from __future__ import annotations
+
+import abc
 import collections
+import dataclasses
 from typing import Any, Mapping
+
 import numpy as np
 import six
-
-import tensorflow as tf
-
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.features import feature as feature_lib
+from tensorflow_datasets.core.utils import dtype_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 TensorInfo = feature_lib.TensorInfo
 TreeDict = utils.TreeDict
 Shape = utils.Shape
 
 
-class ExampleSerializer(object):
+@dataclasses.dataclass(frozen=True)
+class Serializer(abc.ABC):
+  """Interface of an example serializer."""
+  example_specs: TreeDict[TensorInfo]
+
+  @abc.abstractmethod
+  def serialize_example(self, example: TreeDict[Any]) -> Any:
+    """Serialize the given example.
+
+    Args:
+      example: Nested `dict` containing the input to serialize. The input
+        structure and values dtype/shape must match the `example_specs` provided
+        at construction.
+
+    Returns:
+      the serialized example.
+    """
+    raise NotImplementedError
+
+
+class ExampleSerializer(Serializer):
   """To serialize examples."""
 
   def __init__(self, example_specs: TreeDict[TensorInfo]):
@@ -40,10 +64,10 @@ class ExampleSerializer(object):
       example_specs: Nested `dict` of `tfds.features.TensorInfo`, corresponding
         to the structure of data to write/read.
     """
-    self._example_specs = example_specs
-    self._flat_example_specs = utils.flatten_nest_dict(self._example_specs)
+    super().__init__(example_specs)
+    self._flat_example_specs = utils.flatten_nest_dict(self.example_specs)
 
-  def get_tf_example(self, example) -> tf.train.Example:
+  def get_tf_example(self, example: TreeDict[Any]) -> tf.train.Example:
     """Creates a TF Example for the given example.
 
     Args:
@@ -57,7 +81,7 @@ class ExampleSerializer(object):
     return _dict_to_tf_example(
         utils.flatten_nest_dict(example), self._flat_example_specs)
 
-  def serialize_example(self, example):
+  def serialize_example(self, example: TreeDict[Any]) -> Any:
     """Serialize the given example.
 
     Args:
@@ -93,36 +117,27 @@ def _dict_to_tf_example(
           f"Error while serializing feature `{k}`: `{tensor_info}`: ",
       )
 
-  if tensor_info_dict:
-    # Add the RaggedTensor fields for the nested sequences
-    # Nested sequences are encoded as {'flat_values':, 'row_lengths':}, so need
-    # to flatten the example nested dict again.
-    # Ex:
-    # Input: {'objects/tokens': [[0, 1, 2], [], [3, 4]]}
-    # Output: {
-    #     'objects/tokens/flat_values': [0, 1, 2, 3, 4],
-    #     'objects/tokens/row_lengths_0': [3, 0, 2],
-    # }
-    features = utils.flatten_nest_dict({
-        k: run_with_reraise(_add_ragged_fields, k, example_dict[k], tensor_info)
-        for k, tensor_info in tensor_info_dict.items()
-    })
-    features = {
-        k: run_with_reraise(_item_to_tf_feature, k, item, tensor_info)
-        for k, (item, tensor_info) in features.items()
-    }
-  else:
-    # TODO(epot): The following code is only executed in tests and could be
-    # cleaned-up, as TensorInfo is always passed to _item_to_tf_feature.
-    features = {
-        k: run_with_reraise(_item_to_tf_feature, k, example_data, None)
-        for k, example_data in example_dict.items()
-    }
-
+  # Add the RaggedTensor fields for the nested sequences
+  # Nested sequences are encoded as {'flat_values':, 'row_lengths':}, so need
+  # to flatten the example nested dict again.
+  # Ex:
+  # Input: {'objects/tokens': [[0, 1, 2], [], [3, 4]]}
+  # Output: {
+  #     'objects/tokens/flat_values': [0, 1, 2, 3, 4],
+  #     'objects/tokens/row_lengths_0': [3, 0, 2],
+  # }
+  features = utils.flatten_nest_dict({
+      k: run_with_reraise(_add_ragged_fields, k, example_dict[k], tensor_info)
+      for k, tensor_info in tensor_info_dict.items()
+  })
+  features = {
+      k: run_with_reraise(_item_to_tf_feature, k, item, tensor_info)
+      for k, (item, tensor_info) in features.items()
+  }
   return tf.train.Example(features=tf.train.Features(feature=features))
 
 
-def _is_string(item):
+def _is_string(item) -> bool:
   """Check if the object contains string or bytes."""
   if isinstance(item, (six.binary_type, six.string_types)):
     return True
@@ -134,12 +149,11 @@ def _is_string(item):
   return False
 
 
-def _item_to_np_array(item, dtype: tf.dtypes.DType, numpy_dtype: np.dtype,
-                      shape: Shape) -> np.ndarray:
+def _item_to_np_array(item, dtype: np.dtype, shape: Shape) -> np.ndarray:
   """Single item to a np.array."""
-  result = np.asanyarray(item, dtype=numpy_dtype)
+  result = np.asanyarray(item, dtype=dtype)
   utils.assert_shape_match(result.shape, shape)
-  if utils.is_same_tf_dtype(dtype, tf.string) and not _is_string(item):
+  if dtype_utils.is_string(dtype) and not _is_string(item):
     raise ValueError(
         f"Unsupported value: {result}\nCould not convert to bytes list.")
   return result
@@ -151,20 +165,19 @@ def _item_to_tf_feature(
   v = _item_to_np_array(
       item,
       shape=tensor_info.shape,
-      dtype=tensor_info.dtype,
-      numpy_dtype=tensor_info.numpy_dtype,
+      dtype=tensor_info.np_dtype,
   )
 
   # Convert boolean to integer (tf.train.Example does not support bool)
-  if utils.is_same_np_dtype(v.dtype, np.bool_):
+  if v.dtype == np.bool_:
     v = v.astype(int)
 
   vals = v.flat  # Convert v into a 1-d array (without extra copy)
-  if utils.is_np_sub_dtype(v.dtype, np.integer):
+  if dtype_utils.is_integer(v.dtype):
     return tf.train.Feature(int64_list=tf.train.Int64List(value=vals))
-  elif utils.is_np_sub_dtype(v.dtype, np.floating):
+  elif dtype_utils.is_floating(v.dtype):
     return tf.train.Feature(float_list=tf.train.FloatList(value=vals))
-  elif utils.is_same_tf_dtype(tensor_info.dtype, tf.string):
+  elif dtype_utils.is_string(tensor_info.np_dtype):
     vals = [tf.compat.as_bytes(x) for x in vals]
     return tf.train.Feature(bytes_list=tf.train.BytesList(value=vals))
   else:
@@ -230,7 +243,7 @@ def _add_ragged_fields(example_data, tensor_info: feature_lib.TensorInfo):
     return (example_data, tensor_info)
   # Multiple level sequence:
   else:
-    tensor_info_length = feature_lib.TensorInfo(shape=(None,), dtype=tf.int64)
+    tensor_info_length = feature_lib.TensorInfo(shape=(None,), dtype=np.int64)
     ragged_attr_dict = {
         "ragged_row_lengths_{}".format(i): (length, tensor_info_length)
         for i, length in enumerate(nested_row_lengths)
@@ -247,7 +260,7 @@ def _extract_ragged_attributes(nested_list,
                                tensor_info: feature_lib.TensorInfo):
   """Extract the values for the tf.RaggedTensor __init__.
 
-  This extract the ragged tensor attributes which allow reconstruct the
+  This extracts the ragged tensor attributes which allow to reconstruct the
   ragged tensor with `tf.RaggedTensor.from_nested_row_lengths`.
 
   Args:
@@ -263,7 +276,7 @@ def _extract_ragged_attributes(nested_list,
 
   flat_values = []
   nested_row_lengths = [[] for _ in range(tensor_info.sequence_rank)]
-  # Reccursivelly append to `flat_values`, `nested_row_lengths`
+  # Recursively append to `flat_values`, `nested_row_lengths`
   _fill_ragged_attribute(
       RaggedExtraction(
           nested_list=nested_list,
@@ -275,7 +288,7 @@ def _extract_ragged_attributes(nested_list,
   if not flat_values:  # The full sequence is empty
     flat_values = np.empty(
         shape=(0,) + tensor_info.shape[tensor_info.sequence_rank:],
-        dtype=tensor_info.numpy_dtype,
+        dtype=tensor_info.np_dtype,
     )
   else:  # Otherwise, merge all flat values together, some might be empty
     flat_values = np.stack(flat_values)
@@ -289,7 +302,7 @@ def _fill_ragged_attribute(ext: RaggedExtraction) -> None:
     ext: RaggedExtraction tuple containing the input/outputs
 
   Returns:
-    None, the function mutate instead `ext.nested_row_lengths` and
+    None, the function mutates instead `ext.nested_row_lengths` and
       `ext.flat_values` lists.
   """
   # Register the current sequence length.
@@ -318,8 +331,7 @@ def _fill_ragged_attribute(ext: RaggedExtraction) -> None:
     for item in ext.nested_list:
       item = _item_to_np_array(  # Normalize the item
           item,
-          dtype=ext.tensor_info.dtype,
-          numpy_dtype=ext.tensor_info.numpy_dtype,
+          dtype=ext.tensor_info.np_dtype,
           # We only check the non-ragged shape
           shape=ext.tensor_info.shape[ext.tensor_info.sequence_rank:],
       )
