@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,12 +15,11 @@
 
 """Tests for bounding_boxes."""
 
-import tensorflow.compat.v2 as tf
+import numpy as np
+import tensorflow as tf
 from tensorflow_datasets import testing
 from tensorflow_datasets.core import features
 
-tf.enable_v2_behavior()
-
 
 class BBoxFeatureTest(testing.FeatureExpectationsTestCase):
 
@@ -31,7 +30,6 @@ class BBoxFeatureTest(testing.FeatureExpectationsTestCase):
         shape=(4,),
         dtype=tf.float32,
         tests=[
-            # Numpy array
             testing.FeatureExpectationItem(
                 value=features.BBox(
                     ymin=0.0,
@@ -41,6 +39,16 @@ class BBoxFeatureTest(testing.FeatureExpectationsTestCase):
                 ),
                 expected=[0.0, 0.25, 1.0, 0.75],
             ),
+            testing.FeatureExpectationItem(
+                # 1D numpy array, float32
+                value=np.array([0.0, 0.25, 1.0, 0.75], dtype=np.float32),
+                expected=[0.0, 0.25, 1.0, 0.75],
+            ),
+            testing.FeatureExpectationItem(
+                # 1D numpy array, float64
+                value=np.array([0.0, 0.25, 1.0, 0.75], dtype=np.float64),
+                expected=[0.0, 0.25, 1.0, 0.75],
+            ),
         ],
     )
 
