@@ -15,6 +15,8 @@
 
 """Feature connector."""
 
+from __future__ import annotations
+
 import abc
 import collections
 import dataclasses
@@ -22,19 +24,22 @@ import functools
 import html
 import importlib
 import json
-import os
+import typing
 from typing import Any, Dict, List, Mapping, Optional, Type, TypeVar, Union
 
+from etils import enp
 from etils import epath
 import numpy as np
 import six
-import tensorflow as tf
-
+from tensorflow_datasets.core import constants
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.proto import feature_pb2
+from tensorflow_datasets.core.utils import dtype_utils
 from tensorflow_datasets.core.utils import py_utils
 from tensorflow_datasets.core.utils import tf_utils
+from tensorflow_datasets.core.utils import tree_utils
 from tensorflow_datasets.core.utils import type_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 from google.protobuf import descriptor
 from google.protobuf import json_format
@@ -47,40 +52,42 @@ TreeDict = type_utils.TreeDict
 T = TypeVar('T', bound='FeatureConnector')
 
 # FeatureConnector-like input accepted by `Sequence()`, `Optional()`,...
-FeatureConnectorArg = Union['FeatureConnector',
-                            Dict[str, 'FeatureConnectorArg'], tf.dtypes.DType]  # pytype: disable=not-supported-yet
-
-# Name of the file to output the features information.
-FEATURES_FILENAME = 'features.json'
+if typing.TYPE_CHECKING:
+  FeatureConnectorArg = Union['FeatureConnector',
+                              Dict[str, 'FeatureConnectorArg'], tf.dtypes.DType]
+else:
+  FeatureConnectorArg = Any
 
 
 class TensorInfo(object):
   """Structure containing info on the `tf.Tensor` shape/dtype."""
 
   __slots__ = [
-      'shape', 'dtype', 'numpy_dtype', 'default_value', 'sequence_rank',
-      'dataset_lvl'
+      'shape', '_dtype', 'numpy_dtype', 'default_value', 'sequence_rank',
+      'dataset_lvl', 'np_dtype'
   ]
 
   def __init__(self,
                shape: Shape,
-               dtype: tf.dtypes.DType,
+               dtype: type_utils.TfdsDType,
                default_value=None,
                sequence_rank: Optional[int] = None,
                dataset_lvl: int = 0):
     """Constructor.
 
     Args:
-      shape: `tuple[int]`, shape of the tensor
-      dtype: Tensor dtype
+      shape: `tuple[int]`, shape of the tensor.
+      dtype: Tensor DType that will be converted to NumPy DType.
       default_value: Used for retrocompatibility with previous files if a new
         field is added to provide a default value when reading the file.
       sequence_rank: `int`, Number of `tfds.features.Sequence` dimension.
       dataset_lvl: `int`, if >0, nesting level of a `tfds.features.Dataset`.
     """
     self.shape = tf_utils.convert_to_shape(shape)
-    self.dtype = dtype
-    self.numpy_dtype: np.dtype = dtype.as_numpy_dtype
+    self._dtype = dtype
+    self.np_dtype: np.dtype = dtype_utils.cast_to_numpy(dtype)
+    # For backwards compatibility: now it is named np_dtype.
+    self.numpy_dtype = self.np_dtype
     self.default_value = default_value
     self.sequence_rank = sequence_rank or 0
     self.dataset_lvl = dataset_lvl
@@ -111,21 +118,31 @@ class TensorInfo(object):
     Returns:
       The tf.TensorSpec corresponding to this instance.
     """
+    dtype = self.tf_dtype
+    shape = _to_tensor_shape(self.shape)
     if self.dataset_lvl > 1 or self.sequence_rank > 1:
-      return tf.RaggedTensorSpec(
-          dtype=self.dtype, shape=_to_tensor_shape(self.shape))
-    return tf.TensorSpec(dtype=self.dtype, shape=_to_tensor_shape(self.shape))
+      return tf.RaggedTensorSpec(dtype=dtype, shape=shape)
+    return tf.TensorSpec(dtype=dtype, shape=shape)
+
+  @property
+  def dtype(self) -> TreeDict[type_utils.TfdsDType]:
+    """Return the TensorFlow DType of this TensorInfo."""
+    return self.tf_dtype
+
+  @property
+  def tf_dtype(self) -> TreeDict[tf.dtypes.DType]:
+    return tf.dtypes.as_dtype(self.np_dtype)
 
   def __eq__(self, other):
     """Equality."""
-    return (self.shape == other.shape and self.dtype == other.dtype and
+    return (self.shape == other.shape and self._dtype == other._dtype and
             self.default_value == other.default_value)
 
   def __repr__(self):
     return '{}(shape={}, dtype={})'.format(
         type(self).__name__,
         self.shape,
-        repr(self.dtype),
+        dtype_to_str(self.np_dtype),
     )
 
 
@@ -159,7 +176,8 @@ class CatalogFeatureDocumentation:
   tensor_info: Optional[TensorInfo] = None
 
   def replace(self, **kwargs: Any) -> 'CatalogFeatureDocumentation':
-    """Returns a copy of the `CatalogFeatureDocumentation` with updated attributes."""
+    """Returns a copy of the `CatalogFeatureDocumentation` with updated attributes.
+    """
     return dataclasses.replace(self, **kwargs)
 
 
@@ -179,7 +197,6 @@ class FeatureConnector(object):
 
   The connector can either get raw or dictionary values as input, depending on
   the connector type.
-
   """
 
   # Keep track of all sub-classes.
@@ -230,9 +247,9 @@ class FeatureConnector(object):
 
     ```
     return {
-        'image': tfds.features.TensorInfo(shape=(None,), dtype=tf.uint8),
-        'height': tfds.features.TensorInfo(shape=(), dtype=tf.int32),
-        'width': tfds.features.TensorInfo(shape=(), dtype=tf.int32),
+        'image': tfds.features.TensorInfo(shape=(None,), dtype=np.uint8),
+        'height': tfds.features.TensorInfo(shape=(), dtype=np.int32),
+        'width': tfds.features.TensorInfo(shape=(), dtype=np.int32),
     }
     ```
 
@@ -240,13 +257,12 @@ class FeatureConnector(object):
     directly:
 
     ```
-    return tfds.features.TensorInfo(shape=(256, 256), dtype=tf.uint8)
+    return tfds.features.TensorInfo(shape=(256, 256), dtype=np.uint8)
     ```
 
     Returns:
       tensor_info: Either a dict of `tfds.features.TensorInfo` object, or a
         `tfds.features.TensorInfo`
-
     """
     raise NotImplementedError
 
@@ -257,23 +273,38 @@ class FeatureConnector(object):
     of the dataset. For example, currently this method does not support
     RaggedTensorSpec.
     """
-    return tf.nest.map_structure(lambda ti: ti.to_tensor_spec(),
-                                 self.get_tensor_info())
+    return tree_utils.map_structure(lambda ti: ti.to_tensor_spec(),
+                                    self.get_tensor_info())
 
   @py_utils.memoized_property
   def shape(self):
     """Return the shape (or dict of shape) of this FeatureConnector."""
-    return tf.nest.map_structure(lambda t: t.shape, self.get_tensor_info())
+    return tree_utils.map_structure(lambda t: t.shape, self.get_tensor_info())
 
   @py_utils.memoized_property
   def dtype(self) -> TreeDict[tf.dtypes.DType]:
     """Return the dtype (or dict of dtype) of this FeatureConnector."""
-    return tf.nest.map_structure(lambda t: t.dtype, self.get_tensor_info())
+    return self.tf_dtype
 
+  @py_utils.memoized_property
+  def np_dtype(self) -> TreeDict[np.dtype]:
+    return tree_utils.map_structure(lambda t: t.np_dtype,
+                                    self.get_tensor_info())
+
+  # For backwards compatibility: now it is named np_dtype.
   @py_utils.memoized_property
   def numpy_dtype(self) -> TreeDict[np.dtype]:
-    return tf.nest.map_structure(lambda t: t.numpy_dtype,
-                                 self.get_tensor_info())
+    return self.np_dtype
+
+  @py_utils.memoized_property
+  def tf_dtype(self) -> TreeDict[np.dtype]:
+
+    def convert_to_tensorflow(value):
+      if enp.lazy.is_np_dtype(value):
+        return tf.dtypes.as_dtype(value)
+      return value.tf_dtype
+
+    return tree_utils.map_structure(convert_to_tensorflow, self.np_dtype)
 
   @classmethod
   def cls_from_name(cls, python_class_name: str) -> Type['FeatureConnector']:
@@ -290,10 +321,10 @@ class FeatureConnector(object):
       try:
         # Import to register the FeatureConnector
         importlib.import_module(module_name)
-      except ImportError:
+      except ImportError as exception:
         raise ValueError(
             f'{err_msg}\nCould not import {module_name}. You might have to '
-            'install additional dependencies.')
+            'install additional dependencies.') from exception
 
     feature_class = cls._registered_features.get(python_class_name)
     if feature_class is None:
@@ -504,9 +535,8 @@ class FeatureConnector(object):
     Args:
       root_dir: `path/to/dir` containing the `features.json`
     """
-    with tf.io.gfile.GFile(make_config_path(root_dir), 'w') as f:
-      json_dict = json_format.MessageToDict(self.to_proto())
-      f.write(json.dumps(json_dict, indent=4))
+    json_dict = json_format.MessageToDict(self.to_proto())
+    make_config_path(root_dir).write_text(json.dumps(json_dict, indent=4))
     self.save_metadata(root_dir, feature_name=None)
 
   @classmethod
@@ -525,9 +555,8 @@ class FeatureConnector(object):
     Returns:
       The reconstructed feature instance.
     """
-    with tf.io.gfile.GFile(make_config_path(root_dir)) as f:
-      content = json.loads(f.read())
-      feature = FeatureConnector.from_json(content)
+    content = json.loads(make_config_path(root_dir).read_text())
+    feature = FeatureConnector.from_json(content)
     feature.load_metadata(root_dir, feature_name=None)
     return feature
 
@@ -544,9 +573,9 @@ class FeatureConnector(object):
 
     ```
     return {
-        'image': tfds.features.TensorInfo(shape=(None,), dtype=tf.uint8),
-        'height': tfds.features.TensorInfo(shape=(), dtype=tf.int32),
-        'width': tfds.features.TensorInfo(shape=(), dtype=tf.int32),
+        'image': tfds.features.TensorInfo(shape=(None,), dtype=np.uint8),
+        'height': tfds.features.TensorInfo(shape=(), dtype=np.int32),
+        'width': tfds.features.TensorInfo(shape=(), dtype=np.int32),
     }
     ```
 
@@ -554,7 +583,7 @@ class FeatureConnector(object):
     directly:
 
     ```
-    return tfds.features.TensorInfo(shape=(64, 64), tf.uint8)
+    return tfds.features.TensorInfo(shape=(64, 64), np.uint8)
     ```
 
     If not defined, the retuned values are automatically deduced from the
@@ -562,7 +591,6 @@ class FeatureConnector(object):
 
     Returns:
       features: Either a dict of feature proto object, or a feature proto object
-
     """
     return self.get_tensor_info()
 
@@ -674,9 +702,10 @@ class FeatureConnector(object):
       # `(0,)` -> `(0, None, None, 3)`.
       # Instead, we arbitrarily set unknown shape to `0`:
       # `(0,)` -> `(0, 0, 0, 3)`
+      tf_type = tf.dtypes.as_dtype(self.dtype)
       return tf.cond(
           tf.equal(tf.shape(ex)[0], 0),  # Empty sequence
-          lambda: _make_empty_seq_output(shape=self.shape, dtype=self.dtype),
+          lambda: _make_empty_seq_output(shape=self.shape, dtype=tf_type),
           lambda: decode_map_fn(ex),
       )
 
@@ -832,7 +861,7 @@ class FeatureConnector(object):
     # Ensure ordering of keys by adding them one-by-one
     repr_info = collections.OrderedDict()
     repr_info['shape'] = tensor_info.shape
-    repr_info['dtype'] = repr(tensor_info.dtype)
+    repr_info['dtype'] = dtype_to_str(tensor_info.np_dtype)
     additional_info = self._additional_repr_info()
     for k, v in additional_info.items():
       repr_info[k] = v
@@ -921,9 +950,9 @@ class FeatureConnector(object):
     pass
 
 
-def make_config_path(root_dir: str) -> str:
+def make_config_path(root_dir: epath.PathLike) -> epath.Path:
   """Returns the path to the features config."""
-  return os.path.join(root_dir, FEATURES_FILENAME)
+  return epath.Path(root_dir) / constants.FEATURES_FILENAME
 
 
 def _repr_html(ex) -> str:
@@ -995,7 +1024,8 @@ def _make_empty_seq_output(
 
 
 def to_shape_proto(shape: utils.Shape) -> feature_pb2.Shape:
-  """Converts TFDS shape to Shape proto (-1 is used for unspecified dimensions)."""
+  """Converts TFDS shape to Shape proto (-1 is used for unspecified dimensions).
+  """
   dimensions = []
   for dimension in shape:
     if dimension is None or dimension < 0:
@@ -1018,12 +1048,24 @@ def from_shape_proto(shape: feature_pb2.Shape) -> utils.Shape:
   return [parse_dimension(dimension) for dimension in shape.dimensions]
 
 
-def encode_dtype(dtype: tf.dtypes.DType) -> str:
-  return dtype.name
+_STRING_NAME = 'string'
 
 
-def parse_dtype(dtype: str) -> tf.dtypes.DType:
-  return tf.dtypes.as_dtype(dtype)
+def dtype_to_str(dtype: type_utils.TfdsDType) -> str:
+  np_dtype: np.dtype = dtype_utils.cast_to_numpy(dtype)
+  if np_dtype == np.object_:
+    return _STRING_NAME
+  return np.dtype(np_dtype).name
+
+
+def dtype_from_str(dtype: str) -> type_utils.TfdsDType:
+  if dtype == _STRING_NAME:
+    return np.object_
+  try:
+    np_dtype = np.dtype(dtype)
+    return dtype_utils.cast_to_numpy(np_dtype)
+  except TypeError:
+    return tf.dtypes.as_dtype(dtype)
 
 
 def convert_feature_name_to_filename(
