@@ -18,7 +18,11 @@
 import pathlib
 import sys
 import textwrap
+from unittest import mock
 
+from absl import logging
+from absl.testing import parameterized
+import numpy as np
 import tensorflow as tf
 from tensorflow_datasets import testing
 from tensorflow_datasets.core import features as features_lib
@@ -31,12 +35,12 @@ class AnInputConnector(features_lib.FeatureConnector):
     # With this connector, the way the data is on disk ({'a', 'b'}) do not match
     # the way it is exposed to the user (int64), so we overwrite
     # FeaturesDict.get_tensor_info
-    return features_lib.TensorInfo(shape=(), dtype=tf.int64)
+    return features_lib.TensorInfo(shape=(), dtype=np.int64)
 
   def get_serialized_info(self):
     return {
-        'a': features_lib.TensorInfo(shape=(), dtype=tf.int64),
-        'b': features_lib.TensorInfo(shape=(), dtype=tf.int64),
+        'a': features_lib.TensorInfo(shape=(), dtype=np.int64),
+        'b': features_lib.TensorInfo(shape=(), dtype=np.int64),
     }
 
   def encode_example(self, example_data):
@@ -52,7 +56,7 @@ class AnOutputConnector(features_lib.FeatureConnector):
   """Simple FeatureConnector implementing the based methods used for test."""
 
   def get_tensor_info(self):
-    return features_lib.TensorInfo(shape=(), dtype=tf.float32)
+    return features_lib.TensorInfo(shape=(), dtype=np.float32)
 
   def encode_example(self, example_data):
     return example_data * 10.0
@@ -61,7 +65,8 @@ class AnOutputConnector(features_lib.FeatureConnector):
     return tfexample_data / 10.0
 
 
-class FeatureDictTest(testing.FeatureExpectationsTestCase):
+class FeatureDictTest(parameterized.TestCase,
+                      testing.FeatureExpectationsTestCase):
 
   def test_tensor_info(self):
 
@@ -70,6 +75,11 @@ class FeatureDictTest(testing.FeatureExpectationsTestCase):
         features_lib.TensorInfo(shape=(None, 3), dtype=tf.string),
     )
 
+    self.assertEqual(
+        features_lib.TensorInfo(shape=(None, 3), dtype=np.object_),
+        features_lib.TensorInfo(shape=(None, 3), dtype=np.object_),
+    )
+
     self.assertNotEqual(
         features_lib.TensorInfo(shape=(None, 3), dtype=tf.string),
         features_lib.TensorInfo(shape=(None, 3), dtype=tf.int32),
@@ -80,10 +90,53 @@ class FeatureDictTest(testing.FeatureExpectationsTestCase):
         features_lib.TensorInfo(shape=(5, 3), dtype=tf.string),
     )
 
+    self.assertNotEqual(
+        features_lib.TensorInfo(shape=(2, 3), dtype=np.object_),
+        features_lib.TensorInfo(shape=(5, 3), dtype=np.object_),
+    )
+
     t = features_lib.TensorInfo(shape=(None, 3), dtype=tf.string)
     self.assertEqual(t, features_lib.TensorInfo.copy_from(t))
 
-  def test_fdict(self):
+  @parameterized.parameters(
+      (np.int64, np.str_,
+       mock.Mock(
+           side_effect=Exception(
+               'no warning should be raised when using NumPy types'))),
+      (tf.int64, tf.string, mock.Mock()))
+  def test_tensor_spec(self, image_dtype, metadata_dtype, warning_mock):
+    logging.warning = warning_mock
+    feature = features_lib.FeaturesDict({
+        'input': AnInputConnector(),
+        'output': AnOutputConnector(),
+        'img': {
+            'size': {
+                'height':
+                    features_lib.Tensor(shape=(2, 3), dtype=image_dtype),
+                'width':
+                    features_lib.Tensor(shape=[None, 3], dtype=image_dtype),
+            },
+            'image': features_lib.Image(shape=(28, 28, 1)),
+            'metadata/path': metadata_dtype,
+        }
+    })
+    self.assertAllEqualNested(
+        feature.get_tensor_spec(), {
+            'input': tf.TensorSpec(shape=[], dtype=np.int64),
+            'output': tf.TensorSpec(shape=[], dtype=np.float32),
+            'img': {
+                'size': {
+                    'height': tf.TensorSpec(shape=[2, 3], dtype=np.int64),
+                    'width': tf.TensorSpec(shape=[None, 3], dtype=np.int64),
+                },
+                'image': tf.TensorSpec(shape=[28, 28, 1], dtype=np.uint8),
+                'metadata/path': tf.TensorSpec(shape=[], dtype=np.object_),
+            }
+        })
+
+  @parameterized.parameters((np.int64, np.str_, np.object_),
+                            (tf.int64, tf.string, np.object_))
+  def test_fdict(self, image_dtype, metadata_dtype, output_metadata_dtype):
 
     self.assertFeature(
         feature=features_lib.FeaturesDict({
@@ -91,36 +144,37 @@ class FeatureDictTest(testing.FeatureExpectationsTestCase):
             'output': AnOutputConnector(),
             'img': {
                 'size': {
-                    'height': tf.int64,
-                    'width': tf.int64,
+                    'height': image_dtype,
+                    'width': image_dtype,
                 },
-                'metadata/path': tf.string,
+                'metadata/path': metadata_dtype,
             }
         }),
         serialized_info={
             'input': {
-                'a': features_lib.TensorInfo(shape=(), dtype=tf.int64),
-                'b': features_lib.TensorInfo(shape=(), dtype=tf.int64),
+                'a': features_lib.TensorInfo(shape=(), dtype=np.int64),
+                'b': features_lib.TensorInfo(shape=(), dtype=np.int64),
             },
-            'output': features_lib.TensorInfo(shape=(), dtype=tf.float32),
+            'output': features_lib.TensorInfo(shape=(), dtype=np.float32),
             'img': {
                 'size': {
-                    'height': features_lib.TensorInfo(shape=(), dtype=tf.int64),
-                    'width': features_lib.TensorInfo(shape=(), dtype=tf.int64),
+                    'height': features_lib.TensorInfo(shape=(), dtype=np.int64),
+                    'width': features_lib.TensorInfo(shape=(), dtype=np.int64),
                 },
                 'metadata/path':
-                    features_lib.TensorInfo(shape=(), dtype=tf.string),
+                    features_lib.TensorInfo(
+                        shape=(), dtype=output_metadata_dtype),
             }
         },
         dtype={
-            'input': tf.int64,
-            'output': tf.float32,
+            'input': np.int64,
+            'output': np.float32,
             'img': {
                 'size': {
-                    'height': tf.int64,
-                    'width': tf.int64,
+                    'height': np.int64,
+                    'width': np.int64,
                 },
-                'metadata/path': tf.string,
+                'metadata/path': output_metadata_dtype,
             }
         },
         shape={
@@ -178,13 +232,24 @@ class FeatureDictTest(testing.FeatureExpectationsTestCase):
         ],
     )
 
-  def test_feature_getitem(self):
-    fdict = features_lib.FeaturesDict({
-        'integer': tf.int32,
-        'string': tf.string,
-    })
-    self.assertEqual(fdict['integer'].dtype, tf.int32)
-    self.assertEqual(fdict['string'].dtype, tf.string)
+  @parameterized.parameters(
+      ({
+          'integer': np.int32,
+          'string': np.object_,
+      },),
+      ({
+          'integer': np.int32,
+          'string': np.str_,
+      },),
+      ({
+          'integer': tf.int32,
+          'string': tf.string,
+      },),
+  )
+  def test_feature_getitem(self, features_dict):
+    fdict = features_lib.FeaturesDict(features_dict)
+    self.assertEqual(fdict['integer'].dtype, np.int32)
+    self.assertEqual(fdict['string'].dtype, np.object_)
 
   def test_feature__repr__(self):
 
@@ -202,9 +267,9 @@ class FeatureDictTest(testing.FeatureExpectationsTestCase):
         repr(feature_dict),
         textwrap.dedent("""\
         FeaturesDict({
-            'label': Sequence(ClassLabel(shape=(), dtype=tf.int64, num_classes=2)),
+            'label': Sequence(ClassLabel(shape=(), dtype=int64, num_classes=2)),
             'metadata': Sequence({
-                'frame': Image(shape=(32, 32, 3), dtype=tf.uint8),
+                'frame': Image(shape=(32, 32, 3), dtype=uint8),
             }),
         })"""),
     )
@@ -218,18 +283,19 @@ class FeatureDictTest(testing.FeatureExpectationsTestCase):
       fd.save_metadata(data_dir)
       fd.load_metadata(data_dir)
 
-  def test_repr_tensor(self):
+  @parameterized.parameters((np.int32), (tf.int32))
+  def test_repr_tensor(self, dtype):
 
     # Top level Tensor is printed expanded
     self.assertEqual(
         repr(features_lib.Tensor(shape=(), dtype=tf.int32)),
-        'Tensor(shape=(), dtype=tf.int32)',
+        'Tensor(shape=(), dtype=int32)',
     )
 
     # Sequences colapse tensor repr
     self.assertEqual(
         repr(features_lib.Sequence(tf.int32)),
-        'Sequence(tf.int32)',
+        'Sequence(int32)',
     )
 
     class ChildTensor(features_lib.Tensor):
@@ -238,17 +304,17 @@ class FeatureDictTest(testing.FeatureExpectationsTestCase):
     self.assertEqual(
         repr(
             features_lib.FeaturesDict({
-                'colapsed': features_lib.Tensor(shape=(), dtype=tf.int32),
+                'colapsed': features_lib.Tensor(shape=(), dtype=dtype),
                 # Tensor with defined shape are printed expanded
-                'noncolapsed': features_lib.Tensor(shape=(1,), dtype=tf.int32),
+                'noncolapsed': features_lib.Tensor(shape=(1,), dtype=dtype),
                 # Tensor inherited are expanded
-                'child': ChildTensor(shape=(), dtype=tf.int32),
+                'child': ChildTensor(shape=(), dtype=dtype),
             })),
         textwrap.dedent("""\
         FeaturesDict({
-            'child': ChildTensor(shape=(), dtype=tf.int32),
-            'colapsed': tf.int32,
-            'noncolapsed': Tensor(shape=(1,), dtype=tf.int32),
+            'child': ChildTensor(shape=(), dtype=int32),
+            'colapsed': int32,
+            'noncolapsed': Tensor(shape=(1,), dtype=int32),
         })"""),
     )
 
@@ -260,7 +326,7 @@ def test_custom_feature_connector(tmp_path: pathlib.Path):
   assert feature_qualname not in features_lib.FeatureConnector._registered_features
 
   # Save tfds.feature.Tensor to avoid importing the custom feature connector
-  feature = features_lib.Tensor(shape=(), dtype=tf.int32)
+  feature = features_lib.Tensor(shape=(), dtype=np.int32)
   feature.save_config(tmp_path)
 
   # Update the features.json file to simulate as if the original
