@@ -15,39 +15,46 @@
 
 """Image feature."""
 
+from __future__ import annotations
+
 import dataclasses
 import os
 import tempfile
 from typing import Any, List, Optional, Union
 
+from etils import epath
 import numpy as np
-import tensorflow as tf
-
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.features import feature as feature_lib
 from tensorflow_datasets.core.proto import feature_pb2
+from tensorflow_datasets.core.utils import dtype_utils
 from tensorflow_datasets.core.utils import py_utils
 from tensorflow_datasets.core.utils import type_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 Json = type_utils.Json
 PilImage = Any  # Require lazy deps.
 
+
 _ENCODE_FN = {
-    'png': tf.image.encode_png,
-    'jpeg': tf.image.encode_jpeg,
+    'png': lambda: tf.image.encode_png,
+    'jpeg': lambda: tf.image.encode_jpeg,
 }
 
+
 _ACCEPTABLE_CHANNELS = {
     'png': (0, 1, 2, 3, 4),
     'jpeg': (0, 1, 3),
 }
 
+
 _ACCEPTABLE_DTYPES = {
-    'png': [tf.uint8, tf.uint16, tf.float32],
-    'jpeg': [tf.uint8],
-    None: [tf.uint8, tf.uint16, tf.float32],
+    'png': [np.uint8, np.uint16, np.float32],
+    'jpeg': [np.uint8],
+    None: [np.uint8, np.uint16, np.float32],
 }
 
+
 THUMBNAIL_SIZE = 128
 
 # Framerate for the `tfds.as_dataframe` visualization
@@ -60,9 +67,12 @@ _VISU_FRAMERATE = 10
 class _ImageEncoder:
   """Utils which encode/decode images."""
   shape: utils.Shape
-  dtype: tf.dtypes.DType
-  numpy_dtype: np.dtype
+  dtype: type_utils.TfdsDType
   encoding_format: Optional[str]
+  np_dtype: Optional[np.dtype] = None
+
+  def __post_init__(self):
+    self.np_dtype = dtype_utils.cast_to_numpy(self.dtype)
 
   # TODO(tfds): Should deprecate the TFGraph runner in favor of simpler
   # implementation
@@ -74,7 +84,7 @@ class _ImageEncoder:
     """Convert the given image into a dict convertible to tf example."""
     if isinstance(image_or_path_or_fobj, np.ndarray):
       encoded_image = self._encode_image(image_or_path_or_fobj)
-    elif isinstance(image_or_path_or_fobj, type_utils.PathLikeCls):
+    elif isinstance(image_or_path_or_fobj, epath.PathLikeCls):
       image_or_path_or_fobj = os.fspath(image_or_path_or_fobj)
       with tf.io.gfile.GFile(image_or_path_or_fobj, 'rb') as image_f:
         encoded_image = image_f.read()
@@ -87,11 +97,7 @@ class _ImageEncoder:
 
   def _encode_image(self, np_image: np.ndarray) -> bytes:
     """Returns np_image encoded as jpeg or png."""
-    _validate_np_array(
-        np_image,
-        shape=self.shape,
-        dtype=self.dtype,
-        numpy_dtype=self.numpy_dtype)
+    _validate_np_array(np_image, shape=self.shape, dtype=self.np_dtype)
 
     # When encoding isn't defined, default to PNG.
     # Should we be more strict about explicitly define the encoding (raise
@@ -99,11 +105,13 @@ class _ImageEncoder:
     # It has created subtle issues for imagenet_corrupted: images are read as
     # JPEG images to apply some processing, but final image saved as PNG
     # (default) rather than JPEG.
-    return self._runner.run(_ENCODE_FN[self.encoding_format or 'png'], np_image)
+    return self._runner.run(_ENCODE_FN[self.encoding_format or 'png'](),
+                            np_image)
 
   def decode_image(self, img: tf.Tensor) -> tf.Tensor:
     """Decode the jpeg or png bytes to 3d tensor."""
-    img = tf.image.decode_image(img, channels=self.shape[-1], dtype=self.dtype)
+    tf_dtype = tf.dtypes.as_dtype(self.dtype)
+    img = tf.image.decode_image(img, channels=self.shape[-1], dtype=tf_dtype)
     img.set_shape(self.shape)
     return img
 
@@ -132,8 +140,7 @@ class _FloatImageEncoder(_ImageEncoder):
     self._float_shape = shape
     super().__init__(
         shape=shape[:2] + (4,),
-        dtype=tf.uint8,
-        numpy_dtype=tf.uint8.as_numpy_dtype,
+        dtype=np.uint8,
         encoding_format=encoding_format,
     )
 
@@ -146,7 +153,7 @@ class _FloatImageEncoder(_ImageEncoder):
     return self._encode_image(image_or_path_or_fobj)
 
   def _encode_image(self, np_image: np.ndarray) -> bytes:
-    _validate_np_array(np_image, shape=self._float_shape, dtype=tf.float32)
+    _validate_np_array(np_image, shape=self._float_shape, dtype=np.float32)
     # Bitcast 1 channel float32 -> 4 channels uint8
     np_image = np_image.view(np.uint8)
     np_image = super()._encode_image(np_image)
@@ -202,23 +209,23 @@ class Image(feature_lib.FeatureConnector):
       self,
       *,
       shape: Optional[utils.Shape] = None,
-      dtype: Optional[tf.dtypes.DType] = None,
+      dtype: Optional[type_utils.TfdsDType] = None,
       encoding_format: Optional[str] = None,
       use_colormap: bool = False,
+      doc: feature_lib.DocArg = None,
   ):
     """Construct the connector.
 
     Args:
-      shape: tuple of ints or None, the shape of decoded image.
-        For GIF images: (num_frames, height, width, channels=3). num_frames,
-          height and width can be None.
-        For other images: (height, width, channels). height and width can be
-          None. See `tf.image.encode_*` for doc on channels parameter. Defaults
-          to (None, None, 3).
-      dtype: `tf.uint8` (default), `tf.uint16` or `tf.float32`. * `tf.uint16`
-        requires png encoding_format. * `tf.float32` only supports
+      shape: tuple of ints or None, the shape of decoded image. For GIF images:
+        (num_frames, height, width, channels=3). num_frames, height and width
+        can be None. For other images: (height, width, channels). height and
+        width can be None. See `tf.image.encode_*` for doc on channels
+        parameter. Defaults to (None, None, 3).
+      dtype: `np.uint8` (default), `np.uint16` or `np.float32`. * `np.uint16`
+        requires png encoding_format. * `np.float32` only supports
         single-channel image. Internally float images are bitcasted to
-        4-channels `tf.uint8` and saved as PNG.
+        4-channels `np.uint8` and saved as PNG.
       encoding_format: 'jpeg' or 'png'. Format to serialize `np.ndarray` images
         on disk. If None, encode images as PNG. If image is loaded from
         {bmg,gif,jpeg,png} file, this parameter is ignored, and file original
@@ -226,13 +233,15 @@ class Image(feature_lib.FeatureConnector):
       use_colormap: Only used for gray-scale images. If `True`,
         `tfds.as_dataframe` will display each value in the image with a
         different color.
+      doc: Documentation of this feature (e.g. description).
 
     Raises:
       ValueError: If the shape is invalid
     """
+    super().__init__(doc=doc)
     # Set and validate values
     shape = shape or (None, None, 3)
-    dtype = dtype or tf.uint8
+    dtype = dtype_utils.cast_to_numpy(dtype or np.uint8)
     self._encoding_format = get_and_validate_encoding(encoding_format)
     self._shape = get_and_validate_shape(shape, self._encoding_format)
     self._dtype = get_and_validate_dtype(dtype, self._encoding_format)
@@ -240,7 +249,7 @@ class Image(feature_lib.FeatureConnector):
                                                     self._dtype,
                                                     self._encoding_format)
 
-    if self._dtype == tf.float32:  # Float images encoded as 4-channels uint8
+    if self._dtype == np.float32:  # Float images encoded as 4-channels uint8
       self._image_encoder = _FloatImageEncoder(
           shape=self._shape,
           encoding_format=self._encoding_format,
@@ -249,7 +258,6 @@ class Image(feature_lib.FeatureConnector):
       self._image_encoder = _ImageEncoder(
           shape=self._shape,
           dtype=self._dtype,
-          numpy_dtype=self._dtype.as_numpy_dtype,
           encoding_format=self._encoding_format,
       )
 
@@ -269,7 +277,7 @@ class Image(feature_lib.FeatureConnector):
   @py_utils.memoize()
   def get_serialized_info(self):
     # Only store raw image (includes size).
-    return feature_lib.TensorInfo(shape=(), dtype=tf.string)
+    return feature_lib.TensorInfo(shape=(), dtype=np.object_)
 
   def encode_example(self, image_or_path_or_fobj):
     """Convert the given image into a dict convertible to tf example."""
@@ -305,20 +313,20 @@ class Image(feature_lib.FeatureConnector):
       # For backwards compatibility
       return cls(
           shape=tuple(value['shape']),
-          dtype=tf.dtypes.as_dtype(value['dtype']),
+          dtype=feature_lib.dtype_from_str(value['dtype']),
           encoding_format=value['encoding_format'],
           use_colormap=value.get('use_colormap'),
       )
     return cls(
         shape=feature_lib.from_shape_proto(value.shape),
-        dtype=feature_lib.parse_dtype(value.dtype),
+        dtype=feature_lib.dtype_from_str(value.dtype),
         encoding_format=value.encoding_format or None,
         use_colormap=value.use_colormap)
 
   def to_json_content(self) -> feature_pb2.ImageFeature:
     return feature_pb2.ImageFeature(
         shape=feature_lib.to_shape_proto(self._shape),
-        dtype=feature_lib.encode_dtype(self._dtype),
+        dtype=feature_lib.dtype_to_str(self._dtype),
         encoding_format=self._encoding_format,
         use_colormap=self._use_colormap,
     )
@@ -382,7 +390,7 @@ def _get_repr_html_ffmpeg(images: List[PilImage]) -> str:
         # so allow software encoding
         # '-allow_sw', '1',
     ]
-    video_path = utils.as_path(video_dir) / 'output.mp4'
+    video_path = epath.Path(video_dir) / 'output.mp4'
     ffmpeg_args.append(os.fspath(video_path))
     utils.ffmpeg_run(ffmpeg_args)
     video_str = utils.get_base64(video_path.read_bytes())
@@ -413,7 +421,7 @@ def _get_repr_html_gif(images: List[PilImage]) -> str:
 # Other image utils
 
 
-def get_and_validate_encoding(encoding_format):
+def get_and_validate_encoding(encoding_format: Optional[str]) -> Optional[str]:
   """Update the encoding format."""
   supported = _ENCODE_FN.keys()
   if encoding_format and encoding_format not in supported:
@@ -421,9 +429,8 @@ def get_and_validate_encoding(encoding_format):
   return encoding_format
 
 
-def get_and_validate_dtype(dtype, encoding_format):
+def get_and_validate_dtype(dtype: np.dtype, encoding_format: Optional[str]):
   """Update the dtype."""
-  dtype = tf.as_dtype(dtype)
   acceptable_dtypes = _ACCEPTABLE_DTYPES.get(encoding_format)
   if acceptable_dtypes and dtype not in acceptable_dtypes:
     raise ValueError(f'Acceptable `dtype` for {encoding_format}: '
@@ -443,7 +450,8 @@ def get_and_validate_shape(shape, encoding_format):
   return tuple(shape)
 
 
-def _get_and_validate_colormap(use_colormap, shape, dtype, encoding_format):
+def _get_and_validate_colormap(use_colormap, shape, dtype: np.dtype,
+                               encoding_format):
   """Validate that the given colormap is valid."""
   if use_colormap:
     if encoding_format and encoding_format != 'png':
@@ -452,20 +460,17 @@ def _get_and_validate_colormap(use_colormap, shape, dtype, encoding_format):
     if shape[-1] != 1:
       raise ValueError(
           f'Colormap is only available for gray-scale images. Got: {shape}')
-    if not dtype.is_integer:
+    if not dtype_utils.is_integer(dtype):
       raise ValueError(
-          f'Colormap is only available for interger images. Got: {dtype}')
+          f'Colormap is only available for integer images. Got: {dtype}')
 
   return use_colormap
 
 
-def _validate_np_array(np_array: np.ndarray,
-                       shape: utils.Shape,
-                       dtype: tf.dtypes.DType,
-                       numpy_dtype: Optional[np.dtype] = None) -> None:
+def _validate_np_array(np_array: np.ndarray, shape: utils.Shape,
+                       dtype: np.dtype) -> None:
   """Validate the numpy array match the expected shape/dtype."""
-  numpy_dtype = numpy_dtype or dtype.as_numpy_dtype
-  if np_array.dtype != numpy_dtype:
-    raise ValueError(f'Image dtype should be {numpy_dtype}. '
+  if np_array.dtype != dtype:
+    raise ValueError(f'Image dtype should be {dtype}. '
                      f'Detected: {np_array.dtype}.')
   utils.assert_shape_match(np_array.shape, shape)
