@@ -15,9 +15,11 @@
 
 """Image feature."""
 
+from __future__ import annotations
+
 from typing import Dict, List, Optional, Union
 
-import tensorflow as tf
+from etils import epath
 from tensorflow_datasets.core.features import class_label_feature
 from tensorflow_datasets.core.features import feature as feature_lib
 from tensorflow_datasets.core.features import image_feature
@@ -26,7 +28,7 @@ from tensorflow_datasets.core.utils import type_utils
 
 Json = type_utils.Json
 
-_LabelArg = Union[List[str], type_utils.PathLike, None, int]
+_LabelArg = Union[List[str], epath.PathLike, None, int]
 
 
 class LabeledImage(image_feature.Image):
@@ -42,7 +44,6 @@ class LabeledImage(image_feature.Image):
   builder = tfds.builder('my_dataset')
   builder.info.features['label_image'].names == ['background', 'car', ...]
   ```
-
   """
 
   def __init__(
@@ -50,8 +51,9 @@ class LabeledImage(image_feature.Image):
       *,
       labels: _LabelArg,
       shape: Optional[type_utils.Shape] = None,
-      dtype: Optional[tf.dtypes.DType] = None,
+      dtype: Optional[type_utils.TfdsDType] = None,
       encoding_format: Optional[str] = None,
+      doc: feature_lib.DocArg = None,
   ):
     """Constructor.
 
@@ -70,6 +72,7 @@ class LabeledImage(image_feature.Image):
       shape: Image shape (see `tfds.features.Image.__init__`)
       dtype: Image dtype (see `tfds.features.Image.__init__`)
       encoding_format: 'jpeg' or 'png' (see `tfds.features.Image.__init__`)
+      doc: Documentation of this feature (e.g. description).
     """
     super().__init__(
         # Label images have a single channel
@@ -77,6 +80,7 @@ class LabeledImage(image_feature.Image):
         dtype=dtype,
         encoding_format=encoding_format,
         use_colormap=True,  # LabeledImage always use colormap
+        doc=doc,
     )
     if self.shape[-1] != 1:
       raise ValueError(
@@ -113,7 +117,7 @@ class LabeledImage(image_feature.Image):
       return cls(**value)
     return cls(
         shape=feature_lib.from_shape_proto(value.shape),
-        dtype=feature_lib.parse_dtype(value.dtype),
+        dtype=feature_lib.dtype_from_str(value.dtype),
         encoding_format=value.encoding_format or None,
         labels=value.label.num_classes or None,
     )
@@ -130,7 +134,7 @@ def _labels_to_kwarg(labels: _LabelArg) -> Dict[str, _LabelArg]:
     return {}
   elif isinstance(labels, int):
     kwarg_name = 'num_classes'
-  elif isinstance(labels, type_utils.PathLikeCls):
+  elif isinstance(labels, epath.PathLikeCls):
     kwarg_name = 'names_file'
   elif isinstance(labels, list):
     kwarg_name = 'names'
