@@ -15,17 +15,19 @@
 
 """Sequence feature."""
 
-from typing import Optional
+from __future__ import annotations
+
+from typing import List, Optional, Union
 
 import numpy as np
-import tensorflow as tf
-
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.features import feature as feature_lib
 from tensorflow_datasets.core.features import features_dict
 from tensorflow_datasets.core.features import tensor_feature
 from tensorflow_datasets.core.features import top_level_feature
+from tensorflow_datasets.core.proto import feature_pb2
 from tensorflow_datasets.core.utils import py_utils
+from tensorflow_datasets.core.utils import tree_utils
 from tensorflow_datasets.core.utils import type_utils
 
 Json = type_utils.Json
@@ -81,24 +83,26 @@ class Sequence(top_level_feature.TopLevelFeature):
   At generation time, you can specify a list of features dict, a dict of list
   values or a stacked numpy array. The lists will automatically be distributed
   into their corresponding `FeatureConnector`.
-
   """
 
   def __init__(
       self,
       feature: feature_lib.FeatureConnectorArg,
       length: Optional[int] = None,
+      *,
+      doc: feature_lib.DocArg = None,
   ):
     """Construct a sequence dict.
 
     Args:
       feature: The features to wrap (any feature supported)
       length: `int`, length of the sequence if static and known in advance
+      doc: Documentation of this feature (e.g. description).
     """
     # Convert {} => FeaturesDict, tf.int32 => Tensor(shape=(), dtype=tf.int32)
     self._feature = features_dict.to_feature(feature)
     self._length = length
-    super(Sequence, self).__init__()
+    super(Sequence, self).__init__(doc=doc)
 
   @property
   def feature(self):
@@ -117,14 +121,14 @@ class Sequence(top_level_feature.TopLevelFeature):
     """See base class for details."""
     # Add the additional length dimension to every shape
     tensor_info = self._feature.get_tensor_info()
-    return tf.nest.map_structure(self._add_length_dim, tensor_info)
+    return tree_utils.map_structure(self._add_length_dim, tensor_info)
 
   @py_utils.memoize()
   def get_serialized_info(self):
     """See base class for details."""
     # Add the additional length dimension to every serialized features
     tensor_info = self._feature.get_serialized_info()
-    return tf.nest.map_structure(self._add_length_dim, tensor_info)
+    return tree_utils.map_structure(self._add_length_dim, tensor_info)
 
   def encode_example(self, example_dict):
     # Convert nested dict[list] into list[nested dict]
@@ -138,7 +142,8 @@ class Sequence(top_level_feature.TopLevelFeature):
 
     # Empty sequences return empty arrays
     if not sequence_elements:
-      return tf.nest.map_structure(build_empty_np, self.get_serialized_info())
+      return tree_utils.map_structure(build_empty_np,
+                                      self.get_serialized_info())
 
     # Encode each individual elements
     sequence_elements = [
@@ -198,24 +203,61 @@ class Sequence(top_level_feature.TopLevelFeature):
       inner_feature_repr = inner_feature_repr[len('FeaturesDict('):-len(')')]
     return '{}({})'.format(type(self).__name__, inner_feature_repr)
 
-  @classmethod
-  def from_json_content(cls, value: Json) -> 'Sequence':
-    return cls(
-        feature=feature_lib.FeatureConnector.from_json(value['feature']),
-        length=value['length'])
+  def catalog_documentation(
+      self) -> List[feature_lib.CatalogFeatureDocumentation]:
+    sub_feature_docs = self._feature.catalog_documentation()
 
-  def to_json_content(self) -> Json:
-    return {
-        'feature': self.feature.to_json(),
-        'length': self._length,
-    }
+    # If it's a sequence of a single feature, then we add more details.
+    if len(sub_feature_docs) == 1:
+      sub_feature_doc = sub_feature_docs[0]
+      return [
+          sub_feature_doc.replace(
+              # Embed type of feature in class name, e.g. Sequence(tf.int64)
+              cls_name=f'{type(self).__name__}({sub_feature_doc.cls_name})',
+              tensor_info=self._add_length_dim(sub_feature_doc.tensor_info),
+              description=self._doc.desc or sub_feature_doc.description,
+              value_range=self._doc.value_range or sub_feature_doc.value_range,
+          )
+      ]
+
+    result = []
+    for documentation in sub_feature_docs:
+      if not documentation.name:
+        # Override the nested FeaturesDict to become a Sequence.
+        documentation = documentation.replace(
+            cls_name=type(self).__name__,
+            description=self._doc.desc or documentation.description,
+            value_range=self._doc.value_range or documentation.value_range,
+        )
+      result.append(documentation)
+    return result
+
+  @classmethod
+  def from_json_content(
+      cls,
+      value: Union[Json, feature_pb2.Sequence],
+  ) -> 'Sequence':
+    if isinstance(value, dict):
+      # For backwards compatibility
+      return cls(
+          feature=feature_lib.FeatureConnector.from_json(value['feature']),
+          length=value['length'])
+    return cls(
+        feature=feature_lib.FeatureConnector.from_proto(value.feature),
+        length=None if value.length == -1 else value.length)
+
+  def to_json_content(self) -> feature_pb2.Sequence:
+    return feature_pb2.Sequence(
+        feature=self.feature.to_proto(),
+        length=-1 if self._length is None else self._length,
+    )
 
 
 def build_empty_np(serialized_info: feature_lib.TensorInfo):
   """Build empty sequence with the shape of serialized_info."""
   return np.empty(
       shape=tuple(s if s else 0 for s in serialized_info.shape),
-      dtype=serialized_info.numpy_dtype,
+      dtype=serialized_info.np_dtype,
   )
 
 
