@@ -15,17 +15,20 @@
 
 """Feature connector."""
 
+from __future__ import annotations
+
 import enum
-from typing import Union
+from typing import Optional, Union
 import zlib
 
 import numpy as np
-import tensorflow as tf
-
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.features import feature as feature_lib
 from tensorflow_datasets.core.proto import feature_pb2
+from tensorflow_datasets.core.utils import dtype_utils
 from tensorflow_datasets.core.utils import py_utils
+from tensorflow_datasets.core.utils import type_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 Json = utils.Json
 Shape = utils.Shape
@@ -63,13 +66,16 @@ class Tensor(feature_lib.FeatureConnector):
       self,
       *,
       shape: utils.Shape,
-      dtype: tf.dtypes.DType,
+      dtype: type_utils.TfdsDType,
       # TODO(tfds): Could add an Encoding.AUTO to automatically compress
       # tensors using some heuristic. However, careful about backward
       # compatibility.
       # Would require some `DatasetInfo.api_version = 1` which would be
       # increased when triggering backward-incompatible changes.
       encoding: Union[str, Encoding] = Encoding.NONE,
+      doc: feature_lib.DocArg = None,
+      serialized_dtype: Optional[type_utils.TfdsDType] = None,
+      serialized_shape: Optional[utils.Shape] = None,
   ):
     """Construct a Tensor feature.
 
@@ -78,9 +84,19 @@ class Tensor(feature_lib.FeatureConnector):
       dtype: Tensor dtype
       encoding: Internal encoding. See `tfds.features.Encoding` for available
         values.
+      doc: Documentation of this feature (e.g. description).
+      serialized_dtype: Tensor dtype. Used to validate that serialized examples
+        have this dtype. If `None` then defaults to `dtype`
+      serialized_shape: Tensor shape. Used to validate that serialized examples
+        have this shape. If `None` then defaults to `shape`
     """
+    super().__init__(doc=doc)
     self._shape = tuple(shape)
-    self._dtype = dtype
+    self._dtype = dtype_utils.cast_to_numpy(dtype)
+    self._serialized_dtype = dtype_utils.cast_to_numpy(serialized_dtype or
+                                                       self._dtype)
+    self._serialized_shape = tuple(
+        self._shape if serialized_shape is None else serialized_shape)
     if isinstance(encoding, str):
       encoding = encoding.lower()
     self._encoding = Encoding(encoding)
@@ -88,10 +104,10 @@ class Tensor(feature_lib.FeatureConnector):
     self._encoded_to_bytes = self._encoding != Encoding.NONE
     self._dynamic_shape = self._shape.count(None) > 1
 
-    if self._dtype == tf.string and self._encoded_to_bytes:
+    if dtype_utils.is_string(self._dtype) and self._encoded_to_bytes:
       raise NotImplementedError(
           'tfds.features.Tensor() does not support `encoding=` when '
-          'dtype=tf.string. Please open a PR if you need this feature.')
+          'dtype is string. Please open a PR if you need this feature.')
 
   @py_utils.memoize()
   def get_tensor_info(self) -> feature_lib.TensorInfo:
@@ -102,11 +118,11 @@ class Tensor(feature_lib.FeatureConnector):
   def get_serialized_info(self):
     """See base class for details."""
     if self._encoded_to_bytes:  # Values encoded (stored as bytes)
-      serialized_spec = feature_lib.TensorInfo(shape=(), dtype=tf.string)
+      serialized_spec = feature_lib.TensorInfo(shape=(), dtype=np.object_)
     else:
       serialized_spec = feature_lib.TensorInfo(
-          shape=self._shape,
-          dtype=self._dtype,
+          shape=self._serialized_shape,
+          dtype=self._serialized_dtype,
       )
 
     # Dynamic shape, need an additional field to restore the shape after
@@ -116,7 +132,7 @@ class Tensor(feature_lib.FeatureConnector):
           'shape':
               feature_lib.TensorInfo(
                   shape=(len(self._shape),),
-                  dtype=tf.int32,
+                  dtype=np.int32,
               ),
           'value':
               serialized_spec,
@@ -134,7 +150,12 @@ class Tensor(feature_lib.FeatureConnector):
                        "`Tensor(..., encoding='zlib')` (or 'bytes'). "
                        f'For {self}')
 
-    np_dtype = np.dtype(self.numpy_dtype)
+    np_dtype = self._serialized_dtype
+    if np_dtype == np.bool_ and isinstance(example_data, str):
+      raise TypeError(
+          f'Error encoding: {example_data!r}. {example_data!r} is a string, so '
+          'converting it to `bool` will always output `True`. Please, fix '
+          '`_generate_examples` with a better parsing.')
     if isinstance(example_data, tf.Tensor):
       raise TypeError(
           f'Error encoding: {example_data!r}. `_generate_examples` should '
@@ -149,7 +170,7 @@ class Tensor(feature_lib.FeatureConnector):
     shape = example_data.shape
     if isinstance(shape, tf.TensorShape):
       shape = tuple(shape.as_list())
-    utils.assert_shape_match(shape, self._shape)
+    utils.assert_shape_match(shape, self._serialized_shape)
 
     # Eventually encode the data
     if self._encoded_to_bytes:
@@ -180,7 +201,7 @@ class Tensor(feature_lib.FeatureConnector):
     if self._encoded_to_bytes:
       if self._encoding == Encoding.ZLIB:
         value = tf.io.decode_compressed(value, compression_type='ZLIB')
-      value = tf.io.decode_raw(value, self._dtype)
+      value = tf.io.decode_raw(value, self.tf_dtype)
       value = tf.reshape(value, shape)
 
     return value
@@ -213,20 +234,20 @@ class Tensor(feature_lib.FeatureConnector):
     if isinstance(value, dict):
       return cls(
           shape=tuple(value['shape']),
-          dtype=tf.dtypes.as_dtype(value['dtype']),
+          dtype=feature_lib.dtype_from_str(value['dtype']),
           # Use .get for backward-compatibility
           encoding=value.get('encoding', Encoding.NONE),
       )
     return cls(
         shape=feature_lib.from_shape_proto(value.shape),
-        dtype=feature_lib.parse_dtype(value.dtype),
+        dtype=feature_lib.dtype_from_str(value.dtype),
         encoding=value.encoding or Encoding.NONE,
     )
 
   def to_json_content(self) -> feature_pb2.TensorFeature:
     return feature_pb2.TensorFeature(
         shape=feature_lib.to_shape_proto(self._shape),
-        dtype=feature_lib.encode_dtype(self._dtype),
+        dtype=feature_lib.dtype_to_str(self._dtype),
         encoding=self._encoding.value)
 
 
@@ -234,8 +255,8 @@ def get_inner_feature_repr(feature):
   """Utils which returns the object which should get printed in __repr__.
 
   This is used in container features (Sequence, FeatureDict) to print scalar
-  Tensor in a less verbose way `Sequence(tf.int32)` rather than
-  `Sequence(Tensor(shape=(), dtype=tf.in32))`.
+  Tensor in a less verbose way `Sequence(int32)` rather than
+  `Sequence(Tensor(shape=(), dtype=int32))`.
 
   Args:
     feature: The feature to display
@@ -243,10 +264,10 @@ def get_inner_feature_repr(feature):
   Returns:
     Either the feature or it's inner value.
   """
-  # We only print `tf.int32` rather than `Tensor(shape=(), dtype=tf.int32)`
+  # We only print `int32` rather than `Tensor(shape=(), dtype=int32)`
   # * For the base `Tensor` class (and not subclass).
   # * When shape is scalar (explicit check to avoid trigger when `shape=None`).
   if type(feature) == Tensor and feature.shape == ():  # pylint: disable=unidiomatic-typecheck,g-explicit-bool-comparison
-    return repr(feature.dtype)
+    return feature_lib.dtype_to_str(feature.np_dtype)
   else:
     return repr(feature)
