@@ -21,13 +21,13 @@ import textwrap
 from typing import Union
 
 from absl import logging
-import tensorflow as tf
-
+import numpy as np
 from tensorflow_datasets.core.deprecated import text as text_lib
 from tensorflow_datasets.core.features import feature as feature_lib
 from tensorflow_datasets.core.features import tensor_feature
 from tensorflow_datasets.core.proto import feature_pb2
 from tensorflow_datasets.core.utils import type_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 Json = type_utils.Json
 
@@ -72,7 +72,7 @@ class Text(tensor_feature.Tensor):
       )
     super(Text, self).__init__(
         shape=(None,) if has_encoder else (),
-        dtype=tf.int64 if has_encoder else tf.string,
+        dtype=np.int64 if has_encoder else np.object_,
         doc=doc,
     )
 
