@@ -15,11 +15,13 @@
 
 """Wrapper around FeatureDict to allow better control over decoding."""
 
-from typing import Union
+from __future__ import annotations
+
+from typing import Any, Union
 
-import tensorflow as tf
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.features import feature as feature_lib
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 
 class TopLevelFeature(feature_lib.FeatureConnector):
@@ -41,7 +43,7 @@ class TopLevelFeature(feature_lib.FeatureConnector):
       decoders: Nested dict of `Decoder` objects which allow to customize the
         decoding. The structure should match the feature structure, but only
         customized feature keys need to be present. See [the
-          guide](https://github.com/tensorflow/datasets/tree/master/docs/decode.md)
+          guide](https://github.com/tensorflow/datasets/blob/master/docs/decode.md)
             for more info.
 
     Returns:
@@ -109,6 +111,15 @@ class TopLevelFeature(feature_lib.FeatureConnector):
     example_data = self._example_parser.parse_example(serialized_example)
     return self.decode_example(example_data, decoders=decoders)
 
+  @property
+  def tf_example_spec(self) -> dict[str, Any]:
+    """Returns the `tf.Example` proto structure.
+
+    Returns:
+      The flat `dict[str, tf.io.FixedLenFeature | tf.io.XyzFeature | ...]`
+    """
+    return self._example_parser.flat_feature_specs
+
   @utils.memoized_property
   def _example_parser(self):
     from tensorflow_datasets.core import example_parser  # pytype: disable=import-error  # pylint: disable=g-import-not-at-top
