@@ -15,14 +15,16 @@
 
 """Adapters for file formats."""
 
+from __future__ import annotations
+
 import abc
 import enum
 import os
 from typing import Any, ClassVar, Dict, Iterable, List, Optional, Type, Union
 
 from etils import epath
-import tensorflow as tf
 from tensorflow_datasets.core.utils import type_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 ExamplePositions = List[Any]
 
