@@ -15,6 +15,8 @@
 
 """Access registered datasets."""
 
+from __future__ import annotations
+
 import dataclasses
 import difflib
 import json
@@ -23,10 +25,9 @@ import posixpath
 import re
 import textwrap
 import typing
-from typing import Any, Callable, Dict, Iterable, Iterator, Mapping, List, Optional, Type, Union
+from typing import Any, Callable, Dict, Iterable, Iterator, List, Mapping, Optional, Type, Union
 
 from absl import logging
-import tensorflow as tf
 from tensorflow_datasets.core import community
 from tensorflow_datasets.core import constants
 from tensorflow_datasets.core import dataset_builder
@@ -39,12 +40,14 @@ from tensorflow_datasets.core import registered
 from tensorflow_datasets.core import splits as splits_lib
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core import visibility
+from tensorflow_datasets.core.dataset_builders import huggingface_dataset_builder  # pylint:disable=unused-import
 from tensorflow_datasets.core.utils import error_utils
 from tensorflow_datasets.core.utils import gcs_utils
 from tensorflow_datasets.core.utils import py_utils
 from tensorflow_datasets.core.utils import read_config as read_config_lib
 from tensorflow_datasets.core.utils import type_utils
 from tensorflow_datasets.core.utils import version
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 # pylint: disable=logging-format-interpolation
 
@@ -137,8 +140,16 @@ def builder(
       arguments `a=True` and `b=3` (for builders with configs, it would be
       `'foo_bar/zoo/a=True,b=3'` to use the `'zoo'` config and pass to the
       builder keyword arguments `a=True` and `b=3`).
-    try_gcs: `bool`, if True, tfds.load will see if the dataset exists on the
-      public GCS bucket before building it locally.
+    try_gcs: `bool`, if True, `tfds.load` will see if the dataset exists on the
+      public GCS bucket before building it locally. This is equivalent to
+      passing `data_dir='gs://tfds-data/datasets'`. Warning: `try_gcs` is
+      different than `builder_kwargs.download_config.try_download_gcs`.
+      `try_gcs` (default: False) overrides `data_dir` to be the public GCS
+      bucket. `try_download_gcs` (default: True) allows downloading from GCS
+      while keeping a different `data_dir` than the public GCS bucket.  So, to
+      fully bypass GCS, please use `try_gcs=False` and
+      `download_and_prepare_kwargs={'download_config':
+      tfds.core.download.DownloadConfig(try_download_gcs=False)})`.
     **builder_kwargs: `dict` of keyword arguments passed to the
       `tfds.core.DatasetBuilder`.
 
@@ -164,6 +175,9 @@ def builder(
           'explicitly set')
     builder_kwargs['data_dir'] = gcs_utils.gcs_path('datasets')
   if name.namespace:
+    if name.namespace == 'huggingface':
+      return huggingface_dataset_builder.builder(
+          name=name.name, **builder_kwargs)
     if (visibility.DatasetType.COMMUNITY_PUBLIC.is_available() and
         community.community_register.has_namespace(name.namespace)):
       return community.community_register.builder(name=name, **builder_kwargs)
@@ -330,6 +344,7 @@ class DatasetCollectionLoader:
     # in the same order of the splits.
     elif isinstance(load_output, list):
       for split_name, d in zip(loader_kwargs['split'], load_output):
+        assert isinstance(split_name, str)
         if isinstance(d, tuple):
           ds, _ = d
           loaded_datasets[split_name] = ds
@@ -551,8 +566,16 @@ def load(
       cache_dir and manual_dir will automatically be deduced from data_dir.
     as_dataset_kwargs: `dict` (optional), keyword arguments passed to
       `tfds.core.DatasetBuilder.as_dataset`.
-    try_gcs: `bool`, if True, tfds.load will see if the dataset exists on the
-      public GCS bucket before building it locally.
+    try_gcs: `bool`, if True, `tfds.load` will see if the dataset exists on the
+      public GCS bucket before building it locally. This is equivalent to
+      passing `data_dir='gs://tfds-data/datasets'`. Warning: `try_gcs` is
+      different than `builder_kwargs.download_config.try_download_gcs`.
+      `try_gcs` (default: False) overrides `data_dir` to be the public GCS
+      bucket. `try_download_gcs` (default: True) allows downloading from GCS
+      while keeping a different `data_dir` than the public GCS bucket.  So, to
+      fully bypass GCS, please use `try_gcs=False` and
+      `download_and_prepare_kwargs={'download_config':
+      tfds.core.download.DownloadConfig(try_download_gcs=False)})`.
 
   Returns:
     ds: `tf.data.Dataset`, the dataset requested, or if `split` is None, a
