@@ -33,7 +33,7 @@ _NAME_CLASS_REG = re.compile(r'^' + _NAME_CLASS + r'$')
 # Regex matching 'dataset/config:1.*.*/arg=123'
 _NAME_REG = re.compile(r'^'
                        r'(?P<dataset_name>([\w\-]+:)?' + _NAME_CLASS + r')'
-                       r'(/(?P<config>[\w\-\.]+))?'
+                       r'(/(?P<config>[\w\+\-\.]+))?'
                        r'(:(?P<version>(\d+|\*)(\.(\d+|\*)){2}))?'
                        r'(/(?P<kwargs>(\w+=\w+)(,\w+=[^,]+)*))?'
                        r'$')
@@ -183,29 +183,31 @@ def _dataset_name_and_kwargs_from_name_str(
     py_utils.reraise(e, prefix=err_msg)  # pytype: disable=bad-return-type
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(order=True)
 class DatasetReference:
   """Reference to a dataset.
 
   Attributes:
     dataset_name: name of the dataset.
+    namespace: optional namespace in which this dataset belongs.
+    config: optional config to be used in the dataset.
     version: version of the dataset to be used. If `None`, the latest version
       will be loaded. An error is raised if the specified version cannot be
       provided.
+    data_dir: Optional data dir where this dataset is located. If None, defaults
+      to the value of the environment variable TFDS_DATA_DIR, if set, otherwise
     split_mapping: mapping between split names. If the `DatasetCollection` wants
       to use different split names than the source datasets, then this mapping
       can be used. For example, if the collection uses the split `valid`, but
       this dataset uses the split `validation`, then the `split_mapping` should
       be `{'validation': 'valid'}`.
-    config: optional config to be used in the dataset.
-    data_dir: Optional data dir where this dataset is located. If None, defaults
-      to the value of the environment variable TFDS_DATA_DIR, if set, otherwise
   """
   dataset_name: str
-  version: Union[None, str, version_lib.Version] = None
-  split_mapping: Optional[Mapping[str, str]] = None
+  namespace: Optional[str] = None
   config: Optional[str] = None
+  version: Union[None, str, version_lib.Version] = None
   data_dir: Union[None, str, os.PathLike] = None  # pylint: disable=g-bare-generic
+  split_mapping: Optional[Mapping[str, str]] = None
 
   def __post_init__(self):
     if isinstance(self.version, str):
@@ -223,6 +225,8 @@ class DatasetReference:
       The TFDS name of the `DatasetReference`.
     """
     dataset_name = self.dataset_name
+    if self.namespace:
+      dataset_name = f'{self.namespace}:{dataset_name}'
     if self.config:
       dataset_name += f'/{self.config}'
     if self.version and include_version:
@@ -234,6 +238,38 @@ class DatasetReference:
       return self.split_mapping.get(split, split)
     return split
 
+  def dataset_dir(
+      self,
+      data_dir: Optional[epath.PathLike] = None,
+  ) -> epath.Path:
+    """Returns the path where the data of this dataset lives.
+
+    Example: `/my_data_dir/datasets/c4/en/3.0.0`.
+
+    Arguments:
+      data_dir: optional path where this dataset is stored. If not specified,
+        then it uses the `data_dir` specified in this dataset reference. If that
+        is not specified either, then a `ValueError` is raised.
+
+    Returns:
+      the path where the data of this dataset lives.
+    """
+    data_dir = data_dir or self.data_dir
+    if data_dir is None:
+      raise ValueError('No data dir was specified!')
+    dataset_dir: epath.Path = epath.Path(data_dir) / self.dataset_name
+    if self.config:
+      dataset_dir = dataset_dir / self.config
+    if self.version is None:
+      raise ValueError(
+          "Version wasn't specified and is needed to get the dataset dir!")
+    dataset_dir = dataset_dir / str(self.version)
+    return dataset_dir
+
+  def replace(self, **kwargs: Any) -> 'DatasetReference':
+    """Returns a copy with updated attributes."""
+    return dataclasses.replace(self, **kwargs)
+
   @classmethod
   def from_tfds_name(
       cls,
@@ -248,6 +284,7 @@ class DatasetReference:
     config = builder_kwargs.get('config')
     return cls(
         dataset_name=parsed_name.name,
+        namespace=parsed_name.namespace,
         version=version,
         config=config,
         split_mapping=split_mapping,
@@ -491,7 +528,8 @@ class ShardedFileTemplate:
       shard_index: int,
       num_shards: Optional[int],
   ) -> epath.Path:
-    """Returns the filename (including full path if `data_dir` is set) for the given shard."""
+    """Returns the filename (including full path if `data_dir` is set) for the given shard.
+    """
     return self.data_dir / self.relative_filepath(
         shard_index=shard_index, num_shards=num_shards)
 
@@ -643,6 +681,10 @@ class FilenameInfo:
         filetype_suffix=self.filetype_suffix,
     )
 
+  def replace(self, **kwargs: Any) -> 'FilenameInfo':
+    """Returns a copy with updated attributes."""
+    return dataclasses.replace(self, **kwargs)
+
   @classmethod
   def from_str(
       cls,
