@@ -20,7 +20,6 @@ import typing
 from typing import Any, List, Optional, Tuple, Type
 
 from etils import epath
-import tensorflow as tf
 from tensorflow_datasets.core import dataset_builder
 from tensorflow_datasets.core import dataset_info
 from tensorflow_datasets.core import logging as tfds_logging
@@ -33,6 +32,7 @@ from tensorflow_datasets.core.proto import dataset_info_pb2
 from tensorflow_datasets.core.utils import error_utils
 from tensorflow_datasets.core.utils import file_utils
 from tensorflow_datasets.core.utils import version as version_lib
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 
 class ReadOnlyBuilder(
@@ -127,7 +127,7 @@ def builder_from_directory(
       e.g. `~/tensorflow_datasets/mnist/3.0.0/`).
 
   Returns:
-    builder: `tf.core.DatasetBuilder`, builder for dataset at the given path.
+    builder: `tfds.core.DatasetBuilder`, builder for dataset at the given path.
   """
   return ReadOnlyBuilder(builder_dir=builder_dir)
 
@@ -212,7 +212,7 @@ def builder_from_metadata(
       requested dataset.
 
   Returns:
-    builder: `tf.core.DatasetBuilder`, builder for dataset at the given path.
+    builder: `tfds.core.DatasetBuilder`, builder for dataset at the given path.
   """
   return ReadOnlyBuilder(builder_dir=builder_dir, info_proto=info_proto)
 
@@ -354,7 +354,7 @@ def _get_dataset_dir(
 
 def _contains_dataset(dataset_dir: epath.PathLike) -> bool:
   try:
-    return tf.io.gfile.exists(feature_lib.make_config_path(dataset_dir))
+    return feature_lib.make_config_path(dataset_dir).exists()
   except (OSError, tf.errors.PermissionDeniedError):
     return False
 
@@ -394,7 +394,8 @@ def _find_builder_dir_single_dir(
   # Dataset wasn't found, try to find a suitable available version.
   found_version_str = _get_version_str(
       builder_dir, config_name=config_name, requested_version=version_str)
-  if found_version_str and found_version_str != version_str:
+  if (found_version_str and
+      (version_str is None or found_version_str != version_str)):
     return _find_builder_dir_single_dir(
         builder_name=builder_name,
         data_dir=data_dir,
@@ -404,7 +405,7 @@ def _find_builder_dir_single_dir(
   # If no builder found, we populate the error_context with useful information
   # and return None.
   error_utils.add_context(('No builder could be found in the directory: '
-                           f'{builder_name} for the builder: {builder_name}.'))
+                           f'{data_dir} for the builder: {builder_name}.'))
   return None
 
 
