@@ -15,13 +15,15 @@
 
 """Defined Reader and ReadInstruction to read tfrecord files."""
 
+from __future__ import annotations
+
 import functools
 import os
+import re
 from typing import Any, Callable, List, NamedTuple, Optional, Sequence
 
 from absl import logging
 import numpy as np
-import tensorflow as tf
 from tensorflow_datasets.core import example_parser
 from tensorflow_datasets.core import file_adapters
 from tensorflow_datasets.core import splits as splits_lib
@@ -30,6 +32,8 @@ from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.utils import file_utils
 from tensorflow_datasets.core.utils import read_config as read_config_lib
 from tensorflow_datasets.core.utils import shard_utils
+from tensorflow_datasets.core.utils import tree_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 Tree = utils.Tree
 TreeDict = utils.TreeDict
@@ -46,10 +50,11 @@ class _IdExample(NamedTuple):
 
 
 class _Instruction(NamedTuple):
-  filepath: Any  # tf.string '/path/to/../train.tfrecord'
-  filename: Any  # tf.string 'train.tfrecord'
-  skip: Any  #  tf.int64
-  take: Any  # tf.int64
+  filepath: tf.string  # e.g. '/path/to/../train.tfrecord'
+  filename: tf.string  # e.g. 'train.tfrecord'
+  tfds_id_prefix: tf.string  # e.g. 'train.tfrecord' or 'folder1/train.tfrecord'
+  skip: tf.int64
+  take: tf.int64
 
 
 def _get_dataset_from_filename(
@@ -68,7 +73,7 @@ def _get_dataset_from_filename(
     ds = ds.take(instruction.take)
   if add_tfds_id:  # For each example, generate a unique id.
     id_ds = _make_id_dataset(
-        filename=instruction.filename,
+        filename=instruction.tfds_id_prefix,
         start_index=instruction.skip if do_skip else 0)
     ds = tf.data.Dataset.zip(_IdExample(id=id_ds, example=ds))
   return ds
@@ -112,6 +117,39 @@ def _decode_with_id(
   return decoded_ex
 
 
+def _get_tfds_id_prefixes(
+    file_instructions: Sequence[shard_utils.FileInstruction],
+) -> dict[str, str]:
+  """Returns the tfds id prefix per filename.
+
+  If the file instructions are for files in different directories, then the TFDS
+  id prefix includes part of the folder. This is to avoid duplicate TFDS ids
+  when different directories contain the same filenames. For example, if the
+  file instructions contain file paths `['/a/b/c', '/x/y/c']`, then this returns
+  `{'/a/b/c': 'b/c', '/x/y/c': 'y/c'}`.
+
+  Arguments:
+    file_instructions: the file instructions for which to get TFDS id prefixes.
+
+  Returns:
+    the TFDS id prefix per filename.
+  """
+  dirnames = set(i.dirname() for i in file_instructions)
+  reversed_dirnames = [d[::-1] for d in dirnames]
+  common_suffix = os.path.commonprefix(reversed_dirnames)[::-1]
+
+  def get_tfds_id(instruction: shard_utils.FileInstruction) -> str:
+    # Return the file name when all files are in the same directory.
+    if len(dirnames) == 1:
+      return instruction.basename()
+    # Otherwise, return the relative path starting from the deepest folder that
+    # differs.
+    return re.sub(rf'.*\/([^/]+{common_suffix}/[^/]+)', r'\1',
+                  instruction.filename)
+
+  return {fi.filename: get_tfds_id(fi) for fi in file_instructions}
+
+
 def _read_files(
     file_instructions: Sequence[shard_utils.FileInstruction],
     read_config: read_config_lib.ReadConfig,
@@ -135,6 +173,33 @@ def _read_files(
   Returns:
     The dataset object.
   """
+
+  def assert_cardinality_and_apply_options(ds):
+    # If the number of examples read in the tf-record is known, we forward
+    # the information to the tf.data.Dataset object.
+    # Check the `tf.data.experimental` for backward compatibility with TF <= 2.1
+    if (read_config.assert_cardinality and
+        not read_config.input_context and  # TODO(epot): Restore cardinality
+        hasattr(tf.data.experimental, 'assert_cardinality')):
+      # TODO(b/154963426): Replace by per-shard cardinality (warning if
+      # `experimental_interleave_sort_fn` is set).
+      cardinality = sum(f.num_examples for f in file_instructions)
+      ds = ds.apply(tf.data.experimental.assert_cardinality(cardinality))
+
+    return ds.with_options(read_config.options)  # Additional users options
+
+  def validate_input_context():
+    if read_config.input_context.num_input_pipelines > 1 and len(
+        file_instructions) < read_config.input_context.num_input_pipelines:
+      raise ValueError(
+          'Cannot shard the pipeline with given `input_context`.'
+          '`num_shards={}` but `num_input_pipelines={}`. This means that some '
+          'workers won\'t read any data. To shard the data, you may want to '
+          'use the subsplit API instead: '
+          'https://www.tensorflow.org/datasets/splits'.format(
+              len(file_instructions),
+              read_config.input_context.num_input_pipelines))
+
   # Eventually apply a transformation to the instruction function.
   # This allow the user to have direct control over the interleave order.
   if read_config.experimental_interleave_sort_fn is not None:
@@ -144,10 +209,13 @@ def _read_files(
   do_skip = any(f.skip > 0 for f in file_instructions)
   do_take = any(f.take > -1 for f in file_instructions)
 
+  tfds_id_prefixes = _get_tfds_id_prefixes(file_instructions)
+
   # Transpose the list[dict] into dict[list]
   tensor_inputs = _Instruction(
       filename=[os.path.basename(i.filename) for i in file_instructions],
       filepath=[os.fspath(i.filename) for i in file_instructions],
+      tfds_id_prefix=[tfds_id_prefixes[i.filename] for i in file_instructions],
       # skip/take need to be converted to int64 explicitly
       skip=np.array([i.skip for i in file_instructions], dtype=np.int64),
       take=np.array([i.take for i in file_instructions], dtype=np.int64),
@@ -173,17 +241,8 @@ def _read_files(
   # On distributed environments, we can shard per-file if a
   # `tf.distribute.InputContext` object is provided (e.g. from
   # `experimental_distribute_datasets_from_function`)
-  if (read_config.input_context and
-      read_config.input_context.num_input_pipelines > 1):
-    if len(file_instructions) < read_config.input_context.num_input_pipelines:
-      raise ValueError(
-          'Cannot shard the pipeline with given `input_context`.'
-          '`num_shards={}` but `num_input_pipelines={}`. '
-          'This means that some workers won\'t read any data. '
-          'To shard the data, you may want to use the subsplit API '
-          'instead: https://www.tensorflow.org/datasets/splits'.format(
-              len(file_instructions),
-              read_config.input_context.num_input_pipelines))
+  if read_config.input_context:
+    validate_input_context()
     instruction_ds = instruction_ds.shard(
         num_shards=read_config.input_context.num_input_pipelines,
         index=read_config.input_context.input_pipeline_id,
@@ -220,19 +279,7 @@ def _read_files(
       deterministic=deterministic,
   )
 
-  # If the number of examples read in the tf-record is known, we forward
-  # the information to the tf.data.Dataset object.
-  # Check the `tf.data.experimental` for backward compatibility with TF <= 2.1
-  if (read_config.assert_cardinality and
-      not read_config.input_context and  # TODO(epot): Restore cardinality
-      hasattr(tf.data.experimental, 'assert_cardinality')):
-    # TODO(b/154963426): Replace by per-shard cardinality (warning if
-    # `experimental_interleave_sort_fn` is set).
-    cardinality = sum(f.num_examples for f in file_instructions)
-    ds = ds.apply(tf.data.experimental.assert_cardinality(cardinality))
-
-  ds = ds.with_options(read_config.options)  # Additional users options
-  return ds
+  return assert_cardinality_and_apply_options(ds)
 
 
 def _get_default_interleave_cycle_length(disable_shuffling: bool) -> int:
@@ -341,7 +388,7 @@ class Reader(object):
           decode_fn=decode_fn,
       )
 
-    return tf.nest.map_structure(_read_instruction_to_ds, instructions)
+    return tree_utils.map_structure(_read_instruction_to_ds, instructions)
 
   def read_files(
       self,
@@ -356,8 +403,8 @@ class Reader(object):
 
     Args:
       file_instructions: The files information. The filenames contains the
-        relative path, not absolute.
-        skip/take indicates which example read in the shard: `ds.skip().take()`
+        relative path, not absolute. skip/take indicates which example read in
+        the shard: `ds.skip().take()`
       read_config: The input pipeline options
       shuffle_files: If True, input files are shuffled before being read.
       disable_shuffling: Specifies if the dataset being read has shuffling
