@@ -22,10 +22,10 @@ from typing import Iterator, Optional
 import uuid
 
 import six
-import tensorflow as tf
-
 from tensorflow_datasets.core import hashing
+from tensorflow_datasets.core.utils import file_utils
 from tensorflow_datasets.core.utils import type_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 # Approximately how much data to store in memory before writing to disk.
 # If the amount of data to shuffle is < MAX_MEM_BUFFER_SIZE, no intermediary
@@ -76,7 +76,8 @@ def get_bucket_number(
   # We purposely do not use modulo (%) to keep global order across shards.
   # floor(key * num_buckets / HKEYS_NUMBER), with HKEYS_NUMBER = 2**HKEY_SIZE.
   max_hkey = max_hkey or 2**HKEY_SIZE
-  return math.trunc((hkey * num_buckets) / max_hkey)
+  # Make sure that we do not return `bucket_number`.
+  return min(math.trunc((hkey * num_buckets) / max_hkey), num_buckets - 1)
 
 
 class _Bucket(object):
@@ -124,7 +125,7 @@ class _Bucket(object):
       data (binary): the data.
     """
     if not self._fobj:
-      tf.io.gfile.makedirs(os.path.dirname(self._path))
+      file_utils.makedirs_cached(os.path.dirname(self._path))
       self._fobj = tf.io.gfile.GFile(self._path, mode='wb')
     data_size = len(data)
     self._fobj.write(_hkey_to_bytes(key))
