@@ -24,6 +24,9 @@ import typing
 from typing import Any, Callable, Dict, Iterable, Iterator, List, Optional, Tuple, Union
 
 from absl import logging
+import click
+import psutil
+from tensorflow_datasets.core import example_serializer
 from tensorflow_datasets.core import features as features_lib
 from tensorflow_datasets.core import file_adapters
 from tensorflow_datasets.core import lazy_imports_lib
@@ -31,6 +34,7 @@ from tensorflow_datasets.core import naming
 from tensorflow_datasets.core import splits as splits_lib
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core import writer as writer_lib
+from tensorflow_datasets.core.utils import shard_utils
 
 if typing.TYPE_CHECKING:
   import apache_beam as beam  # pytype: disable=import-error
@@ -65,7 +69,7 @@ class SplitGeneratorLegacy:
       the builder.
   """
   name: str
-  gen_kwargs: Dict[str, Any]
+  gen_kwargs: Optional[Dict[str, Any]] = dataclasses.field(default_factory=dict)
 
 
 class _SplitInfoFuture:
@@ -123,13 +127,16 @@ class SplitBuilder:
       *,
       split_dict: splits_lib.SplitDict,  # Used for precomputed nb of examples
       features: features_lib.FeatureConnector,
+      dataset_size: utils.Size,
       beam_options: Optional['beam.options.pipeline_options.PipelineOptions'],
       beam_runner: Optional['beam.runners.PipelineRunner'],
       max_examples_per_split: Optional[int],
       file_format: file_adapters.FileFormat = file_adapters.DEFAULT_FILE_FORMAT,
+      shard_config: Optional[shard_utils.ShardConfig] = None,
   ):
     self._split_dict = split_dict
     self._features = features
+    self._dataset_size = dataset_size
     self._max_examples_per_split = max_examples_per_split
 
     self._in_contextmanager: bool = False
@@ -137,6 +144,7 @@ class SplitBuilder:
     self._beam_runner = beam_runner
     self._beam_pipeline: Optional['beam.Pipeline'] = None
     self._file_format = file_format
+    self._shard_config = shard_config
 
   @contextlib.contextmanager
   def maybe_beam_pipeline(self) -> Iterator[PipelineProxy]:
@@ -225,6 +233,14 @@ class SplitBuilder:
           """)
       print_fn(msg)
 
+      total_memory = psutil.virtual_memory().total
+      if self._dataset_size >= total_memory:
+        if not click.confirm(
+            f'The dataset is {self._dataset_size} in size, but your machine has'
+            f' only {utils.Size(total_memory)} of memory. Continue?',
+            default=True):
+          sys.exit(1)
+
     beam_options = (
         self._beam_options or beam.options.pipeline_options.PipelineOptions())
     # Beam type checking assumes transforms multiple outputs are of same type,
@@ -361,12 +377,14 @@ class SplitBuilder:
         total_num_examples = None
 
     writer = writer_lib.Writer(
-        example_specs=self._features.get_serialized_info(),
+        serializer=example_serializer.ExampleSerializer(
+            self._features.get_serialized_info()),
         filename_template=filename_template,
         hash_salt=split_name,
         disable_shuffling=disable_shuffling,
         # TODO(weide) remove this because it's already in filename_template?
         file_format=self._file_format,
+        shard_config=self._shard_config,
     )
     for key, example in utils.tqdm(
         generator,
@@ -402,11 +420,13 @@ class SplitBuilder:
     beam = lazy_imports_lib.lazy_imports.apache_beam
 
     beam_writer = writer_lib.BeamWriter(
-        example_specs=self._features.get_serialized_info(),
+        serializer=example_serializer.ExampleSerializer(
+            self._features.get_serialized_info()),
         filename_template=filename_template,
         hash_salt=split_name,
         disable_shuffling=disable_shuffling,
         file_format=self._file_format,
+        shard_config=self._shard_config,
     )
 
     def _encode_example(key_ex, encode_fn=self._features.encode_example):
