@@ -16,10 +16,19 @@
 """Tests for subsplits_utils."""
 
 import pytest
+from tensorflow_datasets.core import naming
 from tensorflow_datasets.core import splits as splits_lib
 from tensorflow_datasets.core import subsplits_utils
 
 
+def _filename_template(split: str) -> naming.ShardedFileTemplate:
+  return naming.ShardedFileTemplate(
+      dataset_name='mnist',
+      split=split,
+      filetype_suffix='tfrecord',
+      data_dir='/path')
+
+
 @pytest.mark.parametrize(
     'num_examples, n, drop_remainder, expected',
     [
@@ -38,16 +47,14 @@ from tensorflow_datasets.core import subsplits_utils
     ],
 )
 def test_even_splits(num_examples, n, drop_remainder, expected):
-  split_infos = splits_lib.SplitDict(
-      [
-          splits_lib.SplitInfo(
-              name='train',
-              shard_lengths=[num_examples],
-              num_bytes=0,
-          ),
-      ],
-      dataset_name='mnist',
-  )
+  split_infos = splits_lib.SplitDict(split_infos=[
+      splits_lib.SplitInfo(
+          name='train',
+          shard_lengths=[num_examples],
+          num_bytes=0,
+          filename_template=_filename_template('train'),
+      ),
+  ])
 
   subsplits = subsplits_utils.even_splits(
       'train', n, drop_remainder=drop_remainder)
@@ -60,21 +67,20 @@ def test_even_splits(num_examples, n, drop_remainder, expected):
 
 
 def test_even_splits_subsplit():
-  split_infos = splits_lib.SplitDict(
-      [
-          splits_lib.SplitInfo(
-              name='train',
-              shard_lengths=[2, 3, 2, 3],  # 10
-              num_bytes=0,
-          ),
-          splits_lib.SplitInfo(
-              name='test',
-              shard_lengths=[8],
-              num_bytes=0,
-          ),
-      ],
-      dataset_name='mnist',
-  )
+  split_infos = splits_lib.SplitDict(split_infos=[
+      splits_lib.SplitInfo(
+          name='train',
+          shard_lengths=[2, 3, 2, 3],  # 10
+          num_bytes=0,
+          filename_template=_filename_template('train'),
+      ),
+      splits_lib.SplitInfo(
+          name='test',
+          shard_lengths=[8],
+          num_bytes=0,
+          filename_template=_filename_template('test'),
+      ),
+  ])
 
   # Test to split multiple splits
   subsplits = subsplits_utils.even_splits('train+test[50%:]', 3)
@@ -95,26 +101,26 @@ def test_even_splits_subsplit():
 def test_even_splits_add():
   # Compatibility of even_splits with other splits
 
-  split_infos = splits_lib.SplitDict(
-      [
-          splits_lib.SplitInfo(
-              name='train',
-              shard_lengths=[2, 3, 2, 3],  # 10
-              num_bytes=0,
-          ),
-          splits_lib.SplitInfo(
-              name='test',
-              shard_lengths=[8],
-              num_bytes=0,
-          ),
-          splits_lib.SplitInfo(
-              name='validation',
-              shard_lengths=[8],
-              num_bytes=0,
-          ),
-      ],
-      dataset_name='mnist',
-  )
+  split_infos = splits_lib.SplitDict(split_infos=[
+      splits_lib.SplitInfo(
+          name='train',
+          shard_lengths=[2, 3, 2, 3],  # 10
+          num_bytes=0,
+          filename_template=_filename_template('train'),
+      ),
+      splits_lib.SplitInfo(
+          name='test',
+          shard_lengths=[8],
+          num_bytes=0,
+          filename_template=_filename_template('test'),
+      ),
+      splits_lib.SplitInfo(
+          name='validation',
+          shard_lengths=[8],
+          filename_template=_filename_template('validation'),
+          num_bytes=0,
+      ),
+  ])
 
   # Test to split multiple splits
   split = subsplits_utils.even_splits('train', 3, drop_remainder=True)[0]
