@@ -16,12 +16,12 @@
 """Util import."""
 
 # pylint: disable=wildcard-import
-from tensorflow_datasets.core.naming import DatasetName
 from tensorflow_datasets.core.units import Size
 from tensorflow_datasets.core.utils import docs
 from tensorflow_datasets.core.utils import tree_utils as tree
 from tensorflow_datasets.core.utils.gcs_utils import gcs_path
 from tensorflow_datasets.core.utils.image_utils import *
+from tensorflow_datasets.core.utils.lazy_imports_utils import lazy_imports
 from tensorflow_datasets.core.utils.py_utils import *
 from tensorflow_datasets.core.utils.resource_utils import *
 from tensorflow_datasets.core.utils.tf_utils import *
@@ -29,4 +29,5 @@ from tensorflow_datasets.core.utils.tqdm_utils import *
 from tensorflow_datasets.core.utils.type_utils import *
 from tensorflow_datasets.core.utils.version import Experiment
 from tensorflow_datasets.core.utils.version import Version
+
 # pylint: enable=wildcard-import
