@@ -20,12 +20,10 @@ import os
 import posixpath
 from typing import List, Optional
 
-import tensorflow as tf
-
-from tensorflow_datasets.core.utils import generic_path
+from etils import epath
 from tensorflow_datasets.core.utils import py_utils
 from tensorflow_datasets.core.utils import tqdm_utils
-from tensorflow_datasets.core.utils import type_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 GCS_ROOT_DIR = 'gs://tfds-data'
 
@@ -35,21 +33,24 @@ GCS_DATASETS_DIR = 'datasets'
 
 _is_gcs_disabled = False
 
+
 # Exception raised when GCS isn't available
 # * UnimplementedError: On windows, gs:// isn't supported on old TF versions.
 #   https://github.com/tensorflow/tensorflow/issues/38477
 # * FailedPreconditionError: (e.g. no internet)
 # * PermissionDeniedError: Some environments block GCS access.
 # * AbortedError: All 10 retry attempts failed.
-GCS_UNAVAILABLE_EXCEPTIONS = (
-    tf.errors.UnimplementedError,
-    tf.errors.FailedPreconditionError,
-    tf.errors.PermissionDeniedError,
-    tf.errors.AbortedError,
-)
+def gcs_unavailable_exceptions():
+  return (
+      OSError,
+      tf.errors.UnimplementedError,
+      tf.errors.FailedPreconditionError,
+      tf.errors.PermissionDeniedError,
+      tf.errors.AbortedError,
+  )
 
 
-def gcs_path(*relative_path: type_utils.PathLike) -> type_utils.ReadWritePath:
+def gcs_path(*relative_path: epath.PathLike) -> epath.Path:
   """Returns the GCS URI path.
 
   Args:
@@ -58,7 +59,7 @@ def gcs_path(*relative_path: type_utils.PathLike) -> type_utils.ReadWritePath:
   Returns:
     path: The GCS uri.
   """
-  return generic_path.as_path(GCS_ROOT_DIR).joinpath(*relative_path)
+  return epath.Path(GCS_ROOT_DIR).joinpath(*relative_path)
 
 
 # Community datasets index.
@@ -66,14 +67,14 @@ def gcs_path(*relative_path: type_utils.PathLike) -> type_utils.ReadWritePath:
 # location.
 # Datasets there are downloaded and installed locally by the
 # `PackageRegister` during `tfds.builder`
-GCS_COMMUNITY_INDEX_PATH = gcs_path() / 'community-datasets-list.jsonl'
+GCS_COMMUNITY_INDEX_PATH = gcs_path('community-datasets-list.jsonl')
 
 
-def exists(path: type_utils.ReadWritePath) -> bool:
+def exists(path: epath.Path) -> bool:
   """Checks if path exists. Returns False if issues occur connecting to GCS."""
   try:
     return path.exists()
-  except GCS_UNAVAILABLE_EXCEPTIONS:  # pylint: disable=catching-non-exception
+  except gcs_unavailable_exceptions():  # pylint: disable=catching-non-exception
     return False
 
 
@@ -86,48 +87,62 @@ def gcs_listdir(dir_name: str) -> Optional[List[str]]:
   return [posixpath.join(dir_name, f.name) for f in root_dir.iterdir()]
 
 
-def gcs_dataset_info_files(dataset_dir: str) -> Optional[List[str]]:
-  """Return paths to GCS files in the given dataset directory."""
-  return gcs_listdir(posixpath.join(GCS_DATASET_INFO_DIR, dataset_dir))
+def gcs_dataset_info_files(dataset_name: str) -> Optional[List[epath.Path]]:
+  """Return paths to the dataset info files of the given dataset in gs://tfds-data.
+  """
+  path = gcs_path(posixpath.join(GCS_DATASET_INFO_DIR, dataset_name))
+  if _is_gcs_disabled or not exists(path):
+    return None
+  return list(path.iterdir())
 
 
 def is_dataset_on_gcs(dataset_name: str) -> bool:
   """If the dataset is available on the GCS bucket gs://tfds-data/datasets."""
-  dir_name = posixpath.join(GCS_DATASETS_DIR, dataset_name)
-  return not _is_gcs_disabled and exists(gcs_path(dir_name))
+  path = gcs_path(posixpath.join(GCS_DATASETS_DIR, dataset_name))
+  return not _is_gcs_disabled and exists(path)
 
 
-def download_gcs_dataset(dataset_name,
-                         local_dataset_dir,
-                         max_simultaneous_downloads=25):
-  """Downloads prepared GCS dataset to local dataset directory."""
+def download_gcs_folder(
+    gcs_folder: epath.Path,
+    local_folder: epath.PathLike,
+    max_simultaneous_downloads: int = 25,
+) -> None:
+  """Downloads prepared GCS folder to local folder."""
   if _is_gcs_disabled:
     raise AssertionError('Cannot download from GCS when _is_gcs_disabled')
 
-  prefix = posixpath.join(GCS_DATASETS_DIR, dataset_name)
-  gcs_paths_to_dl = gcs_listdir(prefix)
-
   # Filter out the diffs folder if present
-  filter_prefix = posixpath.join(prefix, 'diffs')
-  gcs_paths_to_dl = [
-      p for p in gcs_paths_to_dl if not p.startswith(filter_prefix)
-  ]
+  paths_to_dl = [p for p in gcs_folder.iterdir() if p.name != 'diffs']
 
   with tqdm_utils.async_tqdm(
-      total=len(gcs_paths_to_dl), desc='Dl Completed...', unit=' file') as pbar:
+      total=len(paths_to_dl), desc='Dl Completed...', unit=' file') as pbar:
 
-    def _copy_from_gcs(gcs_path_):
+    def _copy(gcs_path_: epath.Path):
       # Copy 'gs://tfds-data/datasets/ds/1.0.0/file' -> `local_dir/file`
       tf.io.gfile.copy(
-          os.fspath(gcs_path(gcs_path_)),
-          os.path.join(local_dataset_dir, posixpath.basename(gcs_path_)),
+          os.fspath(gcs_path_),
+          os.path.join(local_folder, gcs_path_.name),
       )
       pbar.update(1)
 
     with concurrent.futures.ThreadPoolExecutor(
         max_workers=max_simultaneous_downloads) as executor:
-      futures = [
-          executor.submit(_copy_from_gcs, path) for path in gcs_paths_to_dl
-      ]
+      futures = [executor.submit(_copy, path) for path in paths_to_dl]
       for future in concurrent.futures.as_completed(futures):
         future.result()
+
+
+def download_gcs_dataset(dataset_name: epath.PathLike,
+                         local_dataset_dir: epath.PathLike,
+                         max_simultaneous_downloads: int = 25,
+                         root_dir: Optional[str] = None):
+  """Downloads prepared GCS dataset to local dataset directory."""
+  if root_dir:
+    gcs_folder = epath.Path(root_dir) / dataset_name
+  else:
+    gcs_folder = epath.Path(GCS_ROOT_DIR) / GCS_DATASETS_DIR / dataset_name
+
+  download_gcs_folder(
+      gcs_folder=gcs_folder,
+      local_folder=local_dataset_dir,
+      max_simultaneous_downloads=max_simultaneous_downloads)
