@@ -24,6 +24,7 @@ import logging
 import operator
 import os
 import random
+import re
 import shutil
 import string
 import sys
@@ -33,10 +34,12 @@ import typing
 from typing import Any, Callable, Dict, Iterable, Iterator, List, NoReturn, Optional, Tuple, Type, TypeVar, Union
 import uuid
 
+from absl import logging as absl_logging
+from etils import epath
 from six.moves import urllib
-import tensorflow as tf
 from tensorflow_datasets.core import constants
 from tensorflow_datasets.core.utils import type_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 Tree = type_utils.Tree
 
@@ -48,6 +51,7 @@ Tree = type_utils.Tree
 memoize = functools.lru_cache
 
 T = TypeVar('T')
+U = TypeVar('U')
 
 Fn = TypeVar('Fn', bound=Callable[..., Any])
 
@@ -66,11 +70,24 @@ def is_notebook() -> bool:
     return True
 
 
+# TODO(tfds): Should likely have a `logging_utils` wrapper around `absl.logging`
+# so logging messages are displayed on Colab.
+
+
+def print_notebook(*args: Any) -> None:
+  """Like `print`/`logging.info`. Colab do not print stderr by default."""
+  msg = ' '.join([str(x) for x in args])
+  if is_notebook():
+    print(msg)
+  else:
+    absl_logging.info(msg)
+
+
 def warning(text: str) -> None:
   if is_notebook():
     print(text)
   else:
-    logging.warning(text)
+    absl_logging.warning(text)
 
 
 @contextlib.contextmanager
@@ -103,7 +120,7 @@ def disable_logging():
     logger.disabled = logger_disabled
 
 
-class NonMutableDict(dict):
+class NonMutableDict(Dict[T, U]):
   """Dict where keys can only be added but not modified.
 
   Raises an error if a key is overwritten. The error message can be customized
@@ -120,12 +137,12 @@ class NonMutableDict(dict):
     super(NonMutableDict, self).__init__(*args, **kwargs)
 
   def __setitem__(self, key, value):
-    if key in self:
+    if key in self.keys():
       raise ValueError(self._error_msg.format(key=key))
     return super(NonMutableDict, self).__setitem__(key, value)
 
   def update(self, other):
-    if any(k in self for k in other):
+    if any(k in self.keys() for k in other):
       raise ValueError(self._error_msg.format(key=set(self) & set(other)))
     return super(NonMutableDict, self).update(other)
 
@@ -155,7 +172,7 @@ class memoized_property(property):  # pylint: disable=invalid-name
 
 
 if typing.TYPE_CHECKING:
-  # TODO(b/171883689): There is likelly better way to annotate descriptors
+  # TODO(b/171883689): There is likely a better way to annotate descriptors
 
   def classproperty(fn: Callable[[Type[Any]], T]) -> T:  # pylint: disable=function-redefined
     return fn(type(None))
@@ -295,11 +312,11 @@ def _get_incomplete_path(filename):
   """Returns a temporary filename based on filename."""
   random_suffix = ''.join(
       random.choice(string.ascii_uppercase + string.digits) for _ in range(6))
-  return filename + '.incomplete' + random_suffix
+  return filename + constants.INCOMPLETE_SUFFIX + random_suffix
 
 
 @contextlib.contextmanager
-def incomplete_dir(dirname: type_utils.PathLike) -> Iterator[str]:
+def incomplete_dir(dirname: epath.PathLike) -> Iterator[str]:
   """Create temporary dir for dirname and rename on exit."""
   dirname = os.fspath(dirname)
   tmp_dir = _get_incomplete_path(dirname)
@@ -313,10 +330,9 @@ def incomplete_dir(dirname: type_utils.PathLike) -> Iterator[str]:
 
 
 @contextlib.contextmanager
-def incomplete_file(
-    path: type_utils.ReadWritePath,) -> Iterator[type_utils.ReadWritePath]:
+def incomplete_file(path: epath.Path,) -> Iterator[epath.Path]:
   """Writes to path atomically, by writing to temp file and renaming it."""
-  tmp_path = path.parent / f'{path.name}.incomplete.{uuid.uuid4().hex}'
+  tmp_path = path.parent / f'{path.name}{constants.INCOMPLETE_SUFFIX}.{uuid.uuid4().hex}'
   try:
     yield tmp_path
     tmp_path.replace(path)
@@ -325,6 +341,14 @@ def incomplete_file(
     tmp_path.unlink(missing_ok=True)
 
 
+def is_incomplete_file(path: epath.Path) -> bool:
+  """Returns whether the given filename suggests that it's incomplete."""
+  return bool(
+      re.search(
+          rf'^.+{re.escape(constants.INCOMPLETE_SUFFIX)}\.[0-9a-fA-F]{{32}}$',
+          path.name))
+
+
 @contextlib.contextmanager
 def atomic_write(path, mode):
   """Writes to path atomically, by writing to temp file and renaming it."""
@@ -476,7 +500,7 @@ def basename_from_url(url: str) -> str:
   return filename or 'unknown_name'
 
 
-def list_info_files(dir_path: type_utils.PathLike) -> List[str]:
+def list_info_files(dir_path: epath.PathLike) -> List[str]:
   """Returns name of info files within dir_path."""
   from tensorflow_datasets.core import file_adapters  # pylint: disable=g-import-not-at-top  # pytype: disable=import-error
   path = os.fspath(dir_path)
@@ -499,7 +523,7 @@ def get_base64(write_fn: Union[bytes, Callable[[io.BytesIO], None]],) -> str:
 
 
 @contextlib.contextmanager
-def add_sys_path(path: type_utils.PathLike) -> Iterator[None]:
+def add_sys_path(path: epath.PathLike) -> Iterator[None]:
   """Temporary add given path to `sys.path`."""
   path = os.fspath(path)
   try:
