@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,22 +15,24 @@
 
 """This module contains the reader config."""
 
-from typing import Callable, Optional, Sequence, Union
+from __future__ import annotations
 
 import dataclasses
+import enum
+from typing import Callable, Optional, Sequence, Union
 
-import tensorflow.compat.v2 as tf
 from tensorflow_datasets.core.utils import shard_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 InterleaveSortFn = Callable[[Sequence[shard_utils.FileInstruction]],
                             Sequence[shard_utils.FileInstruction],]
 
 
-class _MISSING:
+class _MISSING(str):
   pass
 
 
-MISSING = _MISSING()
+MISSING = _MISSING('missing')
 
 
 @dataclasses.dataclass(eq=False)
@@ -39,8 +41,8 @@ class ReadConfig:
 
   Attributes:
     options: `tf.data.Options()`, dataset options to use. Note that when
-      `shuffle_files` is True and no seed is defined, experimental_deterministic
-      will be set to False internally, unless it is defined here.
+      `shuffle_files` is True and no seed is defined, deterministic will be set
+      to False internally, unless it is defined here.
     try_autocache: If True (default) and the dataset satisfy the right
       conditions (dataset small enough, files not shuffled,...) the dataset will
       be cached during the first iteration (through `ds = ds.cache()`).
@@ -78,9 +80,14 @@ class ReadConfig:
       interleaving files. By default using tf.data's AUTOTUNE.
     enable_ordering_guard: When True (default), an exception is raised if
       shuffling or interleaving are used on an ordered dataset.
+    assert_cardinality: When True (default), an exception is raised if at the
+      end of an Epoch the number of read examples does not match the expected
+      number from dataset metadata. A power user would typically want to set
+      False if input files have been tempered with and they don't mind missing
+      records or have too many of them.
   """
   # General tf.data.Dataset parametters
-  options: tf.data.Options = dataclasses.field(default_factory=tf.data.Options)
+  options: Optional[tf.data.Options] = None
   try_autocache: bool = True
   add_tfds_id: bool = False
   # tf.data.Dataset.shuffle parameters
@@ -95,7 +102,14 @@ class ReadConfig:
   input_context: Optional[tf.distribute.InputContext] = None
   experimental_interleave_sort_fn: Optional[InterleaveSortFn] = None
   skip_prefetch: bool = False
-  num_parallel_calls_for_decode: Optional[int] = tf.data.experimental.AUTOTUNE
-  num_parallel_calls_for_interleave_files: Optional[int] = (
-      tf.data.experimental.AUTOTUNE)
+  num_parallel_calls_for_decode: Optional[int] = None
+  num_parallel_calls_for_interleave_files: Optional[int] = None
   enable_ordering_guard: bool = True
+  assert_cardinality: bool = True
+
+  def __post_init__(self):
+    self.options = self.options or tf.data.Options()
+    if self.num_parallel_calls_for_decode is None:
+      self.num_parallel_calls_for_decode = tf.data.experimental.AUTOTUNE
+    if self.num_parallel_calls_for_interleave_files is None:
+      self.num_parallel_calls_for_interleave_files = tf.data.experimental.AUTOTUNE
