@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,14 +15,17 @@
 
 """TensorFlow utils."""
 
+from __future__ import annotations
+
 import collections
 import contextlib
-from typing import Union
+from typing import Any, Union
 
 import numpy as np
-import tensorflow.compat.v2 as tf
 from tensorflow_datasets.core.utils import py_utils
+from tensorflow_datasets.core.utils import tree_utils
 from tensorflow_datasets.core.utils import type_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 # Struct containing a graph for the TFGraphRunner
 GraphRun = collections.namedtuple('GraphRun',
@@ -56,7 +59,6 @@ class TFGraphRunner(object):
   Usage:
     graph_runner = TFGraphRunner()
     output = graph_runner.run(tf.sigmoid, np.ones(shape=(5,)))
-
   """
 
   __slots__ = ['_graph_run_cache']
@@ -126,17 +128,22 @@ class TFGraphRunner(object):
       graph_run.session.close()
 
 
-def is_dtype(value):
-  """Return True is the given value is a TensorFlow dtype."""
-  try:
-    tf.as_dtype(value)
-  except TypeError:
-    return False
-  return True
+def convert_to_shape(shape: Any) -> type_utils.Shape:
+  """Converts a shape to a TFDS shape."""
+  if isinstance(shape, tuple):
+    return shape
+  if isinstance(shape, tf.TensorShape):
+    return tuple(shape.as_list())
+  if isinstance(shape, list):
+    return tuple(shape)
+  raise ValueError(
+      f'Shape of type {type(shape)} with content {shape} is not supported!')
 
 
-def assert_shape_match(shape1, shape2):
-  """Ensure the shape1 match the pattern given by shape2.
+@py_utils.memoize(maxsize=1000)
+def assert_shape_match(shape1: type_utils.Shape,
+                       shape2: type_utils.Shape) -> None:
+  """Ensure the shape1 matches the pattern given by shape2.
 
   Ex:
     assert_shape_match((64, 64, 3), (None, None, 3))
@@ -145,8 +152,11 @@ def assert_shape_match(shape1, shape2):
     shape1 (tuple): Static shape
     shape2 (tuple): Dynamic shape (can contain None)
   """
-  shape1 = tf.TensorShape(shape1)
-  shape2 = tf.TensorShape(shape2)
+  assert_tf_shape_match(tf.TensorShape(shape1), tf.TensorShape(shape2))
+
+
+def assert_tf_shape_match(shape1: tf.TensorShape,
+                          shape2: tf.TensorShape) -> None:
   if shape1.ndims is None or shape2.ndims is None:
     raise ValueError('Shapes must have known rank. Got %s and %s.' %
                      (shape1.ndims, shape2.ndims))
@@ -159,11 +169,11 @@ def shapes_are_compatible(
     shapes1: type_utils.TreeDict[type_utils.Shape],
 ) -> bool:
   """Returns True if all shapes are compatible."""
-  # Use `py_utils.map_nested` instead of `tf.nest.map_structure` as shapes are
-  # tuple/list.
+  # Use `py_utils.map_nested` instead of `tree_utils.map_structure` as shapes
+  # are tuple/list.
   shapes0 = py_utils.map_nested(tf.TensorShape, shapes0, dict_only=True)
   shapes1 = py_utils.map_nested(tf.TensorShape, shapes1, dict_only=True)
-  all_values = tf.nest.map_structure(
+  all_values = tree_utils.map_structure(
       lambda s0, s1: s0.is_compatible_with(s1),
       shapes0,
       shapes1,
@@ -202,7 +212,7 @@ def merge_shape(tf_shape: tf.Tensor, np_shape: type_utils.Shape):
   Returns:
     A tuple like np_shape, but with `None` values replaced by `tf.Tensor` values
   """
-  assert_shape_match(tf_shape.shape, (len(np_shape),))
+  assert_tf_shape_match(tf_shape.shape, tf.TensorShape((len(np_shape),)))
   return tuple(
       tf_shape[i] if dim is None else dim for i, dim in enumerate(np_shape))
 
