@@ -19,14 +19,18 @@ import concurrent.futures
 import functools
 from typing import Callable, TypeVar
 
-import tensorflow as tf
 from tensorflow_datasets.core.utils import tqdm_utils as tqdm
 from tensorflow_datasets.core.utils import type_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
+import tree
 
 Tree = type_utils.Tree
 _Tin = TypeVar('_Tin')
 _Tout = TypeVar('_Tout')
 
+# Wrapper around `tree.map_structure` to easily switch backend if needed.
+map_structure = tree.map_structure
+
 
 def parallel_map(
     map_fn: Callable[..., _Tout],  # Callable[[_Tin0, _Tin1,...], Tout]
@@ -34,7 +38,7 @@ def parallel_map(
     max_workers: int = 32,
     report_progress: bool = False,
 ) -> Tree[_Tout]:  # pytype: disable=invalid-annotation
-  """Same as `tf.nest.map_structure` but apply map_fn in parallel.
+  """Same as `map_structure` but apply map_fn in parallel.
 
   Args:
     map_fn: Worker function
@@ -48,7 +52,7 @@ def parallel_map(
   with concurrent.futures.ThreadPoolExecutor(
       max_workers=max_workers,) as executor:
     launch_worker = functools.partial(executor.submit, map_fn)
-    futures = tf.nest.map_structure(launch_worker, *trees)
+    futures = map_structure(launch_worker, *trees)
 
     leaves = tf.nest.flatten(futures)
 
@@ -60,4 +64,4 @@ def parallel_map(
       if f.exception():
         raise f.exception()
 
-  return tf.nest.map_structure(lambda f: f.result(), futures)
+  return map_structure(lambda f: f.result(), futures)
