@@ -15,12 +15,13 @@
 
 """Graph visualizer."""
 
-from typing import Callable, Any, Optional, Dict, Union
+from __future__ import annotations
 
-import tensorflow as tf
+from typing import Any, Callable, Dict, Optional, Union
 
 from tensorflow_datasets.core import dataset_info
 from tensorflow_datasets.core import lazy_imports_lib
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 from tensorflow_datasets.core.visualization import visualizer
 
 _GraphFn = Callable[[Any], Any]
