@@ -15,15 +15,16 @@
 
 """Image visualizer."""
 
+from __future__ import annotations
+
 from typing import Optional
 
 from absl import logging
-import tensorflow as tf
-
 from tensorflow_datasets.core import dataset_info
 from tensorflow_datasets.core import dataset_utils
 from tensorflow_datasets.core import features as features_lib
 from tensorflow_datasets.core import lazy_imports_lib
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 from tensorflow_datasets.core.visualization import visualizer
 
 
