@@ -18,7 +18,7 @@
 import re
 
 from absl import logging
-import tensorflow as tf
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 _CITATION = """\
@@ -34,8 +34,8 @@ edition = {Proceedings of 14th ACM Conference on Computer and Communications Sec
 }
 """
 
-_URL = ("https://download.microsoft.com/download/3/E/1/3E1C3F21-"
-        "ECDB-4869-8368-6DEBA77B919F/kagglecatsanddogs_3367a.zip")
+_URL = ("https://download.microsoft.com/download/3/E/1/"
+        "3E1C3F21-ECDB-4869-8368-6DEBA77B919F/kagglecatsanddogs_5340.zip")
 _NUM_CORRUPT_IMAGES = 1738
 _DESCRIPTION = (("A large set of images of cats and dogs. "
                  "There are %d corrupted images that are dropped.") %
