@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@ import re
 
 from absl import logging
 import numpy as np
-import tensorflow.compat.v2 as tf
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 _DESCRIPTION = """\
@@ -188,7 +188,7 @@ class CuratedBreastImagingDDSM(tfds.core.GeneratorBasedBuilder):
         'patient': tfds.features.Text(),
         'abnormalities': {
             'id':
-                tfds.features.Tensor(shape=(), dtype=tf.int32),
+                tfds.features.Tensor(shape=(), dtype=np.int32),
             'mask':
                 tfds.features.Image(shape=(None, None, 1)),
             'assessment':
