@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +18,9 @@
 import os
 import re
 
-import tensorflow.compat.v2 as tf
+from etils import epath
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
+from tensorflow_datasets.datasets.imagenet2012 import imagenet_common
 import tensorflow_datasets.public_api as tfds
 
 _CITATION = """\
@@ -52,7 +54,6 @@ _CLASSES = [
     "airplane", "bear", "bicycle", "bird", "boat", "bottle", "car", "cat",
     "chair", "clock", "dog", "elephant", "keyboard", "knife", "oven", "truck"
 ]
-_IMAGENET_LABELS_FNAME = "image_classification/imagenet2012_labels.txt"
 
 
 class GeirhosConflictStimuli(tfds.core.GeneratorBasedBuilder):
@@ -63,7 +64,7 @@ class GeirhosConflictStimuli(tfds.core.GeneratorBasedBuilder):
   def _info(self):
     """Define dataset info."""
 
-    imagenet_names_file = tfds.core.tfds_path(_IMAGENET_LABELS_FNAME)
+    imagenet_names_file = imagenet_common.label_names_file()
     return tfds.core.DatasetInfo(
         builder=self,
         description=(_DESCRIPTION),
@@ -112,7 +113,7 @@ class GeirhosConflictStimuli(tfds.core.GeneratorBasedBuilder):
     # Read ImageNet mapping.
     imagenet_names = set(self.info.features["shape_imagenet_labels"].names)
 
-    with tf.io.gfile.GFile(imagenet_mapping_path) as f:
+    with epath.Path(imagenet_mapping_path).open() as f:
       mapping_txt = f.read()
     mapping = {}
     for match in re.finditer(r"([a-z]+)\s*=\s*\[([^\]]+)\]", mapping_txt):
