@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,15 +15,14 @@
 
 """Test import."""
 
-import tensorflow_datasets as tfds  # pylint: disable=unused-import
-from tensorflow_datasets import testing
+import tensorflow_datasets as tfds
 
 
-class ImportTest(testing.TestCase):
+class ImportTest(tfds.testing.TestCase):
 
   def test_import(self):
     pass
 
 
 if __name__ == '__main__':
-  testing.test_main()
+  tfds.testing.test_main()
