@@ -15,12 +15,16 @@
 
 """TriviaQA: A Reading Comprehension Dataset."""
 
+from __future__ import annotations
+
 import json
 import os
 
 from absl import logging
+from etils import epath
+import numpy as np
 import six
-import tensorflow as tf
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 _CITATION = """
@@ -138,7 +142,7 @@ class TriviaQA(tfds.core.GeneratorBasedBuilder):
                 tfds.features.Sequence({
                     "description": tfds.features.Text(),
                     "filename": tfds.features.Text(),
-                    "rank": tf.int32,
+                    "rank": np.int32,
                     "title": tfds.features.Text(),
                     "url": tfds.features.Text(),
                     "search_context": tfds.features.Text(),
@@ -311,7 +315,7 @@ class TriviaQA(tfds.core.GeneratorBasedBuilder):
       logging.info("generating examples from = %s", filepath)
       fname = os.path.basename(filepath)
 
-      with tf.io.gfile.GFile(filepath) as f:
+      with epath.Path(filepath).open() as f:
         current_record = ""
         for line in f:
           if line == "        {\n":
