@@ -15,10 +15,13 @@
 
 """Builder for DMLab Datasets."""
 
+from __future__ import annotations
+
 import dataclasses
 from typing import Any, Dict
 
-import tensorflow as tf
+import numpy as np
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 from tensorflow_datasets.rl_unplugged import rlu_common
 
@@ -63,6 +66,10 @@ _CITATION = """
 }
 """
 
+_PIXELS_HEIGHT = 72
+_PIXELS_WIDTH = 96
+_PIXELS_SHAPE = (_PIXELS_HEIGHT, _PIXELS_WIDTH, 3)
+
 
 @dataclasses.dataclass
 class BuilderConfig(tfds.core.BuilderConfig):
@@ -89,29 +96,25 @@ class DMLabDatasetBuilder(rlu_common.RLUBuilder, skip_registration=True):
                 'observation': {
                     'pixels':
                         tfds.features.Image(
-                            shape=(
-                                72,
-                                96,
-                                3,
-                            ),
-                            dtype=tf.uint8,
+                            shape=_PIXELS_SHAPE,
+                            dtype=np.uint8,
                             encoding_format='png'),
                     'last_action':
-                        tf.int64,
+                        np.int64,
                     'last_reward':
-                        tf.float32,
+                        np.float32,
                 },
-                'action': tf.int64,
-                'reward': tf.float32,
-                'is_terminal': tf.bool,
-                'is_first': tf.bool,
-                'is_last': tf.bool,
-                'discount': tf.float32,
+                'action': np.int64,
+                'reward': np.float32,
+                'is_terminal': np.bool_,
+                'is_first': np.bool_,
+                'is_last': np.bool_,
+                'discount': np.float32,
             }),
         'episode_id':
-            tf.int64,
+            np.int64,
         'episode_return':
-            tf.float32,
+            np.float32,
     })
 
   def get_description(self):
@@ -153,6 +156,16 @@ class DMLabDatasetBuilder(rlu_common.RLUBuilder, skip_registration=True):
 
     data = tf.io.parse_single_example(tf_example, feature_description)
 
+    # Dmlab observations are compressed with OpenCV imencode function. Encoding
+    # with imencode and decoding with TF PNG decoder flips the order of the
+    # color channels. Here we flip back the order color of channels from BGR to
+    # RGB:
+    pixels = tf.scan(
+        fn=lambda _, png: tf.reshape(tf.io.decode_png(png), _PIXELS_SHAPE),
+        elems=data['observations_pixels'],
+        initializer=tf.zeros(_PIXELS_SHAPE, dtype=tf.uint8))
+    pixels = tf.reverse(pixels, axis=[-1])
+
     episode = {
         # Episode Metadata
         'episode_id': data['episode_id'],
@@ -160,7 +173,7 @@ class DMLabDatasetBuilder(rlu_common.RLUBuilder, skip_registration=True):
         'steps': {
             'observation': {
                 'pixels':
-                    data['observations_pixels'],
+                    pixels,
                 'last_action':
                     tf.argmax(
                         data['observations_action'],
