@@ -15,10 +15,13 @@
 
 """Module including definitions for all RLU datasets."""
 
-import os
+from __future__ import annotations
 
+import os
 from typing import Any, Dict, Generator, List, Tuple
-import tensorflow as tf
+
+import numpy as np
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 
@@ -29,6 +32,9 @@ use, we provide the datasets with a unified API which makes it easy for the
 practitioner to work with all data in the suite once a general pipeline has been
 established.
 
+The datasets follow the [RLDS format](https://github.com/google-research/rlds)
+to represent steps and episodes.
+
 """
 
 
@@ -37,13 +43,18 @@ _HOMEPAGE = 'https://github.com/deepmind/deepmind-research/tree/master/rl_unplug
 
 def filename(prefix: str, num_shards: int, shard_id: int):
   return os.fspath(
-      tfds.core.as_path(f'{prefix}-{shard_id:05d}-of-{num_shards:05d}'))
+      tfds.core.Path(f'{prefix}-{shard_id:05d}-of-{num_shards:05d}'))
 
 
 def get_files(prefix: str, num_shards: int) -> List[str]:
   return [filename(prefix, num_shards, i) for i in range(num_shards)]  # pytype: disable=bad-return-type  # gen-stub-imports
 
 
+def float_tensor_feature(size: int) -> tfds.features.Tensor:
+  return tfds.features.Tensor(
+      shape=(size,), dtype=np.float32, encoding=tfds.features.Encoding.ZLIB)
+
+
 class RLUBuilder(tfds.core.GeneratorBasedBuilder, skip_registration=True):
   """DatasetBuilder for RLU."""
 
