@@ -22,6 +22,7 @@ import subprocess
 import textwrap
 from typing import Optional
 
+from tensorflow_datasets.core import dataset_metadata
 from tensorflow_datasets.core import naming
 from tensorflow_datasets.scripts.cli import builder_templates
 from tensorflow_datasets.scripts.cli import cli_utils as utils
@@ -72,13 +73,8 @@ def create_dataset_files(dataset_name: str,
                          data_format: Optional[str] = None) -> None:
   """Creates the dataset files."""
   # Creates the root directory
-  dataset_dir = dataset_dir.expanduser() / dataset_name
+  dataset_dir = dataset_dir.expanduser().resolve() / dataset_name
   dataset_dir.mkdir(parents=True)
-  # TODO(py3.7): Should be `dir.expanduser().resolve()` but `.resolve()` fails
-  # on some environments when the file doesn't exists.
-  # https://stackoverflow.com/questions/55710900/pathlib-resolve-method-not-resolving-non-existant-files
-  dataset_dir = dataset_dir.resolve()
-
   in_tfds = 'tensorflow_datasets' in dataset_dir.parts
 
   info = utils.DatasetInfo(
@@ -89,6 +85,9 @@ def create_dataset_files(dataset_name: str,
 
   _create_dataset_file(info)
   _create_dataset_test(info)
+  _create_dataset_readme(info)
+  _create_dataset_citations(info)
+  _create_dataset_tags(info)
   _create_init(info)
   _create_dummy_data(info)
   _create_checksum(info)
@@ -144,6 +143,36 @@ def _create_dataset_test(info: utils.DatasetInfo) -> None:
   file_path.write_text(content)
 
 
+def _create_dataset_readme(info: utils.DatasetInfo) -> None:
+  """Adds the `README.md` file."""
+  file_path = info.path / 'README.md'
+  content = textwrap.dedent(f"""\
+      {info.todo}: Markdown description of that will appear on the catalog page.
+      Description is **formatted** as markdown.
+
+      It should also contain any processing which has been applied (if any),
+      (e.g. corrupted example skipped, images cropped,...):
+      """)
+  file_path.write_text(content)
+
+
+def _create_dataset_citations(info: utils.DatasetInfo) -> None:
+  """Adds the `CITATIONS.bib` file."""
+  file_path = info.path / 'CITATIONS.bib'
+  content = textwrap.dedent(f"""\
+      // {info.todo}: BibTeX citation
+      """)
+  file_path.write_text(content)
+
+
+def _create_dataset_tags(info: utils.DatasetInfo) -> None:
+  """Adds the `TAGS.txt` file."""
+  file_path = info.path / 'TAGS.txt'
+  content = '// {info.todo}: remove tags which do not apply to dataset.\n' + (
+      dataset_metadata.valid_tags_with_comments())
+  file_path.write_text(content)
+
+
 def _create_init(info: utils.DatasetInfo) -> None:
   """Adds the `__init__.py` file."""
   file_path = info.path / '__init__.py'
