@@ -16,7 +16,6 @@
 """Dataset catalog documentation template.
 
 Displayed in https://www.tensorflow.org/datasets/catalog/.
-
 """
 
 import abc
@@ -24,7 +23,7 @@ import html
 import os
 import re
 import textwrap
-from typing import List, Optional, Union
+from typing import List, Mapping, Optional, Union
 
 import tensorflow_datasets as tfds
 from tensorflow_datasets.core.features import feature as feature_lib
@@ -49,7 +48,6 @@ class Block(str):
 
   <block>
   ```
-
   """
   pass
 
@@ -59,7 +57,6 @@ class IntentedBlock(Block):
 
   By default, blocks are stripped to correctly remove extra end-line & cie.
   Intented block will keep the returned formatting.
-
   """
 
 
@@ -407,7 +404,8 @@ class FeatureDocumentationSection(Section):
     def format_row(doc: feature_lib.CatalogFeatureDocumentation) -> str:
       if doc.tensor_info:
         shape = str(doc.tensor_info.shape) if doc.tensor_info.shape else ''
-        dtype = repr(doc.tensor_info.dtype) if doc.tensor_info.dtype else ''
+        dtype = feature_lib.dtype_to_str(
+            doc.tensor_info.np_dtype) if doc.tensor_info.dtype else ''
       else:
         shape = ''
         dtype = ''
@@ -510,6 +508,35 @@ class KnowYourDataSection(Section):
       return _SKIP_SECTION
 
 
+class PapersWithCodeSection(Section):
+
+  NAME = 'Additional Documentation'
+
+  def __init__(self, tfds_to_pwc_links: Optional[Mapping[str, str]] = None):
+    super().__init__()
+    if tfds_to_pwc_links:
+      self._catalog_urls = tfds_to_pwc_links
+    else:
+      self._catalog_urls = doc_utils.get_pwc_catalog_urls()
+
+  def get_key(self, builder: tfds.core.DatasetBuilder):
+    return None  # Single url for all configs
+
+  def content(self, builder: tfds.core.DatasetBuilder):
+    url = self._catalog_urls.get(builder.name)
+    if url:
+      return f"""
+        <a class="button button-with-icon" href="{url}">
+          Explore on Papers With Code
+          <span class="material-icons icon-after" aria-hidden="true">
+            north_east
+          </span>
+        </a>
+      """
+    else:
+      return _SKIP_SECTION
+
+
 class DatasetVisualizationSection(Section):
 
   NAME = 'Figure'
@@ -750,6 +777,7 @@ def get_markdown_string(
   all_sections = [
       KnowYourDataSection(),
       DatasetDescriptionSection(),
+      PapersWithCodeSection(),
       ConfigDescriptionSection(),
       HomepageSection(),
       SourceCodeSection(),
