@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,11 +24,14 @@ from absl import logging
 
 from tensorflow_datasets.scripts.cli import main as main_cli
 
-module_import = flags.DEFINE_string('module_import', None, '`--imports` flag')
+module_import = flags.DEFINE_string('module_import', None, '`--imports` flag.')
+dataset = flags.DEFINE_string('dataset', None, 'singleton `--datasets` flag.')
+
 builder_config_id = flags.DEFINE_integer('builder_config_id', None,
                                          '`--config_idx` flag')
 
 
+
 def _parse_flags(argv: List[str]) -> argparse.Namespace:
   """Command lines flag parsing."""
   return main_cli._parse_flags([argv[0], 'build'] + argv[1:])  # pylint: disable=protected-access
@@ -43,6 +46,8 @@ def main(args: argparse.Namespace) -> None:
         '***`tfds build` should be used instead of `download_and_prepare`.***')
   if module_import.value:
     args.imports = module_import.value
+  if dataset.value:
+    args.datasets = [dataset.value]
   if builder_config_id.value is not None:
     args.config_idx = builder_config_id.value
   main_cli.main(args)
