@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,8 @@
 
 """German Credit (numeric) dataset."""
 
-import tensorflow.compat.v2 as tf
+from etils import epath
+import numpy as np
 import tensorflow_datasets.public_api as tfds
 
 URL = "https://archive.ics.uci.edu/ml/machine-learning-databases/statlog/german/german.data-numeric"
@@ -48,7 +49,7 @@ class GermanCreditNumeric(tfds.core.GeneratorBasedBuilder):
         builder=self,
         description=_DESCRIPTION,
         features=tfds.features.FeaturesDict({
-            "features": tfds.features.Tensor(shape=(24,), dtype=tf.int32),
+            "features": tfds.features.Tensor(shape=(24,), dtype=np.int32),
             "label": tfds.features.ClassLabel(names=["Bad", "Good"]),
         }),
         supervised_keys=("features", "label"),
@@ -59,7 +60,7 @@ class GermanCreditNumeric(tfds.core.GeneratorBasedBuilder):
   def _split_generators(self, dl_manager):
     """Returns SplitGenerators."""
     data_file = dl_manager.download(URL)
-    with tf.io.gfile.GFile(data_file) as f:
+    with epath.Path(data_file).open() as f:
       all_lines = f.read().split("\n")
     records = [l for l in all_lines if l]  # get rid of empty lines
 
