@@ -18,8 +18,9 @@
 import hashlib
 import os
 from typing import Dict, List
+
 from absl import logging
-import tensorflow as tf
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 _DESCRIPTION = """\
