@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,11 +15,14 @@
 
 """Summarizing abstract from covid19 publications."""
 
+from __future__ import annotations
+
 import json
 import os
 from typing import Any, Dict, Iterator, List, Optional, Text, Tuple
 
-import tensorflow.compat.v2 as tf
+import numpy as np
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 _CITATION = """
@@ -56,6 +59,12 @@ _ADDITIONAL_FEATURES = [
     _SHA, "source_x", "title", "doi", "license", "authors", "publish_time",
     "journal", "url"
 ]
+_ALL_COLUMNS = [
+    "cord_uid", "sha", "source_x", "title", "doi", "pmcid", "pubmed_id",
+    "license", "abstract", "publish_time", "authors", "journal",
+    "Microsoft Academic Paper ID", "WHO #Covidence", "has_full_text",
+    "full_text_file", "url"
+]
 
 
 class Covid19sum(tfds.core.GeneratorBasedBuilder):
@@ -73,8 +82,8 @@ class Covid19sum(tfds.core.GeneratorBasedBuilder):
     features = {k: tf.string for k in _ADDITIONAL_FEATURES + [_ABSTRACT]}
     features[_BODY_TEXT] = tfds.features.Sequence(
         tfds.features.FeaturesDict({
-            _SECTION: tf.string,
-            _TEXT: tf.string
+            _SECTION: np.str_,
+            _TEXT: np.str_
         }))
     return tfds.core.DatasetInfo(
         builder=self,
@@ -92,7 +101,10 @@ class Covid19sum(tfds.core.GeneratorBasedBuilder):
     extracted_path = dl_manager.extract(
         os.path.join(dl_manager.manual_dir, "CORD-19-research-challenge.zip"))
     pd = tfds.core.lazy_imports.pandas
-    df = pd.read_csv(os.path.join(extracted_path, "metadata.csv")).fillna("")
+    df = pd.read_csv(
+        os.path.join(extracted_path, "metadata.csv"),
+        header=0,
+        names=_ALL_COLUMNS).fillna("")
     data_paths = []
     for _, row in df.iterrows():
       file_dir = row["full_text_file"]
