@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,8 @@ import json
 import os
 
 from absl import logging
-import tensorflow.compat.v2 as tf
+from etils import epath
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 _CITATION = """
@@ -128,7 +129,7 @@ class Xsum(tfds.core.GeneratorBasedBuilder):
     for i in split_ids:
       filename = os.path.join(path, i + ".data")
       if tf.io.gfile.exists(filename):
-        with tf.io.gfile.GFile(filename) as f:
+        with epath.Path(filename).open() as f:
           text = "".join([
               line for line in f.readlines()
               if line not in _REMOVE_LINES and line.strip()
