@@ -30,16 +30,16 @@ from absl import logging
 from absl.testing import parameterized
 from etils import epath
 import numpy as np
-import tensorflow as tf
-
 from tensorflow_datasets.core import dataset_builder
 from tensorflow_datasets.core import dataset_info
+from tensorflow_datasets.core import dataset_metadata
 from tensorflow_datasets.core import dataset_utils
 from tensorflow_datasets.core import download
 from tensorflow_datasets.core import load
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core import visibility
 from tensorflow_datasets.core.download import checksums
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 from tensorflow_datasets.testing import feature_test_case
 from tensorflow_datasets.testing import test_utils
 
@@ -265,6 +265,13 @@ class DatasetBuilderTestCase(parameterized.TestCase,
     self.assertIsInstance(info, dataset_info.DatasetInfo)
     self.assertEqual(self.builder.name, info.name)
 
+  def test_tags_are_valid(self):
+    metadata = self.builder.get_metadata()
+    tags = set(metadata.tags)
+    valid_tags = set(dataset_metadata.valid_tags())
+    invalid_tags = tags - valid_tags
+    self.assertEmpty(invalid_tags, "There are invalid tags.")
+
   def _add_url(self, url_or_urls):
     if self._stop_record_download:
       # Stop record the checksums if dl_manager.download_checksums has been
