@@ -25,13 +25,13 @@ from unittest import mock
 
 from absl import logging
 import numpy as np
-import tensorflow as tf
-
 from tensorflow_datasets.core import dataset_builder
 from tensorflow_datasets.core import decode
 from tensorflow_datasets.core import features as features_lib
 from tensorflow_datasets.core import read_only_builder
-from tensorflow_datasets.core import tfrecords_reader
+from tensorflow_datasets.core import reader as reader_lib
+from tensorflow_datasets.core.utils import dtype_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 from tensorflow_datasets.testing import test_utils
 
 
@@ -180,7 +180,6 @@ def mock_data(
 
   def mock_as_dataset(self, split, decoders=None, read_config=None, **kwargs):
     """Function which overwrite `builder._as_dataset`."""
-    del split
     del kwargs
 
     # Partial decoding
@@ -224,8 +223,8 @@ def mock_data(
     ds = ds.map(decode_fn, num_parallel_calls=tf.data.experimental.AUTOTUNE)
 
     if read_config and read_config.add_tfds_id:
-      ds_id = tfrecords_reader._make_id_dataset(  # pylint: disable=protected-access
-          filename=f'{self.name}-split.tfrecord-00000-of-00001',
+      ds_id = reader_lib._make_id_dataset(  # pylint: disable=protected-access
+          filename=f'{self.name}-{split}.tfrecord-00000-of-00001',
           start_index=0,  # pytype: disable=wrong-arg-types
       )
       ds = tf.data.Dataset.zip((ds, ds_id))
@@ -351,15 +350,15 @@ class RandomFakeGenerator(object):
       max_value = 255
 
     # We cast the data to make sure `encode_example` don't raise errors
-    dtype = tensor_info.dtype
+    dtype = tensor_info.np_dtype
     # Generate some random values, depending on the dtype
-    if dtype.is_integer:
-      return self._rgn.randint(0, max_value, shape).astype(dtype.as_numpy_dtype)
-    elif dtype.is_floating:
-      return self._rgn.random_sample(shape).astype(dtype.as_numpy_dtype)
-    elif dtype.is_bool:
-      return (self._rgn.random_sample(shape) < .5).astype(dtype.as_numpy_dtype)
-    elif dtype == tf.string:
+    if dtype_utils.is_integer(dtype):
+      return self._rgn.randint(0, max_value, shape).astype(dtype)
+    elif dtype_utils.is_floating(dtype):
+      return self._rgn.random_sample(shape).astype(dtype)
+    elif dtype_utils.is_bool(dtype):
+      return (self._rgn.random_sample(shape) < .5).astype(dtype)
+    elif dtype_utils.is_string(dtype):
       return self._generate_random_string_array(shape)
     raise ValueError('Fake generation not supported for {}'.format(dtype))
 
