@@ -15,6 +15,8 @@
 
 """Test utilities."""
 
+from __future__ import annotations
+
 import contextlib
 import dataclasses
 import functools
@@ -28,8 +30,6 @@ from unittest import mock
 from etils import epath
 from etils import epy
 import numpy as np
-import tensorflow as tf
-
 from tensorflow_datasets.core import dataset_builder
 from tensorflow_datasets.core import dataset_collection_builder
 from tensorflow_datasets.core import dataset_info
@@ -38,6 +38,7 @@ from tensorflow_datasets.core import features
 from tensorflow_datasets.core import lazy_imports_lib
 from tensorflow_datasets.core import naming
 from tensorflow_datasets.core import utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 
 @contextlib.contextmanager
@@ -586,6 +587,8 @@ def _assert_feature_equal(feature0, feature1):
   assert repr(feature0) == repr(feature1)
   assert feature0.shape == feature1.shape
   assert feature0.dtype == feature1.dtype
+  assert feature0.np_dtype == feature1.np_dtype
+  assert feature0.tf_dtype == feature1.tf_dtype
   if isinstance(feature0, features.FeaturesDict):
     _assert_features_equal(dict(feature0), dict(feature1))
   if isinstance(feature0, features.Sequence):
