@@ -22,10 +22,12 @@ from tensorflow_datasets.text.beir import Beir
 from tensorflow_datasets.text.blimp import Blimp
 from tensorflow_datasets.text.bool_q import BoolQ
 from tensorflow_datasets.text.c4 import C4
+from tensorflow_datasets.text.c4_wsrs import C4WSRS
 from tensorflow_datasets.text.cfq import CFQ
 from tensorflow_datasets.text.cfq import CFQConfig
 from tensorflow_datasets.text.civil_comments import CivilComments
 from tensorflow_datasets.text.clinc_oos import ClincOOS
+from tensorflow_datasets.text.conll2002 import Conll2002
 from tensorflow_datasets.text.conll2003 import Conll2003
 from tensorflow_datasets.text.cos_e import CosE
 from tensorflow_datasets.text.definite_pronoun_resolution import DefinitePronounResolution
@@ -58,6 +60,7 @@ from tensorflow_datasets.text.paws_x_wiki import PawsXWiki
 from tensorflow_datasets.text.pg19 import Pg19
 from tensorflow_datasets.text.piqa import PIQA
 from tensorflow_datasets.text.qa4mre import Qa4mre
+from tensorflow_datasets.text.qrecc import QReCC
 from tensorflow_datasets.text.quac import Quac
 from tensorflow_datasets.text.quality import Quality
 from tensorflow_datasets.text.race import Race
