@@ -23,7 +23,9 @@ from typing import Optional, Sequence
 import uuid
 
 from absl import logging
-import tensorflow as tf
+from etils import epath
+from tensorflow_datasets.core.utils import tree_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 from tensorflow_datasets.text import c4_utils
 
@@ -263,7 +265,7 @@ class C4Config(tfds.core.BuilderConfig):
         **kwargs)
 
     if clean and tuple(languages) != ("en",):
-      logging.warn(
+      logging.warning(
           "C4 cleaning is only expected to work reliably for English pages.")
 
     if not 0.0 <= badwords_filter_fraction <= 1.0:
@@ -383,7 +385,7 @@ class C4(tfds.core.BeamBasedBuilder):
                 _OPENWEBTEXT_URLS_ZIP))
       file_paths["openwebtext_urls_zip"] = dl_manager.extract(owt_path)
 
-    file_paths = tf.nest.map_structure(os.fspath, file_paths)
+    file_paths = tree_utils.map_structure(os.fspath, file_paths)
 
     pages_pcollection = self._get_pages_pcollection(pipeline, file_paths,
                                                     dl_manager)
@@ -497,9 +499,8 @@ class C4(tfds.core.BeamBasedBuilder):
     pages = (
         pages
         | "normalize_url" >> beam.Map(lambda p: (p.normalized_url, p))
-        | "group_url" >> beam.GroupByKey()
-        | beam.Values()
-        | beam.Map(c4_utils.select_newest_page))
+        | "select_newest" >> beam.CombinePerKey(c4_utils.select_newest_page)
+        | beam.Values())
 
     # Optionally filter for WebText-like URLs.
     # Output: [PageFeatures]
@@ -545,7 +546,7 @@ class C4(tfds.core.BeamBasedBuilder):
       badwords = collections.defaultdict(set)
       for lang, path in file_paths["badwords"].items():
         lang = lang.split("-")[0]  # remove suffix if present
-        with tf.io.gfile.GFile(path) as f:
+        with epath.Path(path).open() as f:
           badwords[lang].update(l.strip() for l in f)
 
       for lang, allowlist in _BADWORDS_ALLOWLIST.items():
