@@ -15,12 +15,15 @@
 
 """CivilComments from Jigsaw Unintended Bias Kaggle Competition."""
 
+from __future__ import annotations
+
 import ast
 import csv
 import os
 
+from etils import epath
 import numpy as np
-import tensorflow as tf
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 # General (main) citation for CivilComments and CivilCommentsIdentities.
@@ -137,6 +140,9 @@ in addition to the basic seven labels. However, it only includes the subset
 """
 
 _CC_COVERT_DESCRIPTION = """
+WARNING: there's a potential data quality issue with CivilCommentsCovert that
+we're actively working on fixing (06/28/22); the underlying data may change!
+
 The CivilCommentsCovert set is a subset of CivilCommentsIdentities with ~20% of
 the train and test splits further annotated for covert offensiveness, in
 addition to the toxicity and identity labels. Raters were asked to categorize
@@ -154,7 +160,7 @@ returned in a 'spans' feature.
 """
 
 _CC_CONTEXT_DESCRIPTION = """
-The CivilComments Toxic Spans are a subset of CivilComments that was
+The CivilComments in Context is a subset of CivilComments that was
 labeled by making available to the labelers the parent_text. It includes
 a contextual_toxicity feature.
 """
@@ -271,8 +277,11 @@ class CivilComments(tfds.core.GeneratorBasedBuilder):
           mode='context'),
   ]
 
-  VERSION = tfds.core.Version('1.2.1')
+  VERSION = tfds.core.Version('1.2.3')
   RELEASE_NOTES = {
+      '1.2.3': 'Add warning to CivilCommentsCovert as we fix a data issue.',
+      '1.2.2': 'Update to reflect context only having a train split.',
+      '1.2.1': 'Fix incorrect formatting in context splits.',
       '1.2.0': 'Add toxic spans, context, and parent comment text features.',
       '1.1.3': 'Corrected id types from float to string.',
       '1.1.2': 'Added separate citation for CivilCommentsCovert dataset.',
@@ -299,7 +308,7 @@ class CivilComments(tfds.core.GeneratorBasedBuilder):
         'article_id': tf.int32,
     }
     if mode == 'spans':
-      features['spans'] = tfds.features.Tensor(shape=(None,), dtype=tf.int32)
+      features['spans'] = tfds.features.Tensor(shape=(None,), dtype=np.int32)
       supervised_value = 'spans'
     else:
       for label in _labels(mode):
@@ -330,19 +339,11 @@ class CivilComments(tfds.core.GeneratorBasedBuilder):
                 'mode': mode,
                 'split': 'train',
             },
-        ),
-        tfds.core.SplitGenerator(
-            name=tfds.Split.TEST,
-            gen_kwargs={
-                'filename': filename,
-                'mode': mode,
-                'split': 'test_private',
-            },
-        ),
+        )
     ]
 
-    if mode != 'covert':
-      # Covert does not have validation split.
+    if mode != 'covert' and mode != 'context':
+      # Covert and context do not have validation split.
       validation_split_generator = tfds.core.SplitGenerator(
           name=tfds.Split.VALIDATION,
           gen_kwargs={
@@ -351,8 +352,19 @@ class CivilComments(tfds.core.GeneratorBasedBuilder):
               'split': 'test_public',
           },
       )
-      splits.insert(1, validation_split_generator)
+      splits.append(validation_split_generator)
 
+    if mode != 'context':
+      # Context has only a train split.
+      test_split_generator = tfds.core.SplitGenerator(
+          name=tfds.Split.TEST,
+          gen_kwargs={
+              'filename': filename,
+              'mode': mode,
+              'split': 'test_private',
+          },
+      )
+      splits.append(test_split_generator)
     return splits
 
   def _generate_examples(self, filename, mode, split):
@@ -370,7 +382,7 @@ class CivilComments(tfds.core.GeneratorBasedBuilder):
     Yields:
       A dictionary of features, depending upon the mode.
     """
-    with tf.io.gfile.GFile(filename) as f:
+    with epath.Path(filename).open() as f:
       reader = csv.DictReader(f)
       for row in reader:
         if mode == 'spans':
