@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,10 +15,13 @@
 
 """clinc_oos dataset."""
 
+from __future__ import annotations
+
 import csv
 import os
 
-import tensorflow as tf
+from etils import epath
+import numpy as np
 import tensorflow_datasets.public_api as tfds
 
 _DOWNLOAD_URL = 'https://github.com/jereliu/datasets/raw/master/clinc_oos.zip'
@@ -78,8 +81,8 @@ class ClincOOS(tfds.core.GeneratorBasedBuilder):
         description=_DESCRIPTION,
         features=tfds.features.FeaturesDict({
             'text': tfds.features.Text(),
-            'intent': tf.int32,
-            'domain': tf.int32,
+            'intent': np.int32,
+            'domain': np.int32,
             'intent_name': tfds.features.Text(),
             'domain_name': tfds.features.Text()
         }),
@@ -120,7 +123,7 @@ class ClincOOS(tfds.core.GeneratorBasedBuilder):
 
   def _generate_examples(self, filename):
     """Yields examples."""
-    with tf.io.gfile.GFile(filename) as f:
+    with epath.Path(filename).open() as f:
       reader = csv.DictReader(f)
       for row_id, row in enumerate(reader):
         example = {}
