@@ -15,9 +15,13 @@
 
 """GAP is a gender-balanced text data set."""
 
+from __future__ import annotations
+
 import csv
 
-import tensorflow as tf
+from etils import epath
+import numpy as np
+from tensorflow_datasets.core.utils import bool_utils
 import tensorflow_datasets.public_api as tfds
 
 _CITATION = """
@@ -58,7 +62,11 @@ class Gap(tfds.core.GeneratorBasedBuilder):
   of (ambiguous pronoun, antecedent name), sampled from Wikipedia.
   """
 
-  VERSION = tfds.core.Version('0.1.0')
+  VERSION = tfds.core.Version('0.1.1')
+  RELEASE_NOTES = {
+      '0.1.1': 'Fixes parsing of boolean field `A-coref` and `B-coref`.',
+      '0.1.0': 'Initial release.',
+  }
 
   def _info(self):
     return tfds.core.DatasetInfo(
@@ -68,13 +76,13 @@ class Gap(tfds.core.GeneratorBasedBuilder):
             'ID': tfds.features.Text(),
             'Text': tfds.features.Text(),
             'Pronoun': tfds.features.Text(),
-            'Pronoun-offset': tf.int32,
+            'Pronoun-offset': np.int32,
             'A': tfds.features.Text(),
-            'A-offset': tf.int32,
-            'A-coref': tf.bool,
+            'A-offset': np.int32,
+            'A-coref': np.bool_,
             'B': tfds.features.Text(),
-            'B-offset': tf.int32,
-            'B-coref': tf.bool,
+            'B-offset': np.int32,
+            'B-coref': np.bool_,
             'URL': tfds.features.Text()
         }),
         supervised_keys=None,
@@ -106,7 +114,9 @@ class Gap(tfds.core.GeneratorBasedBuilder):
 
   def _generate_examples(self, filepath):
     """Yields examples."""
-    with tf.io.gfile.GFile(filepath) as tsvfile:
+    with epath.Path(filepath).open() as tsvfile:
       reader = csv.DictReader(tsvfile, dialect='excel-tab')
       for i, row in enumerate(reader):
+        row['A-coref'] = bool_utils.parse_bool(row['A-coref'])
+        row['B-coref'] = bool_utils.parse_bool(row['B-coref'])
         yield i, row
