@@ -15,10 +15,13 @@
 
 """HellaSwag dataset."""
 
+from __future__ import annotations
+
 import json
 import os
 
-import tensorflow as tf
+from etils import epath
+import numpy as np
 import tensorflow_datasets.public_api as tfds
 
 _CITATION = """
@@ -61,7 +64,7 @@ class Hellaswag(tfds.core.GeneratorBasedBuilder):
             'context': tfds.features.Text(),
             'endings': tfds.features.Sequence(tfds.features.Text()),
             'activity_label': tfds.features.Text(),
-            'label': tf.int32,
+            'label': np.int32,
             'split_type': tfds.features.Text(),
             'source_id': tfds.features.Text(),
         }),
@@ -110,7 +113,7 @@ class Hellaswag(tfds.core.GeneratorBasedBuilder):
 
   def _generate_examples(self, filepath, domain=None, source=None):
     """Yields examples."""
-    with tf.io.gfile.GFile(filepath) as f:
+    with epath.Path(filepath).open() as f:
       for idx, line in enumerate(f):
         elem = json.loads(line)
         elem_id = '%s_%d' % (os.path.basename(filepath), idx)
