@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,9 +15,11 @@
 
 """schema_guided_dialogue dataset."""
 
+from __future__ import annotations
+
 import json
 
-import tensorflow as tf
+import numpy as np
 import tensorflow_datasets.public_api as tfds
 
 _DESCRIPTION = """
@@ -68,7 +70,7 @@ class SchemaGuidedDialogue(tfds.core.GeneratorBasedBuilder):
             "metadata": {
                 "services":
                     tfds.features.Sequence({
-                        "name": tf.string,
+                        "name": np.str_,
                         # TODO(arunchaganty): include service definitions
                     })
             }
