@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,11 +15,15 @@
 
 """The SuperGLUE benchmark."""
 
+from __future__ import annotations
+
 import json
 import os
 
+from etils import epath
+import numpy as np
 import six
-import tensorflow.compat.v2 as tf
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 _SUPER_GLUE_CITATION = """\
@@ -441,14 +445,14 @@ class SuperGlue(tfds.core.GeneratorBasedBuilder):
       features["end2"] = tf.int32
     if self.builder_config.name == "multirc":
       features["idx"] = tfds.features.FeaturesDict({
-          "paragraph": tf.int32,
-          "question": tf.int32,
-          "answer": tf.int32,
+          "paragraph": np.int32,
+          "question": np.int32,
+          "answer": np.int32,
       })
     elif self.builder_config.name == "record":
       features["idx"] = tfds.features.FeaturesDict({
-          "passage": tf.int32,
-          "query": tf.int32,
+          "passage": np.int32,
+          "query": np.int32,
       })
     else:
       features["idx"] = tf.int32
@@ -507,7 +511,7 @@ class SuperGlue(tfds.core.GeneratorBasedBuilder):
     ]
 
   def _generate_examples(self, data_file, split):
-    with tf.io.gfile.GFile(data_file) as f:
+    with epath.Path(data_file).open() as f:
       for line in f:
         row = json.loads(line)
 
