@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 
 import os
 
-import tensorflow as tf
+from etils import epath
 import tensorflow_datasets.public_api as tfds
 
 _LICENSE = """WordNet Release 3.0 This software and database is being provided
@@ -118,7 +118,7 @@ _RELATIONS = [
 
 def _make_wn18_metadata(synset_definitions_path):
   synsets = {}
-  with tf.io.gfile.GFile(synset_definitions_path) as f:
+  with epath.Path(synset_definitions_path).open() as f:
     for line in f:
       synset_id, name, definition = line.strip().split('\t')
       synsets[synset_id] = dict(name=name, definition=definition)
@@ -175,7 +175,7 @@ class Wordnet(tfds.core.GeneratorBasedBuilder):
         homepage='https://wordnet.princeton.edu/',
         citation=self.builder_config.citation,
         metadata=tfds.core.MetadataDict(),
-        redistribution_info=dict(license=_LICENSE),
+        license=_LICENSE,
     )
 
   def _split_generators(self, dl_manager):
@@ -209,7 +209,7 @@ class Wordnet(tfds.core.GeneratorBasedBuilder):
 
   def _generate_examples(self, triplets_path):
     """Yields examples."""
-    with tf.io.gfile.GFile(triplets_path) as f:
+    with epath.Path(triplets_path).open() as f:
       for i, line in enumerate(f):
         lhs, relation, rhs = line.strip().split('\t')
         yield i, {'lhs': lhs, 'relation': relation, 'rhs': rhs}
