@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 
 import os
 
-import tensorflow.compat.v2 as tf
+from etils import epath
 import tensorflow_datasets.public_api as tfds
 
 _DESCRIPTION = """\
@@ -155,9 +155,9 @@ class TedHrlrTranslate(tfds.core.GeneratorBasedBuilder):
 
   def _generate_examples(self, source_file, target_file):
     """This function returns the examples in the raw (text) form."""
-    with tf.io.gfile.GFile(source_file) as f:
+    with epath.Path(source_file).open() as f:
       source_sentences = f.read().split("\n")
-    with tf.io.gfile.GFile(target_file) as f:
+    with epath.Path(target_file).open() as f:
       target_sentences = f.read().split("\n")
 
     assert len(target_sentences) == len(
