@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,9 +17,9 @@
 
 import csv
 import os
-import six
 
-import tensorflow.compat.v2 as tf
+from etils import epath
+import six
 import tensorflow_datasets.public_api as tfds
 
 _DESCRIPTION = """\
@@ -95,7 +95,7 @@ class TedMultiTranslate(tfds.core.GeneratorBasedBuilder):
 
   def _generate_examples(self, data_file):
     """This function returns the examples in the raw (text) form."""
-    with tf.io.gfile.GFile(data_file) as f:
+    with epath.Path(data_file).open() as f:
       reader = csv.DictReader(f, delimiter='\t', quoting=csv.QUOTE_NONE)
       for idx, row in enumerate(reader):
         # Everything in the row except for 'talk_name' will be a translation.
