@@ -24,8 +24,10 @@ import re
 import xml.etree.cElementTree as ElementTree
 
 from absl import logging
+from etils import epath
 import six
-import tensorflow as tf
+from tensorflow_datasets.core.utils import tree_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 _DESCRIPTION = """\
@@ -598,7 +600,7 @@ class WmtConfig(tfds.core.BuilderConfig):
       citation: The paper citation for the dataset.
       description: The description of the dataset.
       language_pair: pair of languages that will be used for translation. Should
-                 contain 2 letter coded strings. For example: ("en", "de").
+        contain 2 letter coded strings. For example: ("en", "de").
       subsets: Dict[split, list[str]]. List of the subset to use for each of the
         split. Note that WMT subclasses overwrite this parameter.
       **kwargs: keyword arguments forwarded to super.
@@ -688,6 +690,7 @@ class WmtTranslate(tfds.core.GeneratorBasedBuilder):
 
     manual_paths = {}
     urls_to_download = {}
+    downloaded_files = {}
     for ss_name in itertools.chain.from_iterable(self.subsets.values()):
       if ss_name == "czeng_17":
         # CzEng1.7 is CzEng1.6 with some blocks filtered out. We must download
@@ -698,15 +701,26 @@ class WmtTranslate(tfds.core.GeneratorBasedBuilder):
       if ds.get_manual_dl_files(source):
         manual_paths[ss_name] = _check_manual_files(ds)
       else:
-        urls_to_download[ss_name] = ds.get_url(source)
+        urls = ds.get_url(source)
+        # This domain throws a 503 if we attempt to download in parallel.
+        sequential_dl_urls = [url for url in urls if "www.statmt.org" in url]
+        parallel_dl_urls = [
+            url for url in urls if url not in sequential_dl_urls
+        ]
+        if sequential_dl_urls:
+          downloaded_files[ss_name] = [
+              dl_manager.download_and_extract(url) for url in sequential_dl_urls
+          ] + dl_manager.download_and_extract(parallel_dl_urls)
+        else:
+          urls_to_download[ss_name] = urls
 
     # Download and extract files from URLs.
-    downloaded_files = dl_manager.download_and_extract(urls_to_download)
+    downloaded_files.update(dl_manager.download_and_extract(urls_to_download))
     # Extract manually downloaded files.
     manual_files = dl_manager.extract(manual_paths)
 
-    manual_files = tf.nest.map_structure(os.fspath, manual_files)
-    downloaded_files = tf.nest.map_structure(os.fspath, downloaded_files)
+    manual_files = tree_utils.map_structure(os.fspath, manual_files)
+    downloaded_files = tree_utils.map_structure(os.fspath, downloaded_files)
 
     extraction_map = dict(downloaded_files, **manual_files)
 
@@ -800,7 +814,7 @@ def _parse_parallel_sentences(f1, f2):
       lang = "zh" if lang in ("ch", "cn") else lang
     else:
       lang = split_path[-1]
-    with tf.io.gfile.GFile(path) as f:
+    with epath.Path(path).open() as f:
       return f.read().split("\n"), lang
 
   def _parse_sgm(path):
@@ -810,7 +824,7 @@ def _parse_parallel_sentences(f1, f2):
     # Note: We can't use the XML parser since some of the files are badly
     # formatted.
     seg_re = re.compile(r"<seg id=\"\d+\">(.*)</seg>")
-    with tf.io.gfile.GFile(path) as f:
+    with epath.Path(path).open() as f:
       for line in f:
         seg_match = re.match(seg_re, line)
         if seg_match:
@@ -844,9 +858,9 @@ def _parse_parallel_sentences(f1, f2):
 
 
 def _parse_frde_bitext(fr_path, de_path):
-  with tf.io.gfile.GFile(fr_path) as f:
+  with epath.Path(fr_path).open() as f:
     fr_sentences = f.read().split("\n")
-  with tf.io.gfile.GFile(de_path) as f:
+  with epath.Path(de_path).open() as f:
     de_sentences = f.read().split("\n")
   assert len(fr_sentences) == len(de_sentences), (
       "Sizes do not match: %d vs %d for %s vs %s." %
@@ -892,7 +906,7 @@ def _parse_tsv(path, language_pair=None):
     l1, l2 = lang_match.groups()
   else:
     l1, l2 = language_pair
-  with tf.io.gfile.GFile(path) as f:
+  with epath.Path(path).open() as f:
     for j, line in enumerate(f):
       cols = line.split("\t")
       if len(cols) != 2:
@@ -908,7 +922,7 @@ def _parse_wikiheadlines(path):
   lang_match = re.match(r".*\.([a-z][a-z])-([a-z][a-z])$", path)
   assert lang_match is not None, "Invalid Wikiheadlines filename: %s" % path
   l1, l2 = lang_match.groups()
-  with tf.io.gfile.GFile(path) as f:
+  with epath.Path(path).open() as f:
     for line_id, line in enumerate(f):
       s1, s2 = line.split("|||")
       yield line_id, {l1: s1.strip(), l2: s2.strip()}
@@ -919,7 +933,7 @@ def _parse_czeng(*paths, **kwargs):
   filter_path = kwargs.get("filter_path", None)
   if filter_path:
     re_block = re.compile(r"^[^-]+-b(\d+)-\d\d[tde]")
-    with tf.io.gfile.GFile(filter_path) as f:
+    with epath.Path(filter_path).open() as f:
       bad_blocks = set(
           re.search(r"qw{([\s\d]*)}", f.read()).groups()[0].split())  # pytype: disable=attribute-error
     logging.info("Loaded %d bad blocks to filter from CzEng v1.6 to make v1.7.",
@@ -946,7 +960,7 @@ def _parse_czeng(*paths, **kwargs):
 
 
 def _parse_hindencorp(path):
-  with tf.io.gfile.GFile(path) as f:
+  with epath.Path(path).open() as f:
     for line_id, line in enumerate(f):
       split_line = line.split("\t")
       if len(split_line) != 5:
