@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,8 +15,10 @@
 
 """DAVIS 2017 dataset for video object segmentation."""
 
+from __future__ import annotations
+
 import numpy as np
-import tensorflow as tf
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 _DESCRIPTION = """
@@ -114,8 +116,8 @@ class Davis(tfds.core.GeneratorBasedBuilder):
                         ),
                 }),
             'metadata': {
-                'num_frames': tf.int64,
-                'video_name': tf.string,
+                'num_frames': np.int64,
+                'video_name': np.str_,
             },
         }),
         supervised_keys=None,
@@ -154,8 +156,10 @@ class Davis(tfds.core.GeneratorBasedBuilder):
       images = []
       annotations = []
       for i in range(seq_len):
-        image_path = images_path / f'{i:05d}.jpg'
-        annotation_path = annotations_path / f'{i:05d}.png'
+        image_fname = f'{i:05d}.jpg'
+        image_path = images_path / image_fname
+        annotation_fname = f'{i:05d}.png'
+        annotation_path = annotations_path / annotation_fname
         annotation = _read_annotation(annotation_path)
         images.append(image_path)
         annotations.append(annotation)
