@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 
 import collections
 
-import tensorflow.compat.v2 as tf
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 
 
 def _create_moving_sequence(image, pad_lefts, total_padding):
@@ -103,7 +103,7 @@ def _bounce_to_bbox(points):
   return tf.math.minimum(2 - points, points)
 
 
-def _get_random_unit_vector(ndims=2, dtype=tf.float32):
+def _get_random_unit_vector(ndims, dtype):
   x = tf.random.normal((ndims,), dtype=dtype)
   return x / tf.linalg.norm(x, axis=-1, keepdims=True)
 
@@ -125,7 +125,7 @@ def image_as_moving_sequence(image,
 
   Example usage:
   ```python
-  import tensorflow.compat.v2 as tf
+  from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
   import tensorflow_datasets as tfds
   from tensorflow_datasets.video import moving_sequence
   tf.enable_v2_behavior()
