@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,9 +15,11 @@
 
 """SCV dataset from http://arxiv.org/abs/1812.01717 ."""
 
-from absl import logging
-import tensorflow.compat.v2 as tf
+from __future__ import annotations
 
+import os
+
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 DATA_URL_DIR = "https://storage.googleapis.com/scv_dataset/data/"
@@ -126,7 +128,7 @@ class StarcraftVideo(tfds.core.GeneratorBasedBuilder):
         "rgb_screen":
             tfds.features.Video(
                 shape=(None, self.builder_config.resolution,
-                       self.builder_config.resolution, 3)),
+                       self.builder_config.resolution, 3)),  # pytype: disable=wrong-arg-types  # gen-stub-imports
     })
     return tfds.core.DatasetInfo(
         builder=self,
@@ -197,9 +199,8 @@ class StarcraftVideo(tfds.core.GeneratorBasedBuilder):
     return video_frames
 
   def _generate_examples(self, files):
-    logging.info("Reading data from %s.", ",".join(files))
     with tf.Graph().as_default():
-      ds = tf.data.TFRecordDataset(sorted(files))
+      ds = tf.data.TFRecordDataset(sorted(os.fspath(f) for f in files))
       ds = ds.map(
           self._parse_single_video,
           num_parallel_calls=tf.data.experimental.AUTOTUNE)
