@@ -15,11 +15,14 @@
 
 """Wikipedia-based Image Text (WIT) Dataset."""
 
+from __future__ import annotations
+
 import csv
 import os
 import sys
 
-import tensorflow as tf
+from etils import epath
+import numpy as np
 import tensorflow_datasets.public_api as tfds
 
 _DESCRIPTION = """
@@ -66,11 +69,11 @@ class Wit(tfds.core.GeneratorBasedBuilder):
             "caption_attribution_description": tfds.features.Text(),
             "caption_alt_text_description": tfds.features.Text(),
             "mime_type": tfds.features.Text(),
-            "original_height": tf.int32,
-            "original_width": tf.int32,
-            "is_main_image": tf.bool,
-            "attribution_passes_lang_id": tf.bool,
-            "page_changed_recently": tf.bool,
+            "original_height": np.int32,
+            "original_width": np.int32,
+            "is_main_image": np.bool_,
+            "attribution_passes_lang_id": np.bool_,
+            "page_changed_recently": np.bool_,
             "context_page_description": tfds.features.Text(),
             "context_section_description": tfds.features.Text(),
         }),
@@ -156,7 +159,7 @@ class Wit(tfds.core.GeneratorBasedBuilder):
     def _read_rows(filename):
       # Limit to 100 MB. Value must be smaller than the C long maximum value.
       csv.field_size_limit(sys.maxsize)
-      with tf.io.gfile.GFile(filename) as f:
+      with epath.Path(filename).open() as f:
         csv_reader = csv.DictReader(
             f, delimiter="\t", quoting=csv.QUOTE_MINIMAL)
         for i, row in enumerate(csv_reader):
