@@ -14,6 +14,8 @@
 # limitations under the License.
 
 """Wikipedia-based Image Text (WIT) Dataset for the Kaggle competition."""
+from __future__ import annotations
+
 import base64
 import csv
 import functools
@@ -22,10 +24,11 @@ import io
 import sys
 from typing import List, Optional
 
-import tensorflow as tf
+import numpy as np
+from tensorflow_datasets.core.utils import bool_utils
+from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
-
 _DESCRIPTION = """
 Wikipedia - Image/Caption Matching Kaggle Competition.
 
@@ -102,7 +105,7 @@ class WitKaggleConfig(tfds.core.BuilderConfig):
             ),  # Url to the image's commons page (if existent, else '').
         "embedding":
             tfds.features.Tensor(
-                shape=(resnet_embedding_shape,), dtype=tf.float32
+                shape=(resnet_embedding_shape,), dtype=np.float32
             ),  # A tensor of 2048 floats (if existent, else zeros).
     })
     self.features = tfds.features.FeaturesDict({
@@ -119,8 +122,10 @@ class WitKaggleConfig(tfds.core.BuilderConfig):
 class WitKaggle(tfds.core.GeneratorBasedBuilder):
   """DatasetBuilder for wit_kaggle dataset."""
 
-  VERSION = tfds.core.Version("1.0.1")
+  VERSION = tfds.core.Version("1.0.2")
   RELEASE_NOTES = {
+      "1.0.2":
+          "Fixes parsing of boolean fields.",
       "1.0.1":
           "Optimize Beam pipeline to avoid strugglers, ignoring rows without "
           "an image URL. Also added more Beam counters.",
@@ -175,11 +180,11 @@ class WitKaggle(tfds.core.GeneratorBasedBuilder):
               "caption_attribution_description": tfds.features.Text(),
               "caption_alt_text_description": tfds.features.Text(),
               "mime_type": tfds.features.Text(),
-              "original_height": tf.int32,
-              "original_width": tf.int32,
-              "is_main_image": tf.bool,
-              "attribution_passes_lang_id": tf.bool,
-              "page_changed_recently": tf.bool,
+              "original_height": np.int32,
+              "original_width": np.int32,
+              "is_main_image": np.bool_,
+              "attribution_passes_lang_id": np.bool_,
+              "page_changed_recently": np.bool_,
               "context_page_description": tfds.features.Text(),
               "context_section_description": tfds.features.Text(),
               "caption_title_and_reference_description": tfds.features.Text(),
@@ -280,6 +285,8 @@ class WitKaggle(tfds.core.GeneratorBasedBuilder):
       else:
         counter("normal_csv_files").inc()
         f = tf.io.gfile.GFile(filename, "r")
+      # Limit to 100 MB. Value must be smaller than the C long maximum value.
+      csv.field_size_limit(sys.maxsize)
       return csv.reader(f, delimiter="\t")
 
     def _read_pixel_rows(filename):
@@ -333,6 +340,10 @@ class WitKaggle(tfds.core.GeneratorBasedBuilder):
         sample = {"image_url": sample_url}
         for feature_key in self.builder_config.split_specific_features.keys():
           sample[feature_key] = sample_info[feature_key]
+          is_boolean_feature = self.builder_config.split_specific_features[
+              feature_key].np_dtype == np.bool_
+          if is_boolean_feature:
+            sample[feature_key] = bool_utils.parse_bool(sample[feature_key])
         # Test samples don't have gold captions.
         if "caption_title_and_reference_description" not in sample_info:
           sample["caption_title_and_reference_description"] = ""
